@@ -90,11 +90,12 @@ static void make_key()
 
 static print_members* getMembers(threadData_t *threadData)
 {
+  print_members *res;
   if (threadData && threadData->localRoots[LOCAL_ROOT_PRINT_MO]) {
     return threadData->localRoots[LOCAL_ROOT_PRINT_MO];
   }
   pthread_once(&printimpl_once_create_key,make_key);
-  print_members *res = (print_members*) pthread_getspecific(printimplKey);
+  res = (print_members*) pthread_getspecific(printimplKey);
   if (res != NULL) return res;
   res = (print_members*) calloc(1,sizeof(print_members));
   pthread_setspecific(printimplKey,res);
@@ -400,17 +401,20 @@ static int PrintImpl__writeBufConvertLines(threadData_t *threadData,const char *
 #endif
   char *modelicaFileName = NULL;
   char* strtmp = NULL;
-  str[nfilled] = '\0';
 
-  /* First, compile the regular expressions */
-  if (regcomp(&re_begin, re_str[0], REG_EXTENDED) || regcomp(&re_end, re_str[1], 0)) {
-    c_add_message(NULL,21, /* WRITING_FILE_ERROR */
-      ErrorType_scripting,
-      ErrorLevel_error,
-      gettext("Error compiling regular expression: %s or %s."),
-      re_str,
-      2);
-    return 1;
+  if (str!=NULL && nfilled!=0) {
+    str[nfilled] = '\0';
+
+    /* First, compile the regular expressions */
+    if (regcomp(&re_begin, re_str[0], REG_EXTENDED) || regcomp(&re_end, re_str[1], 0)) {
+      c_add_message(NULL,21, /* WRITING_FILE_ERROR */
+        ErrorType_scripting,
+        ErrorLevel_error,
+        gettext("Error compiling regular expression: %s or %s."),
+        re_str,
+        2);
+      return 1;
+    }
   }
 
   /* check if we have something to write */
@@ -431,8 +435,6 @@ static int PrintImpl__writeBufConvertLines(threadData_t *threadData,const char *
   }
   if (str == NULL || str[0]=='\0') {
     /* nothing to write to file, just close it and return ! */
-    regfree(&re_begin);
-    regfree(&re_end);
     fclose(file);
     return 1;
   }
