@@ -76,6 +76,7 @@
 #include "simulation/solver/solver_main.h"
 #include "simulation_info_json.h"
 #include "modelinfo.h"
+#include "simulation/solver/events.h"
 #include "simulation/solver/model_help.h"
 #include "simulation/solver/mixedSystem.h"
 #include "simulation/solver/linearSystem.h"
@@ -93,6 +94,7 @@ using namespace std;
 #ifndef NO_INTERACTIVE_DEPENDENCY
   Socket sim_communication_port;
   static int sim_communication_port_open = 0;
+  static int isXMLTCP=0;
 #endif
 
 extern "C" {
@@ -101,6 +103,9 @@ int sim_noemit = 0;           /* Flag for not emitting data */
 
 const std::string *init_method = NULL; /* method for  initialization. */
 
+static int callSolver(DATA* simData, threadData_t *threadData, string init_initMethod, string init_file,
+      double init_time, int lambda_steps, string outputVariablesAtEnd, int cpuTime, const char *argv_0);
+
 /*! \fn void setGlobalVerboseLevel(int argc, char**argv)
  *
  *  \brief determine verboselevel by investigating flag -lv flags
@@ -209,7 +214,7 @@ void setGlobalVerboseLevel(int argc, char**argv)
   delete flags;
 }
 
-int getNonlinearSolverMethod(int argc, char**argv)
+static int getNonlinearSolverMethod()
 {
   int i;
   const char *cflags = omc_flagValue[FLAG_NLS];
@@ -231,7 +236,7 @@ int getNonlinearSolverMethod(int argc, char**argv)
   return NLS_NONE;
 }
 
-int getlinearSolverMethod(int argc, char**argv)
+static int getlinearSolverMethod()
 {
   int i;
   const char *cflags = omc_flagValue[FLAG_LS];
@@ -253,7 +258,29 @@ int getlinearSolverMethod(int argc, char**argv)
   return LS_NONE;
 }
 
-int getNewtonStrategy(int argc, char**argv)
+static int getlinearSparseSolverMethod()
+{
+  int i;
+  const char *cflags = omc_flagValue[FLAG_LSS];
+  const string *method = cflags ? new string(cflags) : NULL;
+
+  if(!method)
+    return LSS_KLU; /* default method */
+
+  for(i=1; i<LSS_MAX; ++i)
+    if(*method == LSS_NAME[i])
+      return i;
+
+  warningStreamPrint(LOG_STDOUT, 1, "unrecognized option -lss=%s, current options are:", method->c_str());
+  for(i=1; i<LSS_MAX; ++i)
+    warningStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", LSS_NAME[i], LSS_DESC[i]);
+  messageClose(LOG_STDOUT);
+  throwStreamPrint(NULL,"see last warning");
+
+  return LSS_NONE;
+}
+
+static int getNewtonStrategy()
 {
   int i;
   const char *cflags = omc_flagValue[FLAG_NEWTON_STRATEGY];
@@ -275,6 +302,42 @@ int getNewtonStrategy(int argc, char**argv)
   return NEWTON_NONE;
 }
 
+static int getNlsLSSolver()
+{
+  int i;
+  const char *cflags = omc_flagValue[FLAG_NLS_LS];
+  const string *method = cflags ? new string(cflags) : NULL;
+
+  if(!method)
+    return NLS_LS_LAPACK; /* default method */
+
+  for(i=1; i<NLS_LS_MAX; ++i)
+    if(*method == NLS_LS_METHOD[i])
+      return i;
+
+  warningStreamPrint(LOG_STDOUT, 1, "unrecognized option -nls=%s, current options are:", method->c_str());
+  for(i=1; i<NLS_LS_MAX; ++i)
+    warningStreamPrint(LOG_STDOUT, 0, "%-18s [%s]", NLS_LS_METHOD[i], NLS_LS_METHOD_DESC[i]);
+  messageClose(LOG_STDOUT);
+  throwStreamPrint(NULL,"see last warning");
+
+  return NLS_LS_UNKNOWN;
+}
+
+static double getFlagReal(enum _FLAG flag, double res)
+{
+  const char *flagStr = omc_flagValue[flag];
+  char *endptr;
+  if (flagStr==NULL || *flagStr=='\0') {
+    return res;
+  }
+  res = strtod(flagStr, &endptr);
+  if (*endptr) {
+    throwStreamPrint(NULL, "Simulation flag %s expects a real number, got: %s", FLAG_NAME[flag], flagStr);
+  }
+  return res;
+}
+
 /**
  * Read the variable filter and mark variables that should not be part of the result file.
  * This phase is skipped for interactive simulations
@@ -446,6 +509,22 @@ int startNonInteractiveSimulation(int argc, char**argv, DATA* data, threadData_t
     infoStreamPrint(LOG_STDOUT, 0, "Linearization will performed at point of time: %f", data->simulationInfo->stopTime);
   }
 
+  /* set delta x for linearization */
+  if(omc_flag[FLAG_DELTA_X_LINEARIZE]) {
+    numericalDifferentiationDeltaXlinearize = atof(omc_flagValue[FLAG_DELTA_X_LINEARIZE]);
+    infoStreamPrint(LOG_SOLVER, 0, "Set delta x for numerical differentiation of the linearization to %f", numericalDifferentiationDeltaXlinearize);
+  }else{
+    numericalDifferentiationDeltaXlinearize = sqrt(DBL_EPSILON*2e1);
+  }
+
+  /* set delta x for integration methods dassl, ida */
+  if(omc_flag[FLAG_DELTA_X_SOLVER]) {
+    numericalDifferentiationDeltaXsolver = atof(omc_flagValue[FLAG_DELTA_X_SOLVER]);
+    infoStreamPrint(LOG_SOLVER, 0, "Set delta x for numerical differentiation of the integrator to %f", numericalDifferentiationDeltaXsolver);
+  }else{
+    numericalDifferentiationDeltaXsolver = sqrt(DBL_EPSILON);
+  }
+
   if(omc_flag[FLAG_S]) {
     if (omc_flagValue[FLAG_S]) {
       data->simulationInfo->solverMethod = GC_strdup(omc_flagValue[FLAG_S]);
@@ -486,11 +565,19 @@ int startNonInteractiveSimulation(int argc, char**argv, DATA* data, threadData_t
     init_lambda_steps_string = omc_flagValue[FLAG_ILS];
     init_lambda_steps = atoi(init_lambda_steps_string.c_str());
   }
+  if(omc_flag[FLAG_MAX_BISECTION_ITERATIONS]) {
+    maxBisectionIterations = atoi(omc_flagValue[FLAG_MAX_BISECTION_ITERATIONS]);
+    infoStreamPrint(LOG_STDOUT, 0, "Maximum number of bisection iterations changed to %d", maxBisectionIterations);
+  }
+  if(omc_flag[FLAG_MAX_EVENT_ITERATIONS]) {
+    maxEventIterations = atoi(omc_flagValue[FLAG_MAX_EVENT_ITERATIONS]);
+    infoStreamPrint(LOG_STDOUT, 0, "Maximum number of event iterations changed to %d", maxEventIterations);
+  }
   if(omc_flag[FLAG_OUTPUT]) {
     outputVariablesAtEnd = omc_flagValue[FLAG_OUTPUT];
   }
 
-  retVal = callSolver(data, threadData, init_initMethod, init_file, init_time, init_lambda_steps, outputVariablesAtEnd, cpuTime);
+  retVal = callSolver(data, threadData, init_initMethod, init_file, init_time, init_lambda_steps, outputVariablesAtEnd, cpuTime, argv[0]);
 
   if (omc_flag[FLAG_ALARM]) {
     alarm(0);
@@ -590,8 +677,8 @@ int initializeResultData(DATA* simData, threadData_t *threadData, int cpuTime)
  * "euler" calls an Euler solver
  * "rungekutta" calls a fourth-order Runge-Kutta Solver
  */
-int callSolver(DATA* simData, threadData_t *threadData, string init_initMethod, string init_file,
-      double init_time, int lambda_steps, string outputVariablesAtEnd, int cpuTime)
+static int callSolver(DATA* simData, threadData_t *threadData, string init_initMethod, string init_file,
+      double init_time, int lambda_steps, string outputVariablesAtEnd, int cpuTime, const char *argv_0)
 {
   TRACE_PUSH
   int retVal = -1;
@@ -605,6 +692,7 @@ int callSolver(DATA* simData, threadData_t *threadData, string init_initMethod,
     TRACE_POP
     return -1;
   }
+  simData->real_time_sync.scaling = getFlagReal(FLAG_RT, 0.0);
 
   if(std::string("") == simData->simulationInfo->solverMethod) {
 #if defined(WITH_DASSL)
@@ -644,7 +732,7 @@ int callSolver(DATA* simData, threadData_t *threadData, string init_initMethod,
                         simData->simulationInfo->numSteps, simData->simulationInfo->tolerance, 3);
     } else /* standard solver interface */
 #endif
-      retVal = solver_main(simData, threadData, init_initMethod.c_str(), init_file.c_str(), init_time, lambda_steps, solverID, outVars);
+      retVal = solver_main(simData, threadData, init_initMethod.c_str(), init_file.c_str(), init_time, lambda_steps, solverID, outVars, argv_0);
   }
 
   MMC_CATCH_INTERNAL(mmc_jumper)
@@ -738,10 +826,30 @@ int initRuntimeAndSimulation(int argc, char**argv, DATA *data, threadData_t *thr
     EXIT(1);
   }
 
-  data->simulationInfo->nlsMethod = getNonlinearSolverMethod(argc, argv);
-  data->simulationInfo->lsMethod = getlinearSolverMethod(argc, argv);
-  data->simulationInfo->newtonStrategy = getNewtonStrategy(argc, argv);
+  data->simulationInfo->nlsMethod = getNonlinearSolverMethod();
+  data->simulationInfo->lsMethod = getlinearSolverMethod();
+  data->simulationInfo->lssMethod = getlinearSparseSolverMethod();
+  data->simulationInfo->newtonStrategy = getNewtonStrategy();
   data->simulationInfo->nlsCsvInfomation = omc_flag[FLAG_NLS_INFO];
+  data->simulationInfo->nlsLinearSolver = getNlsLSSolver();
+
+  if(omc_flag[FLAG_LSS_MAX_DENSITY]) {
+    linearSparseSolverMaxDensity = atof(omc_flagValue[FLAG_LSS_MAX_DENSITY]);
+    infoStreamPrint(LOG_STDOUT, 0, "Maximum density for using linear sparse solver changed to %f", linearSparseSolverMaxDensity);
+  }
+  if(omc_flag[FLAG_LSS_MIN_SIZE]) {
+    linearSparseSolverMinSize = atoi(omc_flagValue[FLAG_LSS_MIN_SIZE]);
+    infoStreamPrint(LOG_STDOUT, 0, "Maximum system size for using linear sparse solver changed to %d", linearSparseSolverMinSize);
+  }
+  if(omc_flag[FLAG_NEWTON_XTOL]) {
+    newtonXTol = atof(omc_flagValue[FLAG_NEWTON_XTOL]);
+    infoStreamPrint(LOG_STDOUT, 0, "Tolerance for updating solution vector in Newton solver changed to %g", newtonXTol);
+  }
+
+  if(omc_flag[FLAG_NEWTON_FTOL]) {
+    newtonFTol = atof(omc_flagValue[FLAG_NEWTON_FTOL]);
+    infoStreamPrint(LOG_STDOUT, 0, "Tolerance for accepting accuracy in Newton solver changed to %g", newtonFTol);
+  }
 
   rt_tick(SIM_TIMER_INIT_XML);
   read_input_xml(data->modelData, data->simulationInfo);
@@ -754,11 +862,8 @@ int initRuntimeAndSimulation(int argc, char**argv, DATA *data, threadData_t *thr
 
   sim_noemit = omc_flag[FLAG_NOEMIT];
 
-  // ppriv - NO_INTERACTIVE_DEPENDENCY - for simpler debugging in Visual Studio
-
 #ifndef NO_INTERACTIVE_DEPENDENCY
-  if(omc_flag[FLAG_PORT])
-  {
+  if(omc_flag[FLAG_PORT]) {
     std::istringstream stream(omc_flagValue[FLAG_PORT]);
     int port;
     stream >> port;
@@ -766,12 +871,17 @@ int initRuntimeAndSimulation(int argc, char**argv, DATA *data, threadData_t *thr
     sim_communication_port_open &= sim_communication_port.create();
     sim_communication_port_open &= sim_communication_port.connect("127.0.0.1", port);
 
-    if(0 != strcmp("ia", data->simulationInfo->outputFormat))
-    {
-      communicateStatus("Starting", 0.0);
+    if(0 != strcmp("ia", data->simulationInfo->outputFormat)) {
+      communicateStatus("Starting", 0.0, data->simulationInfo->startTime, 0);
     }
   }
+
+  if (isXMLTCP && !sim_communication_port_open) {
+    errorStreamPrint(LOG_STDOUT, 0, "xmltcp log format requires a TCP-port to be passed (and successfully open)");
+    EXIT(1);
+  }
 #endif
+  // ppriv - NO_INTERACTIVE_DEPENDENCY - for simpler debugging in Visual Studio
 
   return 0;
 }
@@ -792,20 +902,6 @@ void SimulationRuntime_printStatus(int sig)
   printf("</status>\n");
 }
 
-void communicateStatus(const char *phase, double completionPercent /*0.0 to 1.0*/)
-{
-#ifndef NO_INTERACTIVE_DEPENDENCY
-  if(sim_communication_port_open)
-  {
-    std::stringstream s;
-    s << (int)(completionPercent*10000) << " " << phase << endl;
-    std::string str(s.str());
-    sim_communication_port.send(str);
-    // cout << str;
-  }
-#endif
-}
-
 void communicateMsg(char id, unsigned int size, const char *data)
 {
 #ifndef NO_INTERACTIVE_DEPENDENCY
@@ -866,4 +962,180 @@ int _main_SimulationRuntime(int argc, char**argv, DATA *data, threadData_t *thre
   return retVal;
 }
 
+#if !defined(OMC_MINIMAL_RUNTIME)
+const char* prettyPrintNanoSec(int64_t ns, int *v)
+{
+  if (ns > 100000000000L || ns < -100000000000L) {
+    *v = ns / 1000000000L;
+    return "s";
+  } if (ns > 100000000L || ns < -100000000L) {
+    *v = ns / 1000000L;
+    return "ms";
+  } else if (ns > 100000L || ns < -100000L) {
+    *v = ns / 1000L;
+    return "µs";
+  } else {
+    *v = ns;
+    return "ns";
+  }
+}
+#endif
+
+#ifndef NO_INTERACTIVE_DEPENDENCY
+static std::stringstream xmlTcpStream;
+static int numOpenTags=0;
+
+static void printEscapedXMLTCP(std::stringstream *s, const char *msg)
+{
+  while (*msg) {
+    if (*msg == '&') {
+      *s << "&amp;";
+    } else if (*msg == '<') {
+      *s << "&lt;";
+    } else if (*msg == '>') {
+      *s << "&gt;";
+    } else if (*msg == '"') {
+      *s << "&quot;";
+    } else {
+      *s << *msg;
+    }
+    msg++;
+  }
+}
+
+static inline void sendXMLTCPIfClosed()
+{
+  if (numOpenTags==0) {
+    sim_communication_port.send(xmlTcpStream.str());
+    xmlTcpStream.str("");
+  }
+}
+
+static void messageXMLTCP(int type, int stream, int indentNext, char *msg, int subline, const int *indexes)
+{
+  numOpenTags++;
+  xmlTcpStream << "<message stream=\"" << LOG_STREAM_NAME[stream] << "\" type=\"" << LOG_TYPE_DESC[type] << "\" text=\"";
+  printEscapedXMLTCP(&xmlTcpStream, msg);
+  if (indexes) {
+    int i;
+    xmlTcpStream << "\">\n";
+    for (i=1; i<=*indexes; i++) {
+      xmlTcpStream << "<used index=\"" << indexes[i] << "\" />\n";
+    }
+    if (!indentNext) {
+      numOpenTags--;
+      xmlTcpStream << "</message>\n";
+    }
+  } else {
+    if (indentNext) {
+      xmlTcpStream << "\">\n";
+    } else {
+      numOpenTags--;
+      xmlTcpStream << "\" />\n";
+    }
+  }
+  sendXMLTCPIfClosed();
+}
+
+static void messageCloseXMLTCP(int stream)
+{
+  if (ACTIVE_STREAM(stream)) {
+    numOpenTags--;
+    xmlTcpStream << "</message>\n";
+    sendXMLTCPIfClosed();
+  }
+}
+
+static void messageCloseXMLTCPWarning(int stream)
+{
+  if (ACTIVE_WARNING_STREAM(stream)) {
+    numOpenTags--;
+    xmlTcpStream << "</message>\n";
+    sendXMLTCPIfClosed();
+  }
+}
+#endif
+
+static void printEscapedXML(const char *msg)
+{
+  while (*msg) {
+    if (*msg == '&') fputs("&amp;", stdout);
+    else if (*msg == '<') fputs("&lt;", stdout);
+    else if (*msg == '>') fputs("&gt;", stdout);
+    else if (*msg == '"') fputs("&quot;", stdout);
+    else fputc(*msg, stdout);
+    msg++;
+  }
+}
+
+static void messageXML(int type, int stream, int indentNext, char *msg, int subline, const int *indexes)
+{
+  printf("<message stream=\"%s\" type=\"%s\" text=\"", LOG_STREAM_NAME[stream], LOG_TYPE_DESC[type]);
+  printEscapedXML(msg);
+  if (indexes) {
+    int i;
+    printf("\">\n");
+    for (i=1; i<=*indexes; i++) {
+      printf("<used index=\"%d\" />\n", indexes[i]);
+    }
+    if (!indentNext) {
+      fputs("</message>\n",stdout);
+    }
+  } else {
+    fputs(indentNext ? "\">\n" : "\" />\n", stdout);
+  }
+  fflush(stdout);
+}
+
+static void messageCloseXML(int stream)
+{
+  if (ACTIVE_STREAM(stream)) {
+    fputs("</message>\n", stdout);
+    fflush(stdout);
+  }
+}
+
+static void messageCloseXMLWarning(int stream)
+{
+  if (ACTIVE_WARNING_STREAM(stream)) {
+    fputs("</message>\n", stdout);
+    fflush(stdout);
+  }
+}
+
+void setStreamPrintXML(int isXML)
+{
+  if (isXML==1) {
+    messageFunction = messageXML;
+    messageClose = messageCloseXML;
+    messageCloseWarning = messageCloseXMLWarning;
+#ifndef NO_INTERACTIVE_DEPENDENCY
+  } else if (isXML==2) {
+    messageFunction = messageXMLTCP;
+    messageClose = messageCloseXMLTCP;
+    messageCloseWarning = messageCloseXMLTCPWarning;
+    isXMLTCP = 1;
+#endif
+  } else {
+    /* Already set... */
+  }
+}
+
+void communicateStatus(const char *phase, double completionPercent /*0.0 to 1.0*/, double currentTime, double currentStepSize)
+{
+#ifndef NO_INTERACTIVE_DEPENDENCY
+  if (sim_communication_port_open && isXMLTCP) {
+    std::stringstream s;
+    s << "<status phase=\"" << phase << "\" currentStepSize=\"" << currentStepSize << "\" time=\"" << currentTime << "\" progress=\"" << (int)(completionPercent*10000) << "\" />" << std::endl;
+    std::string str(s.str());
+    sim_communication_port.send(str);
+  } else if (sim_communication_port_open) {
+    std::stringstream s;
+    s << (int)(completionPercent*10000) << " " << phase << endl;
+    std::string str(s.str());
+    sim_communication_port.send(str);
+  }
+#endif
+}
+
 } // extern "C"
