@@ -34,6 +34,7 @@
 #include <math.h>
 #include <string.h>
 
+#include "model_help.h"
 #include "util/omc_error.h"
 #include "util/rtclock.h"
 #include "linearSystem.h"
@@ -102,11 +103,18 @@ int initializeLinearSystems(DATA *data, threadData_t *threadData)
       if(linsys[i].initialAnalyticalJacobian(data, threadData))
       {
         linsys[i].jacobianIndex = -1;
+        throwStreamPrint(threadData, "Failed to initialize the jacobian for torn linear system %d.", (int)linsys[i].equationIndex);
       }
       nnz = data->simulationInfo->analyticJacobians[linsys[i].jacobianIndex].sparsePattern.numberOfNoneZeros;
       linsys[i].nnz = nnz;
     }
 
+    if(nnz/(double)(size*size)<=linearSparseSolverMaxDensity && size>=linearSparseSolverMinSize)
+    {
+      linsys[i].useSparseSolver = 1;
+      infoStreamPrint(LOG_STDOUT, 0, "Using sparse solver for linear system %d,\nbecause density of %.2f remains under threshold of %.2f and size of %d exceeds threshold of %d.\nThe maximum density and the minimal system size for using sparse solvers can be specified\nusing the runtime flags '<-lssMaxDensity=value>' and '<-lssMinSize=value>'.", i, nnz/(double)(size*size), linearSparseSolverMaxDensity, size, linearSparseSolverMinSize);
+    }
+
     /* allocate more system data */
     linsys[i].nominal = (double*) malloc(size*sizeof(double));
     linsys[i].min = (double*) malloc(size*sizeof(double));
@@ -116,60 +124,95 @@ int initializeLinearSystems(DATA *data, threadData_t *threadData)
 
     /* allocate solver data */
     /* the implementation of matrix A is solver-specific */
+    if(linsys[i].useSparseSolver == 1)
+    {
+      switch(data->simulationInfo->lssMethod)
+      {
+    #if !defined(OMC_MINIMAL_RUNTIME)
+      case LSS_LIS:
+        linsys[i].setAElement = setAElementLis;
+        linsys[i].setBElement = setBElementLis;
+        allocateLisData(size, size, nnz, &linsys[i].solverData);
+        break;
+    #endif
+    #ifdef WITH_UMFPACK
+      case LSS_UMFPACK:
+        linsys[i].setAElement = setAElementUmfpack;
+        linsys[i].setBElement = setBElement;
+        allocateUmfPackData(size, size, nnz, &linsys[i].solverData);
+        break;
+      case LSS_KLU:
+        linsys[i].setAElement = setAElementKlu;
+        linsys[i].setBElement = setBElement;
+        allocateKluData(size, size, nnz, &linsys[i].solverData);
+        break;
+    #else
+      case LSS_UMFPACK:
+        throwStreamPrint(threadData, "OMC is compiled without UMFPACK, if you want use umfpack please compile OMC with UMFPACK.");
+        break;
+    #endif
+
+      default:
+        throwStreamPrint(threadData, "unrecognized linear solver");
+      }
+    }
+
+    else{
     switch(data->simulationInfo->lsMethod)
     {
-    case LS_LAPACK:
-      linsys[i].A = (double*) malloc(size*size*sizeof(double));
-      linsys[i].setAElement = setAElement;
-      linsys[i].setBElement = setBElement;
-      allocateLapackData(size, &linsys[i].solverData);
-      break;
+      case LS_LAPACK:
+        linsys[i].A = (double*) malloc(size*size*sizeof(double));
+        linsys[i].setAElement = setAElement;
+        linsys[i].setBElement = setBElement;
+        allocateLapackData(size, &linsys[i].solverData);
+        break;
 
-#if !defined(OMC_MINIMAL_RUNTIME)
-    case LS_LIS:
-      linsys[i].setAElement = setAElementLis;
-      linsys[i].setBElement = setBElementLis;
-      allocateLisData(size, size, nnz, &linsys[i].solverData);
-      break;
-#endif
-#ifdef WITH_UMFPACK
-    case LS_UMFPACK:
-      linsys[i].setAElement = setAElementUmfpack;
-      linsys[i].setBElement = setBElement;
-      allocateUmfPackData(size, size, nnz, &linsys[i].solverData);
-      break;
-    case LS_KLU:
-      linsys[i].setAElement = setAElementKlu;
-      linsys[i].setBElement = setBElement;
-      allocateKluData(size, size, nnz, &linsys[i].solverData);
-      break;
-#else
-    case LS_UMFPACK:
-      throwStreamPrint(threadData, "OMC is compiled without UMFPACK, if you want use umfpack please compile OMC with UMFPACK.");
-      break;
-#endif
+    #if !defined(OMC_MINIMAL_RUNTIME)
+      case LS_LIS:
+        linsys[i].setAElement = setAElementLis;
+        linsys[i].setBElement = setBElementLis;
+        allocateLisData(size, size, nnz, &linsys[i].solverData);
+        break;
+    #endif
+    #ifdef WITH_UMFPACK
+      case LS_UMFPACK:
+        linsys[i].setAElement = setAElementUmfpack;
+        linsys[i].setBElement = setBElement;
+        allocateUmfPackData(size, size, nnz, &linsys[i].solverData);
+        break;
+      case LS_KLU:
+        linsys[i].setAElement = setAElementKlu;
+        linsys[i].setBElement = setBElement;
+        allocateKluData(size, size, nnz, &linsys[i].solverData);
+        break;
+    #else
+      case LS_UMFPACK:
+        throwStreamPrint(threadData, "OMC is compiled without UMFPACK, if you want use umfpack please compile OMC with UMFPACK.");
+        break;
+    #endif
 
-    case LS_TOTALPIVOT:
-      linsys[i].A = (double*) malloc(size*size*sizeof(double));
-      linsys[i].setAElement = setAElement;
-      linsys[i].setBElement = setBElement;
-      allocateTotalPivotData(size, &(linsys[i].solverData));
-      break;
+      case LS_TOTALPIVOT:
+        linsys[i].A = (double*) malloc(size*size*sizeof(double));
+        linsys[i].setAElement = setAElement;
+        linsys[i].setBElement = setBElement;
+        allocateTotalPivotData(size, &(linsys[i].solverData));
+        break;
 
-    case LS_DEFAULT:
-      defaultSolverData = (struct dataLapackAndTotalPivot*) malloc(sizeof(struct dataLapackAndTotalPivot));
-      linsys[i].A = (double*) malloc(size*size*sizeof(double));
-      linsys[i].setAElement = setAElement;
-      linsys[i].setBElement = setBElement;
+      case LS_DEFAULT:
+        defaultSolverData = (struct dataLapackAndTotalPivot*) malloc(sizeof(struct dataLapackAndTotalPivot));
+        linsys[i].A = (double*) malloc(size*size*sizeof(double));
+        linsys[i].setAElement = setAElement;
+        linsys[i].setBElement = setBElement;
 
-      allocateLapackData(size, &(defaultSolverData->lapackData));
-      allocateTotalPivotData(size, &(defaultSolverData->totalpivotData));
+        allocateLapackData(size, &(defaultSolverData->lapackData));
+        allocateTotalPivotData(size, &(defaultSolverData->totalpivotData));
 
-      linsys[i].solverData = (void*) defaultSolverData;
-      break;
+        linsys[i].solverData = (void*) defaultSolverData;
+        break;
 
-    default:
-      throwStreamPrint(threadData, "unrecognized linear solver");
+      default:
+        throwStreamPrint(threadData, "unrecognized linear solver");
+      }
     }
   }
 
@@ -219,8 +262,8 @@ void printLinearSystemSolvingStatistics(DATA *data, int sysNumber, int logLevel)
                                (int)linsys[sysNumber].equationIndex, (int)linsys[sysNumber].size, (int)linsys[sysNumber].nnz,
                                (((double) linsys[sysNumber].nnz) / ((double)(linsys[sysNumber].size*linsys[sysNumber].size)))*100 );
   infoStreamPrint(logLevel, 0, " number of calls                : %ld", linsys[sysNumber].numberOfCall);
-  infoStreamPrint(logLevel, 0, " average time per call          : %f", linsys[sysNumber].totalTime/linsys[sysNumber].numberOfCall);
-  infoStreamPrint(logLevel, 0, " total time                     : %f", linsys[sysNumber].totalTime);
+  infoStreamPrint(logLevel, 0, " average time per call          : %g", linsys[sysNumber].totalTime/linsys[sysNumber].numberOfCall);
+  infoStreamPrint(logLevel, 0, " total time                     : %g", linsys[sysNumber].totalTime);
   messageClose(logLevel);
 }
 
@@ -246,45 +289,75 @@ int freeLinearSystems(DATA *data, threadData_t *threadData)
     free(linsys[i].min);
     free(linsys[i].max);
 
-    switch(data->simulationInfo->lsMethod)
+    if(linsys[i].useSparseSolver == 1)
     {
-    case LS_LAPACK:
-      freeLapackData(&linsys[i].solverData);
-      free(linsys[i].A);
-      break;
+      switch(data->simulationInfo->lssMethod)
+      {
+    #if !defined(OMC_MINIMAL_RUNTIME)
+      case LSS_LIS:
+        freeLisData(&linsys[i].solverData);
+        break;
+    #endif
 
-#if !defined(OMC_MINIMAL_RUNTIME)
-    case LS_LIS:
-      freeLisData(&linsys[i].solverData);
-      break;
-#endif
+    #ifdef WITH_UMFPACK
+      case LSS_UMFPACK:
+        freeUmfPackData(&linsys[i].solverData);
+        break;
+      case LSS_KLU:
+        freeKluData(&linsys[i].solverData);
+        break;
+    #else
+      case LSS_UMFPACK:
+        throwStreamPrint(threadData, "OMC is compiled without UMFPACK, if you want use umfpack please compile OMC with UMFPACK.");
+        break;
+    #endif
 
-#ifdef WITH_UMFPACK
-    case LS_UMFPACK:
-      freeUmfPackData(&linsys[i].solverData);
-      break;
-    case LS_KLU:
-      freeKluData(&linsys[i].solverData);
-      break;
-#else
-    case LS_UMFPACK:
-      throwStreamPrint(threadData, "OMC is compiled without UMFPACK, if you want use umfpack please compile OMC with UMFPACK.");
-      break;
-#endif
+      default:
+        throwStreamPrint(threadData, "unrecognized linear solver");
+      }
+    }
 
-    case LS_TOTALPIVOT:
-      free(linsys[i].A);
-      freeTotalPivotData(&(linsys[i].solverData));
-      break;
+    else{
+      switch(data->simulationInfo->lsMethod)
+      {
+      case LS_LAPACK:
+        freeLapackData(&linsys[i].solverData);
+        free(linsys[i].A);
+        break;
 
-    case LS_DEFAULT:
-      free(linsys[i].A);
-      freeLapackData(&((struct dataLapackAndTotalPivot*) linsys[i].solverData)->lapackData);
-      freeTotalPivotData(&((struct dataLapackAndTotalPivot*) linsys[i].solverData)->totalpivotData);
-      break;
+  #if !defined(OMC_MINIMAL_RUNTIME)
+      case LS_LIS:
+        freeLisData(&linsys[i].solverData);
+        break;
+  #endif
 
-    default:
-      throwStreamPrint(threadData, "unrecognized linear solver");
+  #ifdef WITH_UMFPACK
+      case LS_UMFPACK:
+        freeUmfPackData(&linsys[i].solverData);
+        break;
+      case LS_KLU:
+        freeKluData(&linsys[i].solverData);
+        break;
+  #else
+      case LS_UMFPACK:
+        throwStreamPrint(threadData, "OMC is compiled without UMFPACK, if you want use umfpack please compile OMC with UMFPACK.");
+        break;
+  #endif
+
+      case LS_TOTALPIVOT:
+        free(linsys[i].A);
+        freeTotalPivotData(&(linsys[i].solverData));
+        break;
+
+      case LS_DEFAULT:
+        free(linsys[i].A);
+        freeLapackData(&((struct dataLapackAndTotalPivot*) linsys[i].solverData)->lapackData);
+        freeTotalPivotData(&((struct dataLapackAndTotalPivot*) linsys[i].solverData)->totalpivotData);
+        break;
+
+      default:
+        throwStreamPrint(threadData, "unrecognized linear solver");
+      }
     }
 
     free(linsys[i].solverData);
@@ -313,56 +386,116 @@ int solve_linear_system(DATA *data, threadData_t *threadData, int sysNumber)
   struct dataLapackAndTotalPivot *defaultSolverData;
 
   rt_ext_tp_tick(&(linsys->totalTimeClock));
-  switch(data->simulationInfo->lsMethod)
+
+  /* enable to avoid division by zero */
+  data->simulationInfo->noThrowDivZero = 1;
+
+  if(linsys->useSparseSolver == 1)
   {
-  case LS_LAPACK:
-    success = solveLapack(data, threadData, sysNumber);
-    break;
-
-#if !defined(OMC_MINIMAL_RUNTIME)
-  case LS_LIS:
-    success = solveLis(data, threadData, sysNumber);
-    break;
-#endif
-#ifdef WITH_UMFPACK
-  case LS_KLU:
-    success = solveKlu(data, threadData, sysNumber);
-    break;
-  case LS_UMFPACK:
-    success = solveUmfPack(data, threadData, sysNumber);
-    break;
-#else
-  case LS_UMFPACK:
-    throwStreamPrint(threadData, "OMC is compiled without UMFPACK, if you want use umfpack please compile OMC with UMFPACK.");
-    break;
-#endif
-
-  case LS_TOTALPIVOT:
-    success = solveTotalPivot(data, threadData, sysNumber);
-    break;
-
-  case LS_DEFAULT:
-    defaultSolverData = linsys->solverData;
-    linsys->solverData = defaultSolverData->lapackData;
-
-    success = solveLapack(data, threadData, sysNumber);
-    if (!success){
-      if (linsys->failed){
-        logLevel = LOG_LS;
-      } else {
-        logLevel = LOG_STDOUT;
+    switch(data->simulationInfo->lssMethod)
+    {
+  #if !defined(OMC_MINIMAL_RUNTIME)
+    case LSS_LIS:
+      success = solveLis(data, threadData, sysNumber);
+      break;
+  #endif
+  #ifdef WITH_UMFPACK
+    case LSS_KLU:
+      success = solveKlu(data, threadData, sysNumber);
+      break;
+    case LSS_UMFPACK:
+      success = solveUmfPack(data, threadData, sysNumber);
+      if (!success && linsys->strictTearingFunctionCall != NULL){
+        debugString(LOG_DT, "Solving the casual tearing set failed! Now the strict tearing set is used.");
+        success = linsys->strictTearingFunctionCall(data, threadData);
+        if (success) success=2;
       }
-      warningStreamPrint(logLevel, 0, "The default linear solver fails, the fallback solver with total pivoting is started at time %f. That might raise performance issues, for more information use -lv LOG_LS.", data->localData[0]->timeValue);
-      linsys->solverData = defaultSolverData->totalpivotData;
+      break;
+  #else
+    case LSS_UMFPACK:
+      throwStreamPrint(threadData, "OMC is compiled without UMFPACK, if you want use umfpack please compile OMC with UMFPACK.");
+      break;
+  #endif
+
+    default:
+      throwStreamPrint(threadData, "unrecognized linear solver");
+    }
+  }
+
+  else{
+    switch(data->simulationInfo->lsMethod)
+    {
+    case LS_LAPACK:
+      success = solveLapack(data, threadData, sysNumber);
+      break;
+
+  #if !defined(OMC_MINIMAL_RUNTIME)
+    case LS_LIS:
+      success = solveLis(data, threadData, sysNumber);
+      break;
+  #endif
+  #ifdef WITH_UMFPACK
+    case LS_KLU:
+      success = solveKlu(data, threadData, sysNumber);
+      break;
+    case LS_UMFPACK:
+      success = solveUmfPack(data, threadData, sysNumber);
+      if (!success && linsys->strictTearingFunctionCall != NULL){
+        debugString(LOG_DT, "Solving the casual tearing set failed! Now the strict tearing set is used.");
+        success = linsys->strictTearingFunctionCall(data, threadData);
+        if (success) success=2;
+      }
+      break;
+  #else
+    case LS_UMFPACK:
+      throwStreamPrint(threadData, "OMC is compiled without UMFPACK, if you want use umfpack please compile OMC with UMFPACK.");
+      break;
+  #endif
+
+    case LS_TOTALPIVOT:
       success = solveTotalPivot(data, threadData, sysNumber);
-      linsys->failed = 1;
-    }else{
-      linsys->failed = 0;
+      break;
+
+    case LS_DEFAULT:
+      defaultSolverData = linsys->solverData;
+      linsys->solverData = defaultSolverData->lapackData;
+
+      success = solveLapack(data, threadData, sysNumber);
+
+      /* check if solution process was successful, if not use alternative tearing set if available (dynamic tearing)*/
+      if (!success && linsys->strictTearingFunctionCall != NULL){
+        debugString(LOG_DT, "Solving the casual tearing set failed! Now the strict tearing set is used.");
+        success = linsys->strictTearingFunctionCall(data, threadData);
+        if (success){
+          success=2;
+          linsys->failed = 0;
+        }
+        else{
+          linsys->failed = 1;
+        }
+      }
+      else{
+      /* if there is no alternative tearing set, use fallback solver */
+      if (!success){
+        if (linsys->failed){
+          logLevel = LOG_LS;
+        } else {
+          logLevel = LOG_STDOUT;
+        }
+        warningStreamPrint(logLevel, 0, "The default linear solver fails, the fallback solver with total pivoting is started at time %f. That might raise performance issues, for more information use -lv LOG_LS.", data->localData[0]->timeValue);
+        linsys->solverData = defaultSolverData->totalpivotData;
+        success = solveTotalPivot(data, threadData, sysNumber);
+        linsys->failed = 1;
+      }else{
+        linsys->failed = 0;
+      }
+      }
+      linsys->solverData = defaultSolverData;
+      break;
+
+    default:
+      throwStreamPrint(threadData, "unrecognized linear solver");
     }
-    linsys->solverData = defaultSolverData;
-    break;
-  default:
-    throwStreamPrint(threadData, "unrecognized linear solver");
   }
   linsys->solved = success;
 
@@ -404,7 +537,7 @@ int check_linear_solutions(DATA *data, int printFailingSystems)
   return 0;
 }
 
-/*! \fn check_linear_solutions
+/*! \fn check_linear_solution
  *   This function check whether some of linear systems
  *   are failed to solve. If one is failed it returns 1 otherwise 0.
  *
@@ -457,6 +590,12 @@ int check_linear_solution(DATA *data, int printFailingSystems, int sysNumber)
     return 1;
   }
 
+  if(linsys[i].solved == 2)
+  {
+    linsys[i].solved = 1;
+    return 2;
+  }
+
   TRACE_POP
   return 0;
 }
