@@ -28,6 +28,12 @@
  *
  */
 
+#if !defined(OMC_NUM_LINEAR_SYSYTEMS) || OMC_NUM_NONLINEAR_SYSTEMS>0
+
+#if defined(__AVR__)
+#warning "AVR CPUs are not suitable for non-linear solvers"
+#endif
+
 /*! \file nonlinearSolverHomotopy.c
 *  \author bbachmann
 */
@@ -49,6 +55,16 @@
 #include "nonlinearSolverHomotopy.h"
 #include "nonlinearSolverHybrd.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+extern int dgesv_(int *n, int *nrhs, doublereal *a, int *lda, int *ipiv, doublereal *b, int *ldb, int *info);
+
+#ifdef __cplusplus
+}
+#endif
+
 /*! \typedef DATA_HOMOTOPY
  * define memory structure for nonlinear system solver
  *  \author bbachmann
@@ -60,10 +76,10 @@ typedef struct DATA_HOMOTOPY
   int n; /* dimension; n == size */
   int m; /* dimension: m == size+1 */
 
-  double xtol; /* tolerance for updating solution vector */
-  double ftol; /* tolerance fo accepting accuracy */
+  double xtol_sqrd; /* tolerance for updating solution vector */
+  double ftol_sqrd; /* tolerance for accepting accuracy */
 
-  double error_f;
+  double error_f_sqrd;
 
   double* resScaling; /* residual scaling */
   double* fvecScaled; /* function values scaled */
@@ -150,10 +166,10 @@ int allocateHomotopyData(int size, void** voiddata)
   data->initialized = 0;
   data->n = size;
   data->m = size + 1;
-  data->xtol = 1e-24;
-  data->ftol = 1e-24;
+  data->xtol_sqrd = newtonXTol*newtonXTol;
+  data->ftol_sqrd = newtonFTol*newtonFTol;
 
-  data->error_f = 0;
+  data->error_f_sqrd = 0;
 
   data->maxNumberOfIterations = size*100;
   data->numberOfIterations = 0;
@@ -508,7 +524,7 @@ void debugDouble(int logName, char* message, double value)
  *  \author bbachmann
  */
 
-double vecNorm(int n, double *x)
+double vec2Norm(int n, double *x)
 {
   int i;
   double norm=0.0;
@@ -517,7 +533,7 @@ double vecNorm(int n, double *x)
   return sqrt(norm);
 }
 
-double vecNorm2(int n, double *x)
+double vec2NormSqrd(int n, double *x)
 {
   int i;
   double norm=0.0;
@@ -613,7 +629,7 @@ void vecDivScaling(int n, double *a, double *b, double *c)
 void vecNormalize(int n, double *a, double *b)
 {
   int i;
-  double norm = vecNorm(n,a);
+  double norm = vec2Norm(n,a);
   for (i=0;i<n;i++)
     b[i] = a[i]/norm;
 }
@@ -862,7 +878,6 @@ static int wrapper_fvec_der(DATA_HOMOTOPY* solverData, double* x, double* fJac)
   if(ACTIVE_STREAM(LOG_NLS_JAC_TEST))
   {
     int n = solverData->n;
-
     /* debugMatrixDouble(LOG_NLS_JAC_TEST,"analytical jacobian:",fJac, n, n+1); */
     getNumericalJacobianHomotopy(solverData, x, solverData->debug_fJac);
     /* debugMatrixDouble(LOG_NLS_JAC_TEST,"numerical jacobian:",solverData->debug_fJac, n, n+1); */
@@ -954,6 +969,7 @@ static int wrapper_fvec_homotopy_fixpoint_der(DATA_HOMOTOPY* solverData, double*
   return 0;
 }
 
+
 /*! \fn getIndicesOfPivotElement for calculating pivot element
  *
  *  \author bbachmann
@@ -977,6 +993,7 @@ static int wrapper_fvec_homotopy_fixpoint_der(DATA_HOMOTOPY* solverData, double*
   }
 }
 
+
 /*! \fn solveSystemWithTotalPivotSearch for solution of overdetermined linear system
  *  used for the homotopy solver, for calculating the direction
  *  used for the newton solver, for calculating the Newton step
@@ -995,6 +1012,7 @@ int solveSystemWithTotalPivotSearch(int n, double* x, double* A, int* indRow, in
    double *res;
 
    debugMatrixDouble(LOG_NLS_JAC,"Linear System Matrix [Jac res]:",A, n, m);
+   debugVectorDouble(LOG_NLS_JAC,"vector b:", A+n*n, n);
 
    /* assume full rank of matrix [n x (n+1)] */
    *rank = n;
@@ -1047,12 +1065,17 @@ int solveSystemWithTotalPivotSearch(int n, double* x, double* A, int* indRow, in
   for (detJac=1.0,k=0; k<n; k++) detJac *= A[indRow[k] + indCol[k]*n];
 
   debugMatrixPermutedDouble(LOG_NLS_JAC,"Linear System Matrix [Jac res] after decomposition",A, n, m, indRow, indCol);
-  debugDouble(LOG_NLS_JAC,"Determinante = ", detJac);
+  debugDouble(LOG_NLS_JAC,"Determinant = ", detJac);
+  if (isnan(detJac)){
+    warningStreamPrint(LOG_NLS, 0, "Jacobian determinant is NaN.");
+    return -1;
+  }
+
   /* Solve even singular matrices !!! */
   for (i=n-1;i>=0; i--) {
     if (i>=*rank) {
       /* this criteria should be evaluated and may be improved in future */
-      if (fabs(A[indRow[i] + indCol[n]*n])>1e-12) {
+      if (fabs(A[indRow[i] + indCol[n]*n])>1e-6) {
         warningStreamPrint(LOG_NLS, 0, "under-determined linear system not solvable!");
         return -1;
       } else {
@@ -1073,20 +1096,85 @@ int solveSystemWithTotalPivotSearch(int n, double* x, double* A, int* indRow, in
     *pos=indCol[n];
   }
 
+  return 0;
+}
+
+
+/*! \fn linearSolverWrapper
+ */
+int linearSolverWrapper(int n, double* x, double* A, int* indRow, int* indCol, int *pos, int *rank, int method)
+{
+  /* First try to use lapack and if it fails then
+   * use solveSystemWithTotalPivotSearch */
+  int returnValue = -1;
+  int solverinfo;
+  int nrhs = 1;
+  int lda = n;
+
+  debugMatrixDouble(LOG_NLS_JAC,"Linear System Matrix [Jac res]:", A, n, n+1);
+  debugVectorDouble(LOG_NLS_JAC,"vector b:", x, n);
+
+  switch(method){
+    case (1): /* NLS_LS_TOTALPIVOT */
+
+      solverinfo = solveSystemWithTotalPivotSearch(n, x, A, indRow, indCol, pos, rank);
+      /* in case of failing */
+      if (solverinfo != 0)
+      {
+        /* debug information */
+        debugString(LOG_NLS_V, "Linear total pivot solver failed!!!");
+        debugString(LOG_NLS_V, "******************************************************");
+      }
+      else
+      {
+        returnValue = 0;
+      }
+      break;
+    case 2: /* NLS_LS_LAPACK */
+      /* Solve system with lapack */
+      dgesv_((int*) &n,
+          (int*) &nrhs,
+          A,
+          (int*) &lda,
+          indRow,
+          x,
+          (int*) &n,
+          &solverinfo);
+
+      debugMatrixDouble(LOG_NLS_JAC,"Linear system matrix [Jac res] after decomposition:", A, n, n+1);
+      /* in case of failing */
+      if (solverinfo != 0)
+      {
+        /* debug information */
+        debugString(LOG_NLS_V, "Linear lapack solver failed!!!");
+        debugString(LOG_NLS_V, "******************************************************");
+      }
+      else
+      {
+        vecScalarMult(n, x, -1, x);
+        returnValue = 0;
+      }
+      break;
+    default:
+      warningStreamPrint(LOG_STDOUT, 0, "Non-Linear solver try to run with a unknown linear solver.");
+  }
+
   /* Debugging error of linear system */
   if(ACTIVE_STREAM(LOG_NLS_JAC))
   {
-    res = (double*) calloc(n,sizeof(double));
-    debugVectorDouble(LOG_NLS_JAC,"solution:", x, m);
-    matVecMult(n, m, A, x, res);
+    double* res = (double*) calloc(n,sizeof(double));
+    debugVectorDouble(LOG_NLS_JAC,"solution:", x, n);
+    matVecMult(n, n, A, x, res);
     debugVectorDouble(LOG_NLS_JAC,"test solution:", res, n);
-    debugDouble(LOG_NLS_JAC,"error of linear system = ", vecNorm(n, res));
+    debugDouble(LOG_NLS_JAC,"error of linear system = ", vec2Norm(n, res));
     free(res);
     messageClose(LOG_NLS_JAC);
   }
 
-  return 0;
+  return returnValue;
 }
+
+
 /*! \fn solve system with damped Newton-Raphson
  *
  *  \author bbachmann
@@ -1096,7 +1184,7 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
 {
   int numberOfIterations = 0 ,i, j, n=solverData->n, m=solverData->m;
   int  pos = solverData->n, rank;
-  double error_f, error_f1, error_f2,error_f_scaled, delta_x, delta_x_scaled, grad_f1, grad_f;
+  double error_f_sqrd, error_f1_sqrd, error_f2_sqrd, error_f_sqrd_scaled, delta_x_sqrd, delta_x_sqrd_scaled, grad_f;
   int numberOfSmallSteps = 0;
   double error_f_old = 1e100;
   int countNegativeSteps = 0;
@@ -1110,6 +1198,7 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
   int assert = 1;
   threadData_t *threadData = solverData->threadData;
   NONLINEAR_SYSTEM_DATA* nonlinsys = &(solverData->data->simulationInfo->nonlinearSystemData[solverData->data->simulationInfo->currentNonlinearSystemIndex]);
+  int linearSolverMethod = solverData->data->simulationInfo->nlsLinearSolver;
 
   /* debug information */
   debugString(LOG_NLS_V, "******************************************************");
@@ -1121,8 +1210,8 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
   solverData->info = 0;
 
   /* calculated error of function values */
-  error_f = vecNorm2(solverData->n, solverData->f1);
-  error_f_scaled = error_f;
+  error_f_sqrd = vec2NormSqrd(solverData->n, solverData->f1);
+  error_f_sqrd_scaled = error_f_sqrd;
 
   while(1)
   {
@@ -1131,7 +1220,7 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
     debugInt(LOG_NLS_V, "Iteration:", numberOfIterations);
 
     /* solve jacobian and function value (both stored in hJac, last column is fvec), side effects: jacobian matrix is changed */
-    if ((numberOfIterations>1) && (solveSystemWithTotalPivotSearch(solverData->n, solverData->dy0, solverData->fJac, solverData->indRow, solverData->indCol, &pos, &rank) != 0))
+    if ((numberOfIterations>1) && linearSolverWrapper(solverData->n, solverData->dy0, solverData->fJac, solverData->indRow, solverData->indCol, &pos, &rank, linearSolverMethod) != 0)
     {
       /* report solver abortion */
       solverData->info=-1;
@@ -1183,18 +1272,18 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
 
       /* Damping (see Numerical Recipes) */
       /* calculate gradient of quadratic function for damping strategy */
-      grad_f = -2.0*error_f;
-      error_f1 = vecNorm2(solverData->n, solverData->f1);
+      grad_f = -2.0*error_f_sqrd;
+      error_f1_sqrd = vec2NormSqrd(solverData->n, solverData->f1);
       debugDouble(LOG_NLS_V,"Need to damp, grad_f = ", grad_f);
-      debugDouble(LOG_NLS_V,"Need to damp, error_f = ", error_f);
+      debugDouble(LOG_NLS_V,"Need to damp, error_f = ", sqrt(error_f_sqrd));
 
       debugDouble(LOG_NLS_V,"Need to damp this!! lambda1 = ", lambda1);
-      debugDouble(LOG_NLS_V,"Need to damp, error_f1 = ", error_f1);
+      debugDouble(LOG_NLS_V,"Need to damp, error_f1 = ", sqrt(error_f1_sqrd));
 
-      debugDouble(LOG_NLS_V,"Need to damp, forced error = ", error_f + alpha*lambda1*grad_f);
-      if ((error_f1 > error_f + alpha*lambda1*grad_f) && (error_f > 1e-12) && (error_f_scaled > 1e-12))
+      debugDouble(LOG_NLS_V,"Need to damp, forced error = ", error_f_sqrd + alpha*lambda1*grad_f);
+      if ((error_f1_sqrd > error_f_sqrd + alpha*lambda1*grad_f) && (error_f_sqrd > 1e-12) && (error_f_sqrd_scaled > 1e-12))
       {
-        lambda2 = fmax(-lambda1*lambda1*grad_f/(2*(error_f1-error_f-lambda1*grad_f)),lambdaMin);
+        lambda2 = fmax(-lambda1*lambda1*grad_f/(2*(error_f1_sqrd-error_f_sqrd-lambda1*grad_f)),lambdaMin);
         debugDouble(LOG_NLS_V,"Need to damp this!! lambda2 = ", lambda2);
         vecAddScal(solverData->n, x, solverData->dy0, lambda2, solverData->x1);
         assert= 1;
@@ -1202,8 +1291,8 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
         MMC_TRY_INTERNAL(simulationJumpBuffer)
 #endif
         solverData->f(solverData, solverData->x1, solverData->f1);
-        error_f2 = vecNorm2(solverData->n, solverData->f1);
-        debugDouble(LOG_NLS_V,"Need to damp, error_f2 = ", error_f2);
+        error_f2_sqrd = vec2NormSqrd(solverData->n, solverData->f1);
+        debugDouble(LOG_NLS_V,"Need to damp, error_f2 = ", sqrt(error_f2_sqrd));
         assert = 0;
 #ifndef OMC_EMCC
         MMC_CATCH_INTERNAL(simulationJumpBuffer)
@@ -1214,10 +1303,10 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
           solverData->info = -1;
           break;
         }
-        if ((error_f1 > error_f + alpha*lambda2*grad_f) && (error_f > 1e-12) && (error_f_scaled > 1e-12))
+        if ((error_f1_sqrd > error_f_sqrd + alpha*lambda2*grad_f) && (error_f_sqrd > 1e-12) && (error_f_sqrd_scaled > 1e-12))
         {
-          rhs1 = error_f1 - grad_f*lambda1 - error_f;
-          rhs2 = error_f2 - grad_f*lambda2 - error_f;
+          rhs1 = error_f1_sqrd - grad_f*lambda1 - error_f_sqrd;
+          rhs2 = error_f2_sqrd - grad_f*lambda2 - error_f_sqrd;
           a3 = (rhs1/(lambda1*lambda1) - rhs2/(lambda2*lambda2))/(lambda1 - lambda2);
           a2 = (-lambda2*rhs1/(lambda1*lambda1) + lambda1*rhs2/(lambda2*lambda2))/(lambda1 - lambda2);
           if (a3==0.0)
@@ -1241,8 +1330,8 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
           MMC_TRY_INTERNAL(simulationJumpBuffer)
 #endif
           solverData->f(solverData, solverData->x1, solverData->f1);
-          error_f1 = vecNorm2(solverData->n, solverData->f1);
-          debugDouble(LOG_NLS_V,"Need to damp, error_f1 = ", error_f1);
+          error_f1_sqrd = vec2NormSqrd(solverData->n, solverData->f1);
+          debugDouble(LOG_NLS_V,"Need to damp, error_f1 = ", sqrt(error_f1_sqrd));
           assert = 0;
 #ifndef OMC_EMCC
           MMC_CATCH_INTERNAL(simulationJumpBuffer)
@@ -1258,7 +1347,7 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
         lambda = lambda1;
       }
     }
-    /* updating x, fvec, error_f */
+    /* updating x, fvec, error_f_sqrd */
     /* event. swapPointer(&x, &(solverData->x1)); */
     vecCopy(solverData->n, solverData->x1, x);
 
@@ -1268,23 +1357,23 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
     debugVectorDouble(LOG_NLS_V,"scaled function values:",solverData->fvecScaled, n);
 
     vecDivScaling(solverData->n, solverData->dy0, solverData->xScaling, solverData->dxScaled);
-    delta_x        = vecNorm2(solverData->n, solverData->dy0);
-    delta_x_scaled = vecNorm2(solverData->n, solverData->dxScaled);
-    error_f        = vecNorm2(solverData->n, solverData->f1);
-    error_f_scaled = vecNorm2(solverData->n, solverData->fvecScaled);
+    delta_x_sqrd        = vec2NormSqrd(solverData->n, solverData->dy0);
+    delta_x_sqrd_scaled = vec2NormSqrd(solverData->n, solverData->dxScaled);
+    error_f_sqrd        = vec2NormSqrd(solverData->n, solverData->f1);
+    error_f_sqrd_scaled = vec2NormSqrd(solverData->n, solverData->fvecScaled);
 
 
     /* debug information */
     debugString(LOG_NLS_V, "error measurements:");
-    debugDouble(LOG_NLS_V, "delta_x        =", delta_x);
-    debugDouble(LOG_NLS_V, "delta_x_scaled =", delta_x_scaled);
-    debugDouble(LOG_NLS_V, "eps_x          =", solverData->xtol);
-    debugDouble(LOG_NLS_V, "error_f        =", error_f);
-    debugDouble(LOG_NLS_V, "error_f_scaled =", error_f_scaled);
-    debugDouble(LOG_NLS_V, "eps_f          =", solverData->ftol);
+    debugDouble(LOG_NLS_V, "delta_x        =", sqrt(delta_x_sqrd));
+    debugDouble(LOG_NLS_V, "delta_x_scaled =", sqrt(delta_x_sqrd_scaled));
+    debugDouble(LOG_NLS_V, "newtonXTol          =", sqrt(solverData->xtol_sqrd));
+    debugDouble(LOG_NLS_V, "error_f        =", sqrt(error_f_sqrd));
+    debugDouble(LOG_NLS_V, "error_f_scaled =", sqrt(error_f_sqrd_scaled));
+    debugDouble(LOG_NLS_V, "newtonFTol          =", sqrt(solverData->ftol_sqrd));
 
-    countNegativeSteps += (error_f > 10*error_f_old);
-    error_f_old = error_f;
+    countNegativeSteps += (error_f_sqrd > 10*error_f_old);
+    error_f_old = error_f_sqrd;
 
 #if !defined(OMC_MINIMAL_RUNTIME)
     if (solverData->data->simulationInfo->nlsCsvInfomation){
@@ -1294,15 +1383,15 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
                              numberOfIterations,
                              solverData->x,
                              solverData->f1,
-                             delta_x,
-                             delta_x_scaled,
-                             error_f,
-                             error_f_scaled,
+                             delta_x_sqrd,
+                             delta_x_sqrd_scaled,
+                             error_f_sqrd,
+                             error_f_sqrd_scaled,
                              lambda
       );
     }
 #endif
-    if ((error_f_scaled < 1e-30*error_f) || countNegativeSteps > 20)
+    if ((error_f_sqrd_scaled < 1e-30*error_f_sqrd) || countNegativeSteps > 20)
     {
       debugInt(LOG_NLS_V,"UPS! Something happened, NegativeSteps = ", countNegativeSteps);
       solverData->info = -1;
@@ -1310,7 +1399,7 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
     }
 
     /* solution found */
-    if (((error_f < solverData->ftol) || (error_f_scaled < solverData->ftol)) && ((delta_x_scaled < solverData->xtol) || (delta_x < solverData->xtol)))
+    if (((error_f_sqrd < solverData->ftol_sqrd) || (error_f_sqrd_scaled < solverData->ftol_sqrd)) && ((delta_x_sqrd_scaled < solverData->xtol_sqrd) || (delta_x_sqrd < solverData->xtol_sqrd)))
     {
       solverData->info = 1;
 
@@ -1321,7 +1410,7 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
 
       /* update statistics */
       solverData->numberOfIterations += numberOfIterations;
-      solverData->error_f = error_f;
+      solverData->error_f_sqrd = error_f_sqrd;
 
       break;
     }
@@ -1340,11 +1429,11 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
       break;
     }
 
-    numberOfSmallSteps += (delta_x < solverData->xtol*1e4) ||  (delta_x_scaled < solverData->xtol*1e4);
+    numberOfSmallSteps += (delta_x_sqrd < solverData->xtol_sqrd*1e4) ||  (delta_x_sqrd_scaled < solverData->xtol_sqrd*1e4);
     /* check changes in unknown vector */
-    if ((delta_x < solverData->xtol) ||  (delta_x_scaled < solverData->xtol) || (numberOfSmallSteps > 20))
+    if ((delta_x_sqrd < solverData->xtol_sqrd) ||  (delta_x_sqrd_scaled < solverData->xtol_sqrd) || (numberOfSmallSteps > 20))
     {
-      if ((error_f < solverData->ftol*1e6) || (error_f_scaled < solverData->ftol*1e6))
+      if ((error_f_sqrd < solverData->ftol_sqrd*1e6) || (error_f_sqrd_scaled < solverData->ftol_sqrd*1e6))
       {
         solverData->info = 1;
 
@@ -1352,7 +1441,7 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
         debugString(LOG_NLS_V, "NEWTON SOLVER DID CONVERGE TO A SOLUTION WITH LESS ACCURACY!!!");
         printUnknowns(LOG_NLS_V, solverData);
         debugString(LOG_NLS_V, "******************************************************");
-        solverData->error_f = 0;
+        solverData->error_f_sqrd = 0;
 
       } else
       {
@@ -1388,6 +1477,7 @@ static int newtonAlgorithm(DATA_HOMOTOPY* solverData, double* x)
     matVecMultAbsBB(solverData->n, solverData->fJac, solverData->ones, solverData->resScaling);
     debugVectorDouble(LOG_NLS_JAC, "residuum scaling:", solverData->resScaling, solverData->n);
     scaleMatrixRows(solverData->n, solverData->m, solverData->fJac);
+    vecCopy(n, solverData->fJac + n*n, solverData->dy0);
   }
   return 0;
 }
@@ -1400,11 +1490,11 @@ static int homotopyAlgorithm(DATA_HOMOTOPY* solverData, double *x)
 {
   int i, j;
   double xerror = -1, xerror_scaled = -1;
-  double error_h, error_h_scaled, delta_x, delta_x_scaled;
+  double error_h, error_h_scaled, delta_x;
   int success = 0;
   int nfunc_evals = 0;
   int continuous = 1;
-  double local_tol = solverData->ftol;
+  double local_tol = solverData->ftol_sqrd;
   double vecScalarProduct;
 
   int giveUp = 0;
@@ -1557,7 +1647,7 @@ static int homotopyAlgorithm(DATA_HOMOTOPY* solverData, double *x)
     /* Corrector step: Newton iteration! */
     for(j=0;j<maxiter;j++)
     {
-      if (vecNorm(solverData->n, solverData->hvec)<hEps || vecNorm(solverData->n, solverData->hvecScaled)<hEps)
+      if (vec2Norm(solverData->n, solverData->hvec)<hEps || vec2Norm(solverData->n, solverData->hvecScaled)<hEps)
       {
         stepAccept = 1;
         break;
@@ -1611,9 +1701,9 @@ static int homotopyAlgorithm(DATA_HOMOTOPY* solverData, double *x)
       /* Calculate different error measurements */
       vecDivScaling(solverData->n, solverData->hvec, solverData->resScaling, solverData->hvecScaled);
 
-      delta_x        = vecNorm(solverData->m, solverData->dy1);
-      error_h        = vecNorm(solverData->n, solverData->hvec);
-      error_h_scaled = vecNorm(solverData->n, solverData->hvecScaled);
+      delta_x        = vec2Norm(solverData->m, solverData->dy1);
+      error_h        = vec2Norm(solverData->n, solverData->hvec);
+      error_h_scaled = vec2Norm(solverData->n, solverData->hvecScaled);
 
 
       /* debug information
@@ -1632,7 +1722,7 @@ static int homotopyAlgorithm(DATA_HOMOTOPY* solverData, double *x)
       vecDiff(solverData->m, solverData->y1, solverData->yt, solverData->dy1);
       vecDiff(solverData->m, solverData->yt, solverData->y0, solverData->dy2);
       printHomotopyCorrectorStep(LOG_NLS_HOMOTOPY, solverData);
-      bend = vecNorm(solverData->m,solverData->dy1)/vecNorm(solverData->m,solverData->dy2);
+      bend = vec2Norm(solverData->m,solverData->dy1)/vec2Norm(solverData->m,solverData->dy2);
     }
     if ((bend > adaptBend) ||   !stepAccept)
     {
@@ -1699,9 +1789,9 @@ int solveHomotopy(DATA *data, threadData_t *threadData, int sysNumber)
   int success = 0;
   int nfunc_evals = 0;
   int continuous = 1;
-  double local_tol = solverData->ftol;
+  double local_tol = solverData->ftol_sqrd;
   double lambda;
-  double error_f, error_f1;
+  double error_f_sqrd, error_f1_sqrd;
 
   int assert = 1;
   int giveUp = 0;
@@ -1716,6 +1806,7 @@ int solveHomotopy(DATA *data, threadData_t *threadData, int sysNumber)
   int runHomotopy = 0;
   int skipNewton = 0;
   int numberOfFunctionEvaluationsOld = solverData->numberOfFunctionEvaluations;
+  int casualTearingSet = systemData->strictTearingFunctionCall != NULL;
 
   modelica_boolean* relationsPreBackup;
   relationsPreBackup = (modelica_boolean*) malloc(data->modelData->nRelations*sizeof(modelica_boolean));
@@ -1735,7 +1826,7 @@ int solveHomotopy(DATA *data, threadData_t *threadData, int sysNumber)
   vecConst(solverData->m,1.0,solverData->ones);
 
   debugString(LOG_NLS_V, "------------------------------------------------------");
-  debugString(LOG_NLS_V, "SOLVING NON-LINEAR SYSTEM USING HOMOTOPY SOLVER");
+  debugString(LOG_NLS_V, "SOLVING NON-LINEAR SYSTEM USING MIXED SOLVER (Newton/Homotopy solver)");
   debugInt(LOG_NLS_V, "EQUATION NUMBER:", eqSystemNumber);
   debugDouble(LOG_NLS_V, "TIME:", solverData->timeValue);
   debugInt(LOG_NLS_V,   "number of function calls (so far!): ",numberOfFunctionEvaluationsOld);
@@ -1781,10 +1872,10 @@ int solveHomotopy(DATA *data, threadData_t *threadData, int sysNumber)
 
     solverData->f(solverData, solverData->x0, solverData->f1);
     /* Try to get out of here!!! */
-    error_f        = vecNorm2(solverData->n, solverData->f1);
-    if ((error_f - solverData->error_f)<=0)
+    error_f_sqrd        = vec2NormSqrd(solverData->n, solverData->f1);
+    if ((error_f_sqrd - solverData->error_f_sqrd)<=0)
     {
-      //infoStreamPrint(LOG_STDOUT, 0, "No Iteration at time %g needed new f = %g  and old f1 = %g", solverData->timeValue, error_f, solverData->error_f);
+      //infoStreamPrint(LOG_STDOUT, 0, "No Iteration at time %g needed new f = %g  and old f1 = %g", solverData->timeValue, error_f_sqrd, solverData->error_f_sqrd);
       if (mixedSystem && data->simulationInfo->discreteCall && isNotEqualVectorInt(((DATA*)data)->modelData->nRelations, ((DATA*)data)->simulationInfo->relations, relationsPreBackup)){}
       else
       {
@@ -1826,6 +1917,11 @@ int solveHomotopy(DATA *data, threadData_t *threadData, int sysNumber)
 #ifndef OMC_EMCC
     MMC_CATCH_INTERNAL(simulationJumpBuffer)
  #endif
+    if (assert && casualTearingSet)
+    {
+      giveUp = 1;
+      break;
+    }
     if (assert)
     {
       tries += 1;
@@ -1833,7 +1929,7 @@ int solveHomotopy(DATA *data, threadData_t *threadData, int sysNumber)
     else
       break;
     /* break symmetry, when varying start values */
-    /* try to find regular initial point, iff necessary */
+    /* try to find regular initial point, if necessary */
     if (tries == 1)
     {
       debugString(LOG_NLS_V, "assert handling:\t vary initial guess by +1%.");
@@ -1860,12 +1956,21 @@ int solveHomotopy(DATA *data, threadData_t *threadData, int sysNumber)
     if (!skipNewton){
 
       /* set x vector */
-      if(data->simulationInfo->discreteCall)
+      if(data->simulationInfo->discreteCall){
         memcpy(systemData->nlsx, solverData->x, solverData->n*(sizeof(double)));
-      else
+      }
+      else{
         memcpy(systemData->nlsxExtrapolation, solverData->x, solverData->n*(sizeof(double)));
+      }
 
       newtonAlgorithm(solverData, solverData->x);
+
+      // If this is the casual tearing set (only exists for dynamic tearing), break after first try
+      if (solverData->info == -1 && casualTearingSet){
+        infoStreamPrint(LOG_NLS, 0, "### No Solution for the casual tearing set at the first try! ###");
+        break;
+      }
+
       if (solverData->info == -1){
         solverDataHybrid = (DATA_HYBRD*)(solverData->dataHybrid);
         systemData->solverData = solverDataHybrid;
@@ -2006,3 +2111,5 @@ int solveHomotopy(DATA *data, threadData_t *threadData, int sysNumber)
 
   return success;
 }
+
+#endif
