@@ -150,17 +150,11 @@ int wrapper_fvec_newton(int* n, double* x, double* fvec, void* userdata, int fj)
       double xsave;
 
       int i,j,l, linear=0;
-      linear = systemData->method;
 
       for(i = 0; i < *n; i++) {
-        if(linear) {
-          delta_hh = 1;
-        } else {
-
-          delta_hh = fmax(delta_h * fmax(fabs(x[i]), fabs(fvec[i])), delta_h);
-          delta_hh = ((fvec[i] >= 0) ? delta_hh : -delta_hh);
-          delta_hh = x[i] + delta_hh - x[i];
-        }
+        delta_hh = fmax(delta_h * fmax(fabs(x[i]), fabs(fvec[i])), delta_h);
+        delta_hh = ((fvec[i] >= 0) ? delta_hh : -delta_hh);
+        delta_hh = x[i] + delta_hh - x[i];
         xsave = x[i];
         x[i] += delta_hh;
         delta_hh = 1. / delta_hh;
@@ -203,6 +197,7 @@ int solveNewton(DATA *data, threadData_t *threadData, int sysNumber)
   int retries2 = 0;
   int nonContinuousCase = 0;
   modelica_boolean *relationsPreBackup = NULL;
+  // int casualTearingSet = systemData->strictTearingFunctionCall != NULL;
   int casualTearingSet = data->simulationInfo->nonlinearSystemData[sysNumber].strictTearingFunctionCall != NULL;
 
   DATA_USER* userdata = (DATA_USER*)malloc(sizeof(DATA_USER));
@@ -291,6 +286,7 @@ int solveNewton(DATA *data, threadData_t *threadData, int sysNumber)
 
       /* Then try with old values (instead of extrapolating )*/
     }
+    // If this is the casual tearing set (only exists for dynamic tearing), break after first try
     else if(retries < 1 && casualTearingSet)
     {
       giveUp = 1;
