@@ -36,14 +36,16 @@
 
 #include "util/omc_error.h"
 #include "nonlinearSystem.h"
+#include "nonlinearValuesList.h"
+#if !defined(OMC_MINIMAL_RUNTIME)
 #include "kinsolSolver.h"
 #include "nonlinearSolverHybrd.h"
 #include "nonlinearSolverNewton.h"
 #include "newtonIteration.h"
+#endif
 #include "nonlinearSolverHomotopy.h"
-#include "simulation/simulation_info_xml.h"
+#include "simulation/simulation_info_json.h"
 #include "simulation/simulation_runtime.h"
-#include "util/write_csv.h"
 
 /* for try and catch simulationJumpBuffer */
 #include "meta/meta_modelica.h"
@@ -56,6 +58,8 @@ struct dataNewtonAndHybrid
   void* hybridData;
 };
 
+#if !defined(OMC_MINIMAL_RUNTIME)
+#include "util/write_csv.h"
 /*! \fn int initializeNLScsvData(DATA* data, NONLINEAR_SYSTEM_DATA* systemData)
  *
  *  This function initializes csv files for analysis propose.
@@ -67,17 +71,25 @@ int initializeNLScsvData(DATA* data, NONLINEAR_SYSTEM_DATA* systemData)
 {
   struct csvStats* stats = (struct csvStats*) malloc(sizeof(struct csvStats));
   char buffer[100];
-  sprintf(buffer, "%s_NLS%dStatsCall.csv", data->modelData.modelFilePrefix, (int)systemData->equationIndex);
+  sprintf(buffer, "%s_NLS%dStatsCall.csv", data->modelData->modelFilePrefix, (int)systemData->equationIndex);
   stats->callStats = omc_write_csv_init(buffer, ',', '"');
 
-  sprintf(buffer, "%s_NLS%dStatsIter.csv", data->modelData.modelFilePrefix, (int)systemData->equationIndex);
+  sprintf(buffer, "%s_NLS%dStatsIter.csv", data->modelData->modelFilePrefix, (int)systemData->equationIndex);
   stats->iterStats = omc_write_csv_init(buffer, ',', '"');
 
   systemData->csvData = stats;
 
   return 0;
 }
+#else
+int initializeNLScsvData(DATA* data, NONLINEAR_SYSTEM_DATA* systemData)
+{
+  fprintf(stderr, "initializeNLScsvData not implemented for OMC_MINIMAL_RUNTIME");
+  abort();
+}
+#endif
 
+#if !defined(OMC_MINIMAL_RUNTIME)
 /*! \fn int print_csvLineCallStatsHeader(OMC_WRITE_CSV* csvData)
  *
  *  This function initializes csv files for analysis propose.
@@ -87,7 +99,8 @@ int initializeNLScsvData(DATA* data, NONLINEAR_SYSTEM_DATA* systemData)
  */
 int print_csvLineCallStatsHeader(OMC_WRITE_CSV* csvData)
 {
-  char buffer[1024] = "";
+  char buffer[1024];
+  buffer[0] = 0;
 
   /* number of call */
   sprintf(buffer,"numberOfCall");
@@ -188,7 +201,7 @@ int print_csvLineIterStatsHeader(DATA* data, NONLINEAR_SYSTEM_DATA* systemData,
 {
   char buffer[1024];
   int j;
-  int size = modelInfoGetEquation(&data->modelData.modelDataXml, systemData->equationIndex).numVar;
+  int size = modelInfoGetEquation(&data->modelData->modelDataXml, systemData->equationIndex).numVar;
 
   /* number of call */
   sprintf(buffer,"numberOfCall");
@@ -202,7 +215,7 @@ int print_csvLineIterStatsHeader(DATA* data, NONLINEAR_SYSTEM_DATA* systemData,
 
   /* variables x */
   for(j=0; j<size; ++j) {
-    sprintf(buffer, "%s", modelInfoGetEquation(&data->modelData.modelDataXml, systemData->equationIndex).vars[j]);
+    sprintf(buffer, "%s", modelInfoGetEquation(&data->modelData->modelDataXml, systemData->equationIndex).vars[j]);
     omc_write_csv(csvData, buffer);
     fputc(csvData->seperator,csvData->handle);
   }
@@ -313,6 +326,7 @@ int print_csvLineIterStats(void* voidCsvData, int size, int num,
 
   return 0;
 }
+#endif
 
 /*! \fn int initializeNonlinearSystems(DATA *data)
  *
@@ -325,12 +339,12 @@ int initializeNonlinearSystems(DATA *data, threadData_t *threadData)
   TRACE_PUSH
   int i;
   int size;
-  NONLINEAR_SYSTEM_DATA *nonlinsys = data->simulationInfo.nonlinearSystemData;
+  NONLINEAR_SYSTEM_DATA *nonlinsys = data->simulationInfo->nonlinearSystemData;
   struct dataNewtonAndHybrid *mixedSolverData;
 
   infoStreamPrint(LOG_NLS, 1, "initialize non-linear system solvers");
 
-  for(i=0; i<data->modelData.nNonLinearSystems; ++i)
+  for(i=0; i<data->modelData->nNonLinearSystems; ++i)
   {
     size = nonlinsys[i].size;
     nonlinsys[i].numberOfFEval = 0;
@@ -353,6 +367,12 @@ int initializeNonlinearSystems(DATA *data, threadData_t *threadData)
     nonlinsys[i].nlsx = (double*) malloc(size*sizeof(double));
     nonlinsys[i].nlsxExtrapolation = (double*) malloc(size*sizeof(double));
     nonlinsys[i].nlsxOld = (double*) malloc(size*sizeof(double));
+    nonlinsys[i].resValues = (double*) malloc(size*sizeof(double));
+
+    /* allocate value list*/
+    nonlinsys[i].oldValueList = (void*) allocValueList(1);
+
+    nonlinsys[i].lastTimeSolved = 0.0;
 
     nonlinsys[i].nominal = (double*) malloc(size*sizeof(double));
     nonlinsys[i].min = (double*) malloc(size*sizeof(double));
@@ -360,8 +380,9 @@ int initializeNonlinearSystems(DATA *data, threadData_t *threadData)
 
     nonlinsys[i].initializeStaticNLSData(data, threadData, &nonlinsys[i]);
 
+#if !defined(OMC_MINIMAL_RUNTIME)
     /* csv data call stats*/
-    if (data->simulationInfo.nlsCsvInfomation)
+    if (data->simulationInfo->nlsCsvInfomation)
     {
       if (initializeNLScsvData(data, &nonlinsys[i]))
       {
@@ -373,9 +394,9 @@ int initializeNonlinearSystems(DATA *data, threadData_t *threadData)
         print_csvLineIterStatsHeader(data, &nonlinsys[i], ((struct csvStats*) nonlinsys[i].csvData)->iterStats);
       }
     }
-
+#endif
     /* allocate solver data */
-    switch(data->simulationInfo.nlsMethod)
+    switch(data->simulationInfo->nlsMethod)
     {
 #if !defined(OMC_MINIMAL_RUNTIME)
     case NLS_HYBRID:
@@ -424,12 +445,12 @@ int updateStaticDataOfNonlinearSystems(DATA *data, threadData_t *threadData)
   TRACE_PUSH
   int i;
   int size;
-  NONLINEAR_SYSTEM_DATA *nonlinsys = data->simulationInfo.nonlinearSystemData;
+  NONLINEAR_SYSTEM_DATA *nonlinsys = data->simulationInfo->nonlinearSystemData;
   struct dataNewtonAndHybrid *mixedSolverData;
 
   infoStreamPrint(LOG_NLS, 1, "update static data of non-linear system solvers");
 
-  for(i=0; i<data->modelData.nNonLinearSystems; ++i)
+  for(i=0; i<data->modelData->nNonLinearSystems; ++i)
   {
     nonlinsys[i].initializeStaticNLSData(data, threadData, &nonlinsys[i]);
   }
@@ -450,29 +471,32 @@ int freeNonlinearSystems(DATA *data, threadData_t *threadData)
 {
   TRACE_PUSH
   int i;
-  NONLINEAR_SYSTEM_DATA* nonlinsys = data->simulationInfo.nonlinearSystemData;
+  NONLINEAR_SYSTEM_DATA* nonlinsys = data->simulationInfo->nonlinearSystemData;
   struct csvStats* stats;
 
   infoStreamPrint(LOG_NLS, 1, "free non-linear system solvers");
 
-  for(i=0; i<data->modelData.nNonLinearSystems; ++i)
+  for(i=0; i<data->modelData->nNonLinearSystems; ++i)
   {
     free(nonlinsys[i].nlsx);
     free(nonlinsys[i].nlsxExtrapolation);
     free(nonlinsys[i].nlsxOld);
+    free(nonlinsys[i].resValues);
     free(nonlinsys[i].nominal);
     free(nonlinsys[i].min);
     free(nonlinsys[i].max);
+    freeValueList(nonlinsys[i].oldValueList, 1);
 
-    if (data->simulationInfo.nlsCsvInfomation)
+#if !defined(OMC_MINIMAL_RUNTIME)
+    if (data->simulationInfo->nlsCsvInfomation)
     {
       stats = nonlinsys[i].csvData;
       omc_write_csv_free(stats->callStats);
       omc_write_csv_free(stats->iterStats);
     }
-
+#endif
     /* free solver data */
-    switch(data->simulationInfo.nlsMethod)
+    switch(data->simulationInfo->nlsMethod)
     {
 #if !defined(OMC_MINIMAL_RUNTIME)
     case NLS_HYBRID:
@@ -506,16 +530,16 @@ int freeNonlinearSystems(DATA *data, threadData_t *threadData)
   return 0;
 }
 
-/*! \fn int printNonLinearSystemSolvingStatistics(DATA *data)
+/*! \fn printNonLinearSystemSolvingStatistics
  *
- *  This function print memory for all non-linear systems.
+ *  This function prints memory for all non-linear systems.
  *
  *  \param [ref] [data]
  *         [in]  [sysNumber] index of corresponding non-linear system
  */
 void printNonLinearSystemSolvingStatistics(DATA *data, int sysNumber, int logLevel)
 {
-  NONLINEAR_SYSTEM_DATA* nonlinsys = data->simulationInfo.nonlinearSystemData;
+  NONLINEAR_SYSTEM_DATA* nonlinsys = data->simulationInfo->nonlinearSystemData;
   infoStreamPrint(logLevel, 1, "Non-linear system %d of size %d solver statistics:", (int)nonlinsys[sysNumber].equationIndex, (int)nonlinsys[sysNumber].size);
   infoStreamPrint(logLevel, 0, " number of calls                : %ld", nonlinsys[sysNumber].numberOfCall);
   infoStreamPrint(logLevel, 0, " number of iterations           : %ld", nonlinsys[sysNumber].numberOfIterations);
@@ -525,6 +549,123 @@ void printNonLinearSystemSolvingStatistics(DATA *data, int sysNumber, int logLev
   messageClose(logLevel);
 }
 
+/*! \fn getInitialGuess
+ *
+ *  This function writes initial guess to nonlinsys->nlsx and nonlinsys->nlsOld.
+ *
+ *  \param [in]  [nonlinsys]
+ *  \param [in]  [time] time for extrapolation
+ *
+ */
+int getInitialGuess(NONLINEAR_SYSTEM_DATA *nonlinsys, double time)
+{
+  /* value extrapolation */
+  printValuesListTimes((VALUES_LIST*)nonlinsys->oldValueList);
+  /* if list is empty use current start values */
+  if (listLen(((VALUES_LIST*)nonlinsys->oldValueList)->valueList)==0)
+  {
+    /* use old value if no values are stored in the list */
+    memcpy(nonlinsys->nlsx, nonlinsys->nlsxOld, nonlinsys->size*(sizeof(double)));
+  }
+  else
+  {
+    /* get extrapolated values */
+    getValues((VALUES_LIST*)nonlinsys->oldValueList, time, nonlinsys->nlsxExtrapolation, nonlinsys->nlsxOld);
+    memcpy(nonlinsys->nlsx, nonlinsys->nlsxOld, nonlinsys->size*(sizeof(double)));
+  }
+
+  return 0;
+}
+
+/*! \fn updateInitialGuessDB
+ *
+ *  This function writes new values to solution list.
+ *
+ *  \param [in]  [nonlinsys]
+ *  \param [in]  [time] time for extrapolation
+ *  \param [in]  [context] current context of evaluation
+ *
+ */
+int updateInitialGuessDB(NONLINEAR_SYSTEM_DATA *nonlinsys, double time, int context)
+{
+  /* write solution to oldValue list for extrapolation */
+  if (nonlinsys->solved == 1)
+  {
+    /* do not use solution of jacobian for next extrapolation */
+    if (context < 4)
+    {
+      addListElement((VALUES_LIST*)nonlinsys->oldValueList,
+              createValueElement(nonlinsys->size, time, nonlinsys->nlsx));
+    }
+  }
+  else if (nonlinsys->solved == 2)
+  {
+    if (listLen(((VALUES_LIST*)nonlinsys->oldValueList)->valueList)>0)
+    {
+      cleanValueList((VALUES_LIST*)nonlinsys->oldValueList, NULL);
+    }
+    /* do not use solution of jacobian for next extrapolation */
+    if (context < 4)
+    {
+      addListElement((VALUES_LIST*)nonlinsys->oldValueList,
+              createValueElement(nonlinsys->size, time, nonlinsys->nlsx));
+    }
+  }
+  messageClose(LOG_NLS_EXTRAPOLATE);
+  return 0;
+}
+
+/*! \fn updateInnerEquation
+ *
+ *  This function updates inner equation with the current x.
+ *
+ *  \param [ref] [data]
+ *         [in]  [sysNumber] index of corresponding non-linear system
+ */
+int updateInnerEquation(void **dataIn, int sysNumber, int discrete)
+{
+  DATA *data = (DATA*) ((void**)dataIn[0]);
+  threadData_t *threadData = (threadData_t*) ((void**)dataIn[1]);
+
+  NONLINEAR_SYSTEM_DATA* nonlinsys = &(data->simulationInfo->nonlinearSystemData[sysNumber]);
+  int success = 0;
+
+  /* solve non continuous at discrete points*/
+  if(discrete)
+  {
+    data->simulationInfo->solveContinuous = 0;
+  }
+
+  /* try */
+#ifndef OMC_EMCC
+  MMC_TRY_INTERNAL(simulationJumpBuffer)
+#endif
+
+  /* call residual function */
+  nonlinsys->residualFunc((void*) dataIn, nonlinsys->nlsx, nonlinsys->resValues, (int*)&nonlinsys->size);
+
+  /* replace extrapolated values by current x for discrete step */
+  memcpy(nonlinsys->nlsxExtrapolation, nonlinsys->nlsx, nonlinsys->size*(sizeof(double)));
+
+  success = 1;
+  /*catch */
+#ifndef OMC_EMCC
+  MMC_CATCH_INTERNAL(simulationJumpBuffer)
+#endif
+
+  if (!success)
+  {
+    warningStreamPrint(LOG_STDOUT, 0, "Non-Linear Solver try to handle a problem with a called assert.");
+  }
+
+  if(discrete)
+  {
+    data->simulationInfo->solveContinuous = 1;
+  }
+
+  return success;
+}
+
 
 /*! \fn solve non-linear systems
  *
@@ -537,63 +678,52 @@ int solve_nonlinear_system(DATA *data, threadData_t *threadData, int sysNumber)
 {
   void *dataAndThreadData[2] = {data, threadData};
   int success = 0, saveJumpState;
-  NONLINEAR_SYSTEM_DATA* nonlinsys = &(data->simulationInfo.nonlinearSystemData[sysNumber]);
+  NONLINEAR_SYSTEM_DATA* nonlinsys = &(data->simulationInfo->nonlinearSystemData[sysNumber]);
   struct dataNewtonAndHybrid *mixedSolverData;
 
-  data->simulationInfo.currentNonlinearSystemIndex = sysNumber;
+  data->simulationInfo->currentNonlinearSystemIndex = sysNumber;
 
   /* enable to avoid division by zero */
-  data->simulationInfo.noThrowDivZero = 1;
-  ((DATA*)data)->simulationInfo.solveContinuous = 1;
-
+  data->simulationInfo->noThrowDivZero = 1;
+  ((DATA*)data)->simulationInfo->solveContinuous = 1;
 
+  /* performance measurement */
   rt_ext_tp_tick(&nonlinsys->totalTimeClock);
 
-  if(data->simulationInfo.discreteCall)
+  /* grab the initial guess */
+  infoStreamPrint(LOG_NLS_EXTRAPOLATE, 1, "############ Start new iteration for system %d at time at %g ############", nonlinsys->equationIndex, data->localData[0]->timeValue);
+  /* if last solving is too long ago use just old values  */
+  if (fabs(data->localData[0]->timeValue - nonlinsys->lastTimeSolved) < 5*data->simulationInfo->stepSize)
   {
-    double *fvec = malloc(sizeof(double)*nonlinsys->size);
-    int success = 0;
-
-#ifndef OMC_EMCC
-    /* try */
-    MMC_TRY_INTERNAL(simulationJumpBuffer)
-#endif
-
-    ((DATA*)data)->simulationInfo.solveContinuous = 0;
-    nonlinsys->residualFunc((void*) dataAndThreadData, nonlinsys->nlsx, fvec, (int*)&nonlinsys->size);
-    ((DATA*)data)->simulationInfo.solveContinuous = 1;
-
-    success = 1;
-#ifndef OMC_EMCC
-    /*catch */
-    MMC_CATCH_INTERNAL(simulationJumpBuffer)
-#endif
-    if (!success)
-    {
-      warningStreamPrint(LOG_STDOUT, 0, "Non-Linear Solver try to handle a problem with a called assert.");
-    }
-
-    free(fvec);
+    getInitialGuess(nonlinsys, data->localData[0]->timeValue);
+  }
+  else
+  {
+    nonlinsys->getIterationVars(data, nonlinsys->nlsx);
+    memcpy(nonlinsys->nlsx, nonlinsys->nlsxOld, nonlinsys->size*(sizeof(double)));
   }
 
-  /* strategy for solving nonlinear system
-   *
-   *
-   *
-   */
+  /* update non continuous */
+  if (data->simulationInfo->discreteCall)
+  {
+    updateInnerEquation(dataAndThreadData, sysNumber, 1);
+  }
+
+  /* try */
 #ifndef OMC_EMCC
-    /* try */
-    MMC_TRY_INTERNAL(simulationJumpBuffer)
+  MMC_TRY_INTERNAL(simulationJumpBuffer)
 #endif
 
-  switch(data->simulationInfo.nlsMethod)
+  /* handle asserts */
+  saveJumpState = threadData->currentErrorStage;
+  threadData->currentErrorStage = ERROR_NONLINEARSOLVER;
+
+  /* use the selected solver for solving nonlinear system */
+  switch(data->simulationInfo->nlsMethod)
   {
 #if !defined(OMC_MINIMAL_RUNTIME)
   case NLS_HYBRID:
-    saveJumpState = threadData->currentErrorStage;
-    threadData->currentErrorStage = ERROR_NONLINEARSOLVER;
     success = solveHybrd(data, threadData, sysNumber);
-    threadData->currentErrorStage = saveJumpState;
     break;
   case NLS_KINSOL:
     success = nonlinearSolve_kinsol(data, threadData, sysNumber);
@@ -609,48 +739,66 @@ int solve_nonlinear_system(DATA *data, threadData_t *threadData, int sysNumber)
     break;
 #endif
   case NLS_HOMOTOPY:
-    saveJumpState = threadData->currentErrorStage;
-    threadData->currentErrorStage = ERROR_NONLINEARSOLVER;
     success = solveHomotopy(data, threadData, sysNumber);
-    threadData->currentErrorStage = saveJumpState;
     break;
 #if !defined(OMC_MINIMAL_RUNTIME)
   case NLS_MIXED:
     mixedSolverData = nonlinsys->solverData;
     nonlinsys->solverData = mixedSolverData->newtonData;
 
-    saveJumpState = threadData->currentErrorStage;
-    threadData->currentErrorStage = ERROR_NONLINEARSOLVER;
-    success = solveHomotopy(data, threadData, sysNumber);
+   success = solveHomotopy(data, threadData, sysNumber);
+
+    /* check if solution process was successful, if not use alternative tearing set if available (dynamic tearing)*/
+    if (!success && nonlinsys->strictTearingFunctionCall != NULL){
+      debugString(LOG_DT, "Solving the casual tearing set failed! Now the strict tearing set is used.");
+      success = nonlinsys->strictTearingFunctionCall(data, threadData);
+      if (success){
+        success=2;
+        /* update iteration variables of the causal set*/
+        nonlinsys->getIterationVars(data, nonlinsys->nlsx);
+      }
+    }
+
     if (!success) {
       nonlinsys->solverData = mixedSolverData->hybridData;
       success = solveHybrd(data, threadData, sysNumber);
     }
-    threadData->currentErrorStage = saveJumpState;
     nonlinsys->solverData = mixedSolverData;
     break;
 #endif
   default:
     throwStreamPrint(threadData, "unrecognized nonlinear solver");
   }
-
-
+  /* set result */
   nonlinsys->solved = success;
 
+  /* handle asserts */
+  threadData->currentErrorStage = saveJumpState;
 
+  /*catch */
 #ifndef OMC_EMCC
-    /*catch */
-    MMC_CATCH_INTERNAL(simulationJumpBuffer)
+  MMC_CATCH_INTERNAL(simulationJumpBuffer)
 #endif
 
-  /* enable to avoid division by zero */
-  data->simulationInfo.noThrowDivZero = 0;
-  ((DATA*)data)->simulationInfo.solveContinuous = 0;
+  /* update value list database */
+  updateInitialGuessDB(nonlinsys, data->localData[0]->timeValue, data->simulationInfo->currentContext);
+  if (nonlinsys->solved == 1)
+  {
+    nonlinsys->lastTimeSolved = data->localData[0]->timeValue;
+  }
 
+
+  /* enable to avoid division by zero */
+  data->simulationInfo->noThrowDivZero = 0;
+  ((DATA*)data)->simulationInfo->solveContinuous = 0;
+
+  /* performance measurement and statistics */
   nonlinsys->totalTime += rt_ext_tp_tock(&(nonlinsys->totalTimeClock));
   nonlinsys->numberOfCall++;
 
-  if (data->simulationInfo.nlsCsvInfomation)
+  /* write csv file for debugging */
+#if !defined(OMC_MINIMAL_RUNTIME)
+  if (data->simulationInfo->nlsCsvInfomation)
   {
     print_csvLineCallStats(((struct csvStats*) nonlinsys->csvData)->callStats,
                            nonlinsys->numberOfCall,
@@ -661,7 +809,7 @@ int solve_nonlinear_system(DATA *data, threadData_t *threadData, int sysNumber)
                            nonlinsys->solved
     );
   }
-
+#endif
   return check_nonlinear_solution(data, 1, sysNumber);
 }
 
@@ -680,7 +828,7 @@ int check_nonlinear_solutions(DATA *data, int printFailingSystems)
 {
   long i;
 
-  for(i=0; i<data->modelData.nNonLinearSystems; ++i) {
+  for(i=0; i<data->modelData->nNonLinearSystems; ++i) {
      if(check_nonlinear_solution(data, printFailingSystems, i))
        return 1;
   }
@@ -690,19 +838,20 @@ int check_nonlinear_solutions(DATA *data, int printFailingSystems)
 
 /*! \fn check_nonlinear_solution
  *
- *   This function check whether one non-linear system
- *   is to solve. If one is failed it returns 1 otherwise 0.
+ *   This function checks if a non-linear system
+ *   is solved. Returns a warning and 1 in case it's not
+ *   solved otherwise 0.
  *
  *  \param [in]  [data]
  *  \param [in]  [printFailingSystems]
  *  \param [in]  [sysNumber] index of corresponding non-linear System
- *  \param [out] [returnValue] It returns 1 if fail otherwise 0.
+ *  \param [out] [returnValue] Returns 1 if fail otherwise 0.
  *
  *  \author wbraun
  */
 int check_nonlinear_solution(DATA *data, int printFailingSystems, int sysNumber)
 {
-  NONLINEAR_SYSTEM_DATA* nonlinsys = data->simulationInfo.nonlinearSystemData;
+  NONLINEAR_SYSTEM_DATA* nonlinsys = data->simulationInfo->nonlinearSystemData;
   long j;
   int i = sysNumber;
 
@@ -711,17 +860,17 @@ int check_nonlinear_solution(DATA *data, int printFailingSystems, int sysNumber)
     int index = nonlinsys[i].equationIndex, indexes[2] = {1,index};
     if (!printFailingSystems) return 1;
     warningStreamPrintWithEquationIndexes(LOG_NLS, 1, indexes, "nonlinear system %d fails: at t=%g", index, data->localData[0]->timeValue);
-    if(data->simulationInfo.initial)
+    if(data->simulationInfo->initial)
     {
       warningStreamPrint(LOG_NLS, 0, "proper start-values for some of the following iteration variables might help");
     }
-    for(j=0; j<modelInfoGetEquation(&data->modelData.modelDataXml, (nonlinsys[i]).equationIndex).numVar; ++j) {
+    for(j=0; j<modelInfoGetEquation(&data->modelData->modelDataXml, (nonlinsys[i]).equationIndex).numVar; ++j) {
       int done=0;
       long k;
-      const MODEL_DATA *mData = &(data->modelData);
+      const MODEL_DATA *mData = data->modelData;
       for(k=0; k<mData->nVariablesReal && !done; ++k)
       {
-        if (!strcmp(mData->realVarsData[k].info.name, modelInfoGetEquation(&data->modelData.modelDataXml, (nonlinsys[i]).equationIndex).vars[j]))
+        if (!strcmp(mData->realVarsData[k].info.name, modelInfoGetEquation(&data->modelData->modelDataXml, (nonlinsys[i]).equationIndex).vars[j]))
         {
         done = 1;
         warningStreamPrint(LOG_NLS, 0, "[%ld] Real %s(start=%g, nominal=%g)", j+1,
@@ -732,7 +881,7 @@ int check_nonlinear_solution(DATA *data, int printFailingSystems, int sysNumber)
       }
       if (!done)
       {
-        warningStreamPrint(LOG_NLS, 0, "[%ld] Real %s(start=?, nominal=?)", j+1, modelInfoGetEquation(&data->modelData.modelDataXml, (nonlinsys[i]).equationIndex).vars[j]);
+        warningStreamPrint(LOG_NLS, 0, "[%ld] Real %s(start=?, nominal=?)", j+1, modelInfoGetEquation(&data->modelData->modelDataXml, (nonlinsys[i]).equationIndex).vars[j]);
       }
     }
     messageCloseWarning(LOG_NLS);
@@ -748,30 +897,23 @@ int check_nonlinear_solution(DATA *data, int printFailingSystems, int sysNumber)
   return 0;
 }
 
-/*! \fn extraPolate
- *   This function extrapolates linear next value from
- *   the both old values.
+/*! \fn cleanUpOldValueListAfterEvent
+ *
+ *   This function clean old value list up to parameter time for all
+ *   non-linear systems.
  *
  *  \param [in]  [data]
+ *  \param [in]  [time]
  *
  *  \author wbraun
  */
-double extraPolate(DATA *data, const double old1, const double old2, const double minValue, const double maxValue)
+void cleanUpOldValueListAfterEvent(DATA *data, double time)
 {
-  double retValue;
+  long i;
+  NONLINEAR_SYSTEM_DATA* nonlinsys = data->simulationInfo->nonlinearSystemData;
 
-  if(data->localData[1]->timeValue == data->localData[2]->timeValue || old1 == old2)
-  {
-    retValue = old1;
+  for(i=0; i<data->modelData->nNonLinearSystems; ++i) {
+    cleanValueListbyTime(nonlinsys[i].oldValueList, time);
   }
-  else
-  {
-    retValue = old2 + ((data->localData[0]->timeValue - data->localData[2]->timeValue)/(data->localData[1]->timeValue - data->localData[2]->timeValue)) * (old1-old2);
-
-    if(retValue < minValue && old1 > minValue)  retValue = minValue;
-    else if(retValue > maxValue && old1 < maxValue ) retValue = maxValue;
-
-  }
-
-  return retValue;
 }
+
