@@ -30,7 +30,7 @@
 
 
 #include "boolean_array.h"
-#include "memory_pool.h"
+#include "gc/omc_gc.h"
 
 #include <stdio.h>
 #include <stdlib.h>
