@@ -37,50 +37,75 @@ const char *FLAG_NAME[FLAG_MAX+1] = {
   /* FLAG_ALARM */                 "alarm",
   /* FLAG_CLOCK */                 "clock",
   /* FLAG_CPU */                   "cpu",
-  /* FLAG_DASSL_JACOBIAN */        "dasslJacobian",
-  /* FLAG_DASSL_NO_ROOTFINDING */  "dasslnoRootFinding",
-  /* FLAG_DASSL_NO_RESTART */      "dasslnoRestart",
+  /* FLAG_CSV_OSTEP */             "csvOstep",
+  /* FLAG_DAE_MODE */              "daeMode",
+  /* FLAG_DELTA_X_LINEARIZE */     "deltaXLinearize",
+  /* FLAG_DELTA_X_SOLVER */        "deltaXSolver",
+  /* FLAG_EMBEDDED_SERVER */       "embeddedServer",
   /* FLAG_EMIT_PROTECTED */        "emit_protected",
   /* FLAG_F */                     "f",
   /* FLAG_HELP */                  "help",
+  /* FLAG_IDA_MAXERRORTESTFAIL */  "idaMaxErrorTestFails",
+  /* FLAG_IDA_MAXNONLINITERS */    "idaMaxNonLinIters",
+  /* FLAG_IDA_MAXCONVFAILS */      "idaMaxConvFails",
+  /* FLAG_IDA_NONLINCONVCOEF */    "idaNonLinConvCoef",
+  /* FLAG_IDA_LS */                "idaLS",
+  /* FLAG_IDAS */                  "idaSensitivity",
+  /* FLAG_IGNORE_HIDERESULT */     "ignoreHideResult",
   /* FLAG_IIF */                   "iif",
   /* FLAG_IIM */                   "iim",
   /* FLAG_IIT */                   "iit",
   /* FLAG_ILS */                   "ils",
   /* FLAG_INITIAL_STEP_SIZE */     "initialStepSize",
+  /* FLAG_INPUT_CSV */             "csvInput",
   /* FLAG_INPUT_FILE */            "exInputFile",
   /* FLAG_INPUT_FILE_STATES */     "stateFile",
   /* FLAG_IPOPT_HESSE*/            "ipopt_hesse",
-  /* FLAG_IPOPT_JAC*/              "ipopt_jac",
   /* FLAG_IPOPT_INIT*/             "ipopt_init",
+  /* FLAG_IPOPT_JAC*/              "ipopt_jac",
   /* FLAG_IPOPT_MAX_ITER */        "ipopt_max_iter",
   /* FLAG_IPOPT_WARM_START */      "ipopt_warm_start",
+  /* FLAG_JACOBIAN */              "jacobian",
   /* FLAG_L */                     "l",
+  /* FLAG_L_DATA_RECOVERY */       "l_datarec",
   /* FLAG_LOG_FORMAT */            "logFormat",
   /* FLAG_LS */                    "ls",
   /* FLAG_LS_IPOPT */              "ls_ipopt",
+  /* FLAG_LSS */                   "lss",
+  /* FLAG_LSS_MAX_DENSITY */       "lssMaxDensity",
+  /* FLAG_LSS_MIN_SIZE */          "lssMinSize",
   /* FLAG_LV */                    "lv",
-  /* FLAG_MAX_STEP_SIZE */         "maxStepSize",
+  /* FLAG_MAX_BISECTION_ITERATIONS */  "mbi",
+  /* FLAG_MAX_EVENT_ITERATIONS */  "mei",
   /* FLAG_MAX_ORDER */             "maxIntegrationOrder",
+  /* FLAG_MAX_STEP_SIZE */         "maxStepSize",
   /* FLAG_MEASURETIMEPLOTFORMAT */ "measureTimePlotFormat",
+  /* FLAG_NEWTON_FTOL */           "newtonFTol",
+  /* FLAG_NEWTON_XTOL */           "newtonXTol",
   /* FLAG_NEWTON_STRATEGY */       "newton",
   /* FLAG_NLS */                   "nls",
   /* FLAG_NLS_INFO */              "nlsInfo",
+  /* FLAG_NLS_LS */                "nlsLS",
   /* FLAG_NOEMIT */                "noemit",
   /* FLAG_NOEQUIDISTANT_GRID */    "noEquidistantTimeGrid",
   /* FLAG_NOEQUIDISTANT_OUT_FREQ*/ "noEquidistantOutputFrequency",
   /* FLAG_NOEQUIDISTANT_OUT_TIME*/ "noEquidistantOutputTime",
   /* FLAG_NOEVENTEMIT */           "noEventEmit",
-  /* FLAG_OPTDEBUGEJAC */          "optDebugeJac",
+  /* FLAG_NO_RESTART */            "noRestart",
+  /* FLAG_NO_ROOTFINDING */        "noRootFinding",
+  /* FLAG_NO_SUPPRESS_ALG */       "noSuppressAlg",
+  /* FLAG_OPTDEBUGEJAC */          "optDebugJac",
+  /* FLAG_OPTIMIZER_NP */          "optimizerNP",
+  /* FLAG_OPTIMIZER_TGRID */       "optimizerTimeGrid",
   /* FLAG_OUTPUT */                "output",
   /* FLAG_OVERRIDE */              "override",
   /* FLAG_OVERRIDE_FILE */         "overrideFile",
-  /* FLAG_OPTIMIZER_NP */          "optimizerNP",
-  /* FLAG_OPTIMIZER_TGRID */       "optimizerTimeGrid",
-  /* FLAG_UP_HESSIAN */            "keepHessian",
   /* FLAG_PORT */                  "port",
   /* FLAG_R */                     "r",
+  /* FLAG_RT */                    "rt",
   /* FLAG_S */                     "s",
+  /* FLAG_SOLVER_STEPS */          "steps",
+  /* FLAG_UP_HESSIAN */            "keepHessian",
   /* FLAG_W */                     "w",
 
   "FLAG_MAX"
@@ -92,51 +117,76 @@ const char *FLAG_DESC[FLAG_MAX+1] = {
   /* FLAG_ABORT_SLOW */            "aborts if the simulation chatters",
   /* FLAG_ALARM */                 "aborts after the given number of seconds (0 disables)",
   /* FLAG_CLOCK */                 "selects the type of clock to use -clock=RT, -clock=CYC or -clock=CPU",
-  /* FLAG_CPU */                   "dumps the cpu-time into the results-file",
-  /* FLAG_DASSL_JACOBIAN */        "selects the type of the jacobians that is used for the dassl solver.\n  dasslJacobian=[coloredNumerical (default) |numerical|internalNumerical|coloredSymbolical|symbolical].",
-  /* FLAG_DASSL_NO_ROOTFINDING */  "flag deactivates the internal root finding procedure of dassl.",
-  /* FLAG_DASSL_NO_RESTART */      "flag deactivates the restart of dassl after an event is performed.",
+  /* FLAG_CPU */                   "dumps the cpu-time into the result file",
+  /* FLAG_CSV_OSTEP */             "value specifies csv-files for debug values for optimizer step",
+  /* FLAG_DAE_MODE */              "flag to let the integrator use daeResiduals",
+  /* FLAG_DELTA_X_LINEARIZE */     "value specifies the delta x value for numerical differentiation used by linearization. The default value is 1e-5.",
+  /* FLAG_DELTA_X_SOLVER */        "value specifies the delta x value for numerical differentiation used by integrator. The default values is sqrt(DBL_EPSILON).",
+  /* FLAG_EMBEDDED_SERVER */       "enables an embedded server. Valid values: none, opc-da [broken], opc-ua [experimental], or the path to a shared object.",
   /* FLAG_EMIT_PROTECTED */        "emits protected variables to the result-file",
   /* FLAG_F */                     "value specifies a new setup XML file to the generated simulation code",
   /* FLAG_HELP */                  "get detailed information that specifies the command-line flag",
+  /* FLAG_IDA_MAXERRORTESTFAIL */  "value specifies the maximum number of error test failures in attempting one step. The default value is 7.",
+  /* FLAG_IDA_MAXNONLINITERS */    "value specifies the maximum number of nonlinear solver iterations at one step. The default value is 3.",
+  /* FLAG_IDA_MAXCONVFAILS */      "value specifies the maximum number of nonlinear solver convergence failures at one step. The default value is 10.",
+  /* FLAG_IDA_NONLINCONVCOEF */    "value specifies the safety factor in the nonlinear convergence test. The default value is 0.33.",
+  /* FLAG_IDA_LS */                "select the linear solver used by ida",
+  /* FLAG_IDAS */                  "flag to add sensitivity information to the result files",
+  /* FLAG_IGNORE_HIDERESULT */     "ignore HideResult=true annotation",
   /* FLAG_IIF */                   "value specifies an external file for the initialization of the model",
   /* FLAG_IIM */                   "value specifies the initialization method",
   /* FLAG_IIT */                   "[double] value specifies a time for the initialization of the model",
   /* FLAG_ILS */                   "[int] default: 1",
-  /* FLAG_INITIAL_STEP_SIZE */     "value specifies an initial stepsize for the dassl solver",
+  /* FLAG_INITIAL_STEP_SIZE */     "value specifies an initial step size for supported solver",
+  /* FLAG_INPUT_CSV */             "value specifies an csv-file with inputs for the simulation/optimization of the model",
   /* FLAG_INPUT_FILE */            "value specifies an external file with inputs for the simulation/optimization of the model",
   /* FLAG_INPUT_FILE_STATES */     "value specifies an file with states start values for the optimization of the model",
   /* FLAG_IPOPT_HESSE */           "value specifies the hessian for Ipopt",
-  /* FLAG_IPOPT_JAC */             "value specifies the jacobian for Ipopt",
   /* FLAG_IPOPT_INIT */            "value specifies the initial guess for optimization",
+  /* FLAG_IPOPT_JAC */             "value specifies the Jacobian for Ipopt",
   /* FLAG_IPOPT_MAX_ITER */        "value specifies the max number of iteration for ipopt",
   /* FLAG_IPOPT_WARM_START */      "value specifies lvl for a warm start in ipopt: 1,2,3,...",
+  /* FLAG_JACOBIAN */              "select the calculation method for Jacobian if needed by the integrator. Valid values depend on the used method.",
   /* FLAG_L */                     "value specifies a time where the linearization of the model should be performed",
-  /* FLAG_LOG_FORMAT */            "value specifies the log format of the executable. -logFormat=text (default) or -logFormat=xml",
-  /* FLAG_LS */                    "value specifies the linear solver method",
+  /* FLAG_L_DATA_RECOVERY */       "emit data recovery matrices with model linearization",
+  /* FLAG_LOG_FORMAT */            "value specifies the log format of the executable. -logFormat=text (default), -logFormat=xml or -logFormat=xmltcp",
+  /* FLAG_LS */                    "value specifies the linear solver method (default: lapack, totalpivot (fallback))",
   /* FLAG_LS_IPOPT */              "value specifies the linear solver method for ipopt",
+  /* FLAG_LSS */                   "value specifies the linear sparse solver method (default: umfpack)",
+  /* FLAG_LSS_MAX_DENSITY */       "[double (default 0.2)] value specifies the maximum density for using a linear sparse solver",
+  /* FLAG_LSS_MIN_SIZE */          "[int (default 4001)] value specifies the minimum system size for using a linear sparse solver",
   /* FLAG_LV */                    "[string list] value specifies the logging level",
-  /* FLAG_MAX_STEP_SIZE */         "value specifies maximum absolute step size, used by dassl solver",
-  /* FLAG_MAX_ORDER */             "value specifies maximum integration order, used by dassl solver",
+  /* FLAG_MAX_BISECTION_ITERATIONS */  "[int (default 0)] value specifies the maximum number of bisection iterations for state event detection or zero for default behavior",
+  /* FLAG_MAX_EVENT_ITERATIONS */  "[int (default 20)] value specifies the maximum number of event iterations",
+  /* FLAG_MAX_ORDER */             "value specifies maximum integration order for supported solver",
+  /* FLAG_MAX_STEP_SIZE */         "value specifies maximum absolute step size for supported solver",
   /* FLAG_MEASURETIMEPLOTFORMAT */ "value specifies the output format of the measure time functionality",
+  /* FLAG_NEWTON_FTOL */           "[double (default 1e-12)] tolerance respecting residuals for updating solution vector in Newton solver",
+  /* FLAG_NEWTON_XTOL */           "[double (default 1e-12)] tolerance respecting newton correction (delta_x) for updating solution vector in Newton solver",
   /* FLAG_NEWTON_STRATEGY */       "value specifies the damping strategy for the newton solver",
   /* FLAG_NLS */                   "value specifies the nonlinear solver",
   /* FLAG_NLS_INFO */              "outputs detailed information about solving process of non-linear systems into csv files.",
+  /* FLAG_NLS_LS */                "value specifies the linear solver used by the non-linear solver",
   /* FLAG_NOEMIT */                "do not emit any results to the result file",
-  /* FLAG_NOEQUIDISTANT_GRID */    "stores results not in equidistant time grid as given by stepSize or numberOfIntervals, instead the variable step size of dassl is used.",
+  /* FLAG_NOEQUIDISTANT_GRID */    "stores results not in equidistant time grid as given by stepSize or numberOfIntervals, instead the variable step size of dassl or ida integrator.",
   /* FLAG_NOEQUIDISTANT_OUT_FREQ*/ "value controls the output frequency in noEquidistantTimeGrid mode",
   /* FLAG_NOEQUIDISTANT_OUT_TIME*/ "value controls the output time point in noEquidistantOutputTime mode",
   /* FLAG_NOEVENTEMIT */           "do not emit event points to the result file",
-  /* FLAG_OPTDEBUGEJAC */          "value specifies the number of iter from the dyn. optimization, which will be debuge, creating *csv and *py file",
+  /* FLAG_NO_RESTART */            "disables the restart of the integration method after an event is performed, used by the methods: dassl, ida",
+  /* FLAG_NO_ROOTFINDING */        "disables the internal root finding procedure of methods: dassl and ida.",
+  /* FLAG_NO_SUPPRESS_ALG */       "flag to not suppress algebraic variables in the local error test of ida solver in daeMode",
+  /* FLAG_OPTDEBUGEJAC */          "value specifies the number of iter from the dyn. optimization, which will be debug, creating *csv and *py file",
+  /* FLAG_OPTIMIZER_NP */          "value specifies the number of points in a subinterval",
+  /* FLAG_OPTIMIZER_TGRID */       "value specifies external file with time points.",
   /* FLAG_OUTPUT */                "output the variables a, b and c at the end of the simulation to the standard output",
   /* FLAG_OVERRIDE */              "override the variables or the simulation settings in the XML setup file",
   /* FLAG_OVERRIDE_FILE */         "will override the variables or the simulation settings in the XML setup file with the values from the file",
-  /* FLAG_OPTIMIZER_NP */          "value specifies the number of points in a subinterval",
-  /* FLAG_OPTIMIZER_TGRID */       "value specifies external file with time points.",
-  /* FLAG_UP_HESSIAN */            "value specifies the number of steps, which keep hessian matrix constant",
   /* FLAG_PORT */                  "value specifies the port for simulation status (default disabled)",
   /* FLAG_R */                     "value specifies a new result file than the default Model_res.mat",
-  /* FLAG_S */                     "value specifies the solver",
+  /* FLAG_RT */                    "value specifies the scaling factor for real-time synchronization (0 disables)",
+  /* FLAG_S */                     "value specifies the integration method",
+  /* FLAG_SOLVER_STEPS */          "dumps the number of integration steps into the result file",
+  /* FLAG_UP_HESSIAN */            "value specifies the number of steps, which keep hessian matrix constant",
   /* FLAG_W */                     "shows all warnings even if a related log-stream is inactive",
 
   "FLAG_MAX"
@@ -154,18 +204,21 @@ const char *FLAG_DETAILED_DESC[FLAG_MAX+1] = {
   "  * CYC (cpu cycles measured with RDTSC)\n"
   "  * CPU (process-based CPU-time)",
   /* FLAG_CPU */
-  "  Dumps the cpu-time into the result-file using the variable named $cpuTime",
-  /* FLAG_DASSL_JACOBIAN */
-  "  Selects the type of the Jacobian that is used for the dassl solver:\n\n"
-  "  * coloredNumerical (colored numerical Jacobian, the default).\n"
-  "  * internalNumerical (internal dassl numerical Jacobian).\n"
-  "  * coloredSymbolical (colored symbolical Jacobian. Only usable if the simulation is compiled with --generateSymbolicJacobian or --generateSymbolicLinearization.\n"
-  "  * numerical - numerical Jacobian.\n\n"
-  "  * symbolical - symbolical Jacobian. Only usable if the simulation is compiled with --generateSymbolicJacobian or --generateSymbolicLinearization.",
-  /* FLAG_DASSL_NO_ROOTFINDING */
-  "  Deactivates the internal root finding procedure of dassl.",
-  /* FLAG_DASSL_NO_RESTART */
-  "  Deactivates the restart of dassl after an event is performed.",
+  "  Dumps the cpu-time into the result file using the variable named $cpuTime",
+  /* FLAG_CSV_OSTEP */
+  "  Value specifies csv-files for debug values for optimizer step",
+  /* FLAG_DAE_MODE */
+  "  Enables daeMode simulation if the model was compiled with the omc flag --daeMode and ida method is used.",
+  /* FLAG_DELTA_X_LINEARIZE */
+  "value specifies the delta x value for numerical differentiation used by linearization. The default value is sqrt(DBL_EPSILON*2e1).",
+  /* FLAG_DELTA_X_SOLVER */
+  "value specifies the delta x value for numerical differentiation used by integration method. The default values is sqrt(DBL_EPSILON).",
+  /* FLAG_EMBEDDED_SERVER */
+  "  Enables an embedded server. Valid values:\n\n"
+  "  * none - default, run without embedded server\n"
+  "  * opc-da - [broken] run with embedded OPC DA server (WIN32 only, uses proprietary OPC SC interface)\n"
+  "  * opc-ua - [experimental] run with embedded OPC UA server (TCP port 4841 for now; will have its own configuration option later)\n"
+  "  * filename - path to a shared object implementing the embedded server interface (requires access to internal OMC data-structures if you want to read or write data)",
   /* FLAG_EMIT_PROTECTED */
   "  Emits protected variables to the result-file.",
   /* FLAG_F */
@@ -173,49 +226,95 @@ const char *FLAG_DETAILED_DESC[FLAG_MAX+1] = {
   /* FLAG_HELP */
   "  Get detailed information that specifies the command-line flag\n"
   "  For example, -help=f prints detailed information for command-line flag f.",
+  /* FLAG_IDA_MAXERRORTESTFAIL */
+  "  value specifies the maximum number of error test failures in attempting one step. The default value is 7.",
+  /* FLAG_IDA_MAXNONLINITERS */
+  "  value specifies the maximum number of nonlinear solver iterations at one step. The default value is 3.",
+  /* FLAG_IDA_MAXCONVFAILS */
+  "  value specifies the maximum number of nonlinear solver convergence failures at one step. The default value is 10.",
+  /* FLAG_IDA_NONLINCONVCOEF */
+  "  value specifies the safety factor in the nonlinear convergence test. The default value is 0.33.",
+  /* FLAG_IDA_LS */
+  "  Value specifies the linear solver of the ida integration method. Valid values:\n"
+  "  * klu - default, fast sparse linear solver\n"
+  "  * dense - dense linear solver, sundials default method\n"
+  "  * spgmr - sparse iterative linear solver based on generalized minimal residual method, convergance is not guaranteed, sundials method\n"
+  "  * spbcg - sparse iterative linear solver based on biconjugate gradient method, convergance is not guaranteed, sundials method\n"
+  "  * spgmr - sparse iterative linear solver based on transpose free quasi-minimal residual method, convergance is not guaranteed, sundials method",
+  /* FLAG_IDAS */
+  "  Enables sensitivity analysis with respect to parameters if the model is compiled with omc flag --calculateSensitivities.",
+  /* FLAG_IGNORE_HIDERESULT */
+  "  Emits also variables with HideResult=true annotation.",
   /* FLAG_IIF */
   "  Value specifies an external file for the initialization of the model.",
   /* FLAG_IIM */
-  "  Value specifies the initialization method.", /* TODO: Fill me in */
+  "  Value specifies the initialization method.\n  Following options are available: 'symbolic' (default) and 'none'.",
   /* FLAG_IIT */
   "  Value [Real] specifies a time for the initialization of the model.",
   /* FLAG_ILS */
-  "  Value specifies the number of steps for homotopy method (required: -iim=symbolic) or 'start value homotopy' method (required: -iim=numeric -iom=nelder_mead_ex).\n"
+  "  Value specifies the number of steps for homotopy method (required: -iim=symbolic).\n"
   "  The value is an Integer with default value 1.",
   /* FLAG_INITIAL_STEP_SIZE */
-  "  Value specifies an initial stepsize for the dassl solver.",
+  "  Value specifies an initial step size, used by the methods: dassl, ida",
+   /* FLAG_INPUT_CSV */
+  "  Value specifies an csv-file with inputs for the simulation/optimization of the model",
   /* FLAG_INPUT_FILE */
   "  Value specifies an external file with inputs for the simulation/optimization of the model.",
   /* FLAG_INPUT_FILE_STATES */
   "  Value specifies an file with states start values for the optimization of the model.",
   /* FLAG_IPOPT_HESSE */
   "  Value specifies the hessematrix for Ipopt(OMC, BFGS, const).",
-  /* FLAG_IPOPT_JAC */
-  "  Value specifies the jacobian for Ipopt(SYM, NUM, NUMDENSE).",
   /* FLAG_IPOPT_INIT */
   "  Value specifies the initial guess for optimization (sim, const).",
+  /* FLAG_IPOPT_JAC */
+  "  Value specifies the Jacobian for Ipopt(SYM, NUM, NUMDENSE).",
   /* FLAG_IPOPT_MAX_ITER */
   "  Value specifies the max number of iteration for ipopt.",
   /* FLAG_IPOPT_WARM_START */
   "  Value specifies lvl for a warm start in ipopt: 1,2,3,...",
+  /* FLAG_JACOBIAN */
+  "  Select the calculation method for Jacobian used by the integration method:\n"
+  "  * coloredNumerical - A dense colored numerical Jacobian, which is default for dassl. Usable with dassl and ida.\n"
+  "  * internalNumerical - A dense internal numerical Jacobian. Usable with dassl and ida.\n"
+  "  * coloredSymbolical - A dense colored symbolical Jacobian. Needs omc compiler flag --postOptModules+=generateSymbolicJacobian. Usable with dassl and ida.\n"
+  "  * numerical - A dense numerical Jacobian. Usable with dassl and ida.\n"
+  "  * symbolical - A dense symbolical Jacobian. Needs omc compiler flag --postOptModules+=generateSymbolicJacobian. Usable with dassl and ida.\n"
+  "  * kluSparse - A sparse colored numerical Jacobian, which is default for ida. Usable with ida.",
   /* FLAG_L */
   "  Value specifies a time where the linearization of the model should be performed.",
+  /* FLAG_L_DATA_RECOVERY */
+  "  Emit data recovery matrices with model linearization.",
   /* FLAG_LOG_FORMAT */
   "  Value specifies the log format of the executable:\n\n"
   "  * text (default)\n"
-  "  * xml",
+  "  * xml\n"
+  "  * xmltcp (required -port flag)",
   /* FLAG_LS */
   "  Value specifies the linear solver method",
   /* FLAG_LS_IPOPT */
   "  Value specifies the linear solver method for Ipopt, default mumps.\n"
   "  Note: Use if you build ipopt with other linear solver like ma27",
+  /* FLAG_LSS */
+  "  Value specifies the linear sparse solver method",
+  /* FLAG_LSS_MAX_DENSITY */
+  "  Value specifies the maximum density for using a linear sparse solver.\n"
+  "  The value is a Double with default value 0.2.",
+  /* FLAG_LSS_MIN_SIZE */
+  "  Value specifies the minimum system size for using a linear sparse solver.\n"
+  "  The value is an Integer with default value 4001.",
   /* FLAG_LV */
   "  Value (a comma-separated String list) specifies which logging levels to\n"
   "  enable. Multiple options can be enabled at the same time.",
-  /* FLAG_MAX_STEP_SIZE */
-  "  Value specifies maximum absolute step size, used by dassl solver.",
+  /* FLAG_MAX_BISECTION_ITERATIONS */
+  "  value specifies the maximum number of bisection iterations for state event\n"
+  "  detection or zero for default behavior",
+  /* FLAG_MAX_EVENT_ITERATIONS */
+  "  Value specifies the maximum number of event iterations.\n"
+  "  The value is an Integer with default value 20.",
   /* FLAG_MAX_ORDER */
-  "  Value specifies maximum integration order, used by dassl solver.",
+  "  Value specifies maximum integration order, used by the methods: dassl, ida.",
+  /* FLAG_MAX_STEP_SIZE */
+  "  Value specifies maximum absolute step size, used by the methods: dassl, ida.",
   /* FLAG_MEASURETIMEPLOTFORMAT */
   "  Value specifies the output format of the measure time functionality\n\n"
   "  * svg\n"
@@ -223,6 +322,14 @@ const char *FLAG_DETAILED_DESC[FLAG_MAX+1] = {
   "  * ps\n"
   "  * gif\n"
   "  * ...",
+  /* FLAG_NEWTON_FTOL */
+  "  Tolerance respecting residuals for updating solution vector in Newton solver."
+  "  Solution is accepted if the (scaled) 2-norm of the residuals is smaller than the tolerance newtonFTol and the (scaled) newton correction (delta_x) is smaller than the tolerance newtonXTol."
+  "  The value is a Double with default value 1e-12.",
+  /* FLAG_NEWTON_XTOL */
+  "  Tolerance respecting newton correction (delta_x) for updating solution vector in Newton solver."
+  "  Solution is accepted if the (scaled) 2-norm of the residuals is smaller than the tolerance newtonFTol and the (scaled) newton correction (delta_x) is smaller than the tolerance newtonXTol."
+  "  The value is a Double with default value 1e-12.",
   /* FLAG_NEWTON_STRATEGY */
   "  Value specifies the damping strategy for the newton solver.",
   /* FLAG_NLS */
@@ -233,10 +340,12 @@ const char *FLAG_DETAILED_DESC[FLAG_MAX+1] = {
   "  * mixed",
   /* FLAG_NLS_INFO */
   "  Outputs detailed information about solving process of non-linear systems into csv files.",
+  /* FLAG_NLS_LS */
+  "  Value specifies the linear solver used by the non-linear solver:\n  * totalpivot\n  * lapack (default)\n  * klu",
   /* FLAG_NOEMIT */
   "  Do not emit any results to the result file.",
   /* FLAG_NOEQUIDISTANT_GRID */
-  "  Output the internal steps given by dassl instead of interpolating results\n"
+  "  Output the internal steps given by dassl/ida instead of interpolating results\n"
   "  into an equidistant time grid as given by stepSize or numberOfIntervals.",
   /* FLAG_NOEQUIDISTANT_OUT_FREQ*/
   "  Integer value n controls the output frequency in noEquidistantTimeGrid mode\n"
@@ -246,9 +355,22 @@ const char *FLAG_DETAILED_DESC[FLAG_MAX+1] = {
   "  mode and outputs every time>=k*timeValue, where k is an integer",
   /* FLAG_NOEVENTEMIT */
   "  Do not emit event points to the result file.",
+  /* FLAG_NO_RESTART */
+  "  Disables the restart of the integration method after an event is performed, used by the methods: dassl, ida",
+  /* FLAG_NO_ROOTFINDING */
+  "  Disables the internal root finding procedure of methods: dassl and ida.",
+  /* FLAG_NO_SUPPRESS_ALG */
+  "  flag to not suppress algebraic variables in the local error test of the ida solver in daeMode.\n"
+  "  In general, the use of this option is discouraged when solving DAE systems of index 1,\n"
+  "  whereas it is generally encouraged for systems of index 2 or more.",
   /* FLAG_OPTDEBUGEJAC */
-  "  Value specifies the number of itereations from the dynamic optimization, which\n"
+  "  Value specifies the number of iterations from the dynamic optimization, which\n"
   "  will be debugged, creating .csv and .py files.",
+  /* FLAG_OPTIMIZER_NP */
+  "  Value specifies the number of points in a subinterval.\n"
+  "  Currently supports numbers 1 and 3.",
+  /* FLAG_OPTIMIZER_TGRID */
+  "  Value specifies external file with time points.",
   /* FLAG_OUTPUT */
   "  Output the variables a, b and c at the end of the simulation to the standard\n"
   "  output: time = value, a = value, b = value, c = value",
@@ -261,21 +383,21 @@ const char *FLAG_DETAILED_DESC[FLAG_MAX+1] = {
   "  Note that: -overrideFile CANNOT be used with -override.\n"
   "  Use when variables for -override are too many.\n"
   "  overrideFileName contains lines of the form: var1=start1",
-  /* FLAG_OPTIMIZER_NP */
-  "  Value specifies the number of points in a subinterval.\n"
-  "  Currently supports numbers 1 and 3.",
-  /* FLAG_OPTIMIZER_TGRID */
-  "  Value specifies external file with time points.",
-  /* FLAG_UP_HESSIAN */
-  "  Value specifies the number of steps, which keep hessian matrix constant.",
   /* FLAG_PORT */
   "  Value specifies the port for simulation status (default disabled).",
   /* FLAG_R */
   "  Value specifies the name of the output result file.\n"
   "  The default file-name is based on the model name and output format.\n"
   "  For example: Model_res.mat.",
+  /* FLAG_RT */
+  "  Value specifies the scaling factor for real-time synchronization (0 disables).\n"
+  "  A value > 1 means the simulation takes a longer time to simulate.\n",
   /* FLAG_S */
-  "  Value specifies the solver (integration method).",
+  "  Value specifies the integration method.",
+  /* FLAG_SOLVER_STEPS */
+  "  dumps the number of integration steps into the result file",
+  /* FLAG_UP_HESSIAN */
+  "  Value specifies the number of steps, which keep Hessian matrix constant.",
   /* FLAG_W */
   "  Shows all warnings even if a related log-stream is inactive.",
 
@@ -289,50 +411,75 @@ const int FLAG_TYPE[FLAG_MAX] = {
   /* FLAG_ALARM */                 FLAG_TYPE_OPTION,
   /* FLAG_CLOCK */                 FLAG_TYPE_OPTION,
   /* FLAG_CPU */                   FLAG_TYPE_FLAG,
-  /* FLAG_DASSL_JACOBIAN */        FLAG_TYPE_OPTION,
-  /* FLAG_DASSL_NO_ROOTFINDING */  FLAG_TYPE_FLAG,
-  /* FLAG_DASSL_NO_RESTART */      FLAG_TYPE_FLAG,
+  /* FLAG_CSV_OSTEP */             FLAG_TYPE_OPTION,
+  /* FLAG_DAE_SOLVING */           FLAG_TYPE_FLAG,
+  /* FLAG_DELTA_X_LINEARIZE */     FLAG_TYPE_OPTION,
+  /* FLAG_DELTA_X_SOLVER */        FLAG_TYPE_OPTION,
+  /* FLAG_EMBEDDED_SERVER */       FLAG_TYPE_OPTION,
   /* FLAG_EMIT_PROTECTED */        FLAG_TYPE_FLAG,
   /* FLAG_F */                     FLAG_TYPE_OPTION,
   /* FLAG_HELP */                  FLAG_TYPE_OPTION,
+  /* FLAG_IDA_MAXERRORTESTFAIL */  FLAG_TYPE_OPTION,
+  /* FLAG_IDA_MAXNONLINITERS */    FLAG_TYPE_OPTION,
+  /* FLAG_IDA_MAXCONVFAILS */      FLAG_TYPE_OPTION,
+  /* FLAG_IDA_NONLINCONVCOEF */    FLAG_TYPE_OPTION,
+  /* FLAG_IDA_LS */                FLAG_TYPE_OPTION,
+  /* FLAG_IDAS */                  FLAG_TYPE_FLAG,
+  /* FLAG_IGNORE_HIDERESULT */     FLAG_TYPE_FLAG,
   /* FLAG_IIF */                   FLAG_TYPE_OPTION,
   /* FLAG_IIM */                   FLAG_TYPE_OPTION,
   /* FLAG_IIT */                   FLAG_TYPE_OPTION,
   /* FLAG_ILS */                   FLAG_TYPE_OPTION,
   /* FLAG_INITIAL_STEP_SIZE */     FLAG_TYPE_OPTION,
+  /* FLAG_INPUT_CSV */             FLAG_TYPE_OPTION,
   /* FLAG_INPUT_FILE */            FLAG_TYPE_OPTION,
   /* FLAG_INPUT_FILE_STATES */     FLAG_TYPE_OPTION,
   /* FLAG_IPOPT_HESSE */           FLAG_TYPE_OPTION,
-  /* FLAG_IPOPT_JAC */             FLAG_TYPE_OPTION,
   /* FLAG_IPOPT_INIT */            FLAG_TYPE_OPTION,
+  /* FLAG_IPOPT_JAC */             FLAG_TYPE_OPTION,
   /* FLAG_IPOPT_MAX_ITER */        FLAG_TYPE_OPTION,
   /* FLAG_IPOPT_WARM_START */      FLAG_TYPE_OPTION,
+  /* FLAG_JACOBIAN */              FLAG_TYPE_OPTION,
   /* FLAG_L */                     FLAG_TYPE_OPTION,
+  /* FLAG_L_DATA_RECOVERY */       FLAG_TYPE_FLAG,
   /* FLAG_LOG_FORMAT */            FLAG_TYPE_OPTION,
   /* FLAG_LS */                    FLAG_TYPE_OPTION,
   /* FLAG_LS_IPOPT */              FLAG_TYPE_OPTION,
+  /* FLAG_LSS */                   FLAG_TYPE_OPTION,
+  /* FLAG_LSS_MAX_DENSITY */       FLAG_TYPE_OPTION,
+  /* FLAG_LSS_MIN_SIZE */          FLAG_TYPE_OPTION,
   /* FLAG_LV */                    FLAG_TYPE_OPTION,
-  /* FLAG_MAX_STEP_SIZE */         FLAG_TYPE_OPTION,
+  /* FLAG_MAX_BISECTION_ITERATIONS */  FLAG_TYPE_OPTION,
+  /* FLAG_MAX_EVENT_ITERATIONS */  FLAG_TYPE_OPTION,
   /* FLAG_MAX_ORDER */             FLAG_TYPE_OPTION,
+  /* FLAG_MAX_STEP_SIZE */         FLAG_TYPE_OPTION,
   /* FLAG_MEASURETIMEPLOTFORMAT */ FLAG_TYPE_OPTION,
+  /* FLAG_NEWTON_FTOL */           FLAG_TYPE_OPTION,
+  /* FLAG_NEWTON_XTOL */           FLAG_TYPE_OPTION,
   /* FLAG_NEWTON_STRATEGY */       FLAG_TYPE_OPTION,
   /* FLAG_NLS */                   FLAG_TYPE_OPTION,
   /* FLAG_NLS_INFO */              FLAG_TYPE_FLAG,
+  /* FLAG_NLS_LS */                FLAG_TYPE_OPTION,
   /* FLAG_NOEMIT */                FLAG_TYPE_FLAG,
   /* FLAG_NOEQUIDISTANT_GRID*/     FLAG_TYPE_FLAG,
   /* FLAG_NOEQUIDISTANT_OUT_FREQ*/ FLAG_TYPE_OPTION,
   /* FLAG_NOEQUIDISTANT_OUT_TIME*/ FLAG_TYPE_OPTION,
+  /* FLAG_NO_RESTART */            FLAG_TYPE_FLAG,
+  /* FLAG_NO_ROOTFINDING */        FLAG_TYPE_FLAG,
+  /* FLAG_NO_SUPPRESS_ALG */       FLAG_TYPE_FLAG,
   /* FLAG_NOEVENTEMIT */           FLAG_TYPE_FLAG,
   /* FLAG_OPTDEBUGEJAC */          FLAG_TYPE_OPTION,
+  /* FLAG_OPTIZER_NP */            FLAG_TYPE_OPTION,
+  /* FLAG_OPTIZER_TGRID */         FLAG_TYPE_OPTION,
   /* FLAG_OUTPUT */                FLAG_TYPE_OPTION,
   /* FLAG_OVERRIDE */              FLAG_TYPE_OPTION,
   /* FLAG_OVERRIDE_FILE */         FLAG_TYPE_OPTION,
-  /* FLAG_OPTIZER_NP */            FLAG_TYPE_OPTION,
-  /* FLAG_OPTIZER_TGRID */         FLAG_TYPE_OPTION,
-  /* FLAG_UP_HESSIAN */            FLAG_TYPE_OPTION,
   /* FLAG_PORT */                  FLAG_TYPE_OPTION,
   /* FLAG_R */                     FLAG_TYPE_OPTION,
+  /* FLAG_RT */                    FLAG_TYPE_OPTION,
   /* FLAG_S */                     FLAG_TYPE_OPTION,
+  /* FLAG_SOLVER_STEPS */          FLAG_TYPE_FLAG,
+  /* FLAG_UP_HESSIAN */            FLAG_TYPE_OPTION,
   /* FLAG_W */                     FLAG_TYPE_FLAG
 };
 
@@ -344,30 +491,36 @@ const char *SOLVER_METHOD_NAME[S_MAX] = {
   "optimization",
   "radau5",
   "radau3",
-  "radau1",
-  "lobatto2",
+  "impeuler",
+  "trapezoid",
   "lobatto4",
   "lobatto6",
   "symEuler",
   "symEulerSsc",
+  "heun",
+  "ida",
+  "rungekutta_ssc",
   "qss"
 };
 
 const char *SOLVER_METHOD_DESC[S_MAX] = {
   "unknown",
-  "euler",
-  "rungekutta",
-  "dassl with colored numerical jacobian, with interval root finding - default",
-  "optimization",
-  "radau5 [sundial/kinsol needed]",
-  "radau3 [sundial/kinsol needed]",
-  "radau1 [sundial/kinsol needed]",
-  "lobatto2 [sundial/kinsol needed]",
-  "lobatto4 [sundial/kinsol needed]",
-  "lobatto6 [sundial/kinsol needed]",
-  "symbolic implicit euler, [compiler flag +symEuler needed]",
-  "symbolic implicit euler with step-size control, [compiler flag +symEuler needed]",
-  "qss"
+  "euler - Explicit Euler (order 1)",
+  "rungekutta - Runge-Kutta (fixed step, order 4)",
+  "dassl - BDF solver with colored numerical Jacobian, with interval root finding - default",
+  "optimization - Special solver for dynamic optimization",
+  "radau5 - Radau IIA with 3 points, \"Implicit Runge-Kutta\", order 5 [sundial/kinsol needed]",
+  "radau3 - Radau IIA with 2 points, \"Implicit Runge-Kutta\", order 3 [sundial/kinsol needed]",
+  "impeuler - Implicit Euler (actually Radau IIA, order 1) [sundial/kinsol needed]",
+  "trapezoid - Trapezoidal rule (actually Lobatto IIA with 2 points) [sundial/kinsol needed]",
+  "lobatto4 - Lobatto IIA with 3 points, order 4 [sundial/kinsol needed]",
+  "lobatto6 - Lobatto IIA with 4 points, order 6 [sundial/kinsol needed]",
+  "symEuler - symbolic implicit euler, [compiler flag +symEuler needed]",
+  "symEulerSsc - symbolic implicit euler with step-size control, [compiler flag +symEuler needed]",
+  "heun - Heun's method (Runge-Kutta fixed step, order 2)",
+  "ida - Sundials ida solver",
+  "rungekutta_ssc - Runge-Kutta (with step size control, see. Novikov (2016), Solving Stiff Systems of ODEs...)",
+  "qss - A QSS solver [experimental]"
 };
 
 const char *INIT_METHOD_NAME[IIM_MAX] = {
@@ -389,6 +542,7 @@ const char *LS_NAME[LS_MAX+1] = {
 #if !defined(OMC_MINIMAL_RUNTIME)
   /* LS_LIS */          "lis",
 #endif
+  /* LS_KLU */          "klu",
   /* LS_UMFPACK */      "umfpack",
   /* LS_TOTALPIVOT */   "totalpivot",
   /* LS_DEFAULT */      "default",
@@ -403,6 +557,7 @@ const char *LS_DESC[LS_MAX+1] = {
 #if !defined(OMC_MINIMAL_RUNTIME)
   /* LS_LIS */          "method using iterativ solver Lis",
 #endif
+  /* LS_KLU */          "method using klu sparse linear solver",
   /* LS_UMFPACK */      "method using umfpack sparse linear solver",
   /* LS_TOTALPIVOT */   "method using a total pivoting LU factorization for underdetermination systems",
   /* LS_DEFAULT */      "default method - lapack with total pivoting as fallback",
@@ -410,6 +565,29 @@ const char *LS_DESC[LS_MAX+1] = {
   "LS_MAX"
 };
 
+const char *LSS_NAME[LS_MAX+1] = {
+  "LS_UNKNOWN",
+
+#if !defined(OMC_MINIMAL_RUNTIME)
+  /* LS_LIS */          "lis",
+#endif
+  /* LS_KLU */          "klu",
+  /* LS_UMFPACK */      "umfpack",
+
+  "LSS_MAX"
+};
+
+const char *LSS_DESC[LS_MAX+1] = {
+  "unknown",
+
+#if !defined(OMC_MINIMAL_RUNTIME)
+  /* LS_LIS */          "method using iterativ solver Lis",
+#endif
+  /* LS_KLU */          "method using klu sparse linear solver",
+  /* LS_UMFPACK */      "method using umfpack sparse linear solver",
+
+  "LSS_MAX"
+};
 
 const char *NLS_NAME[NLS_MAX+1] = {
   "NLS_UNKNOWN",
@@ -464,3 +642,73 @@ const char *NEWTONSTRATEGY_DESC[NEWTON_MAX+1] = {
 
   "NEWTON_MAX"
 };
+
+
+const char *JACOBIAN_METHOD[JAC_MAX+1] = {
+  "unknown",
+
+  "coloredNumerical",
+  "internalNumerical",
+  "coloredSymbolical",
+  "numerical",
+  "symbolical",
+  "kluSparse",
+
+  "JAC_MAX"
+};
+
+const char *JACOBIAN_METHOD_DESC[JAC_MAX+1] = {
+  "unknown",
+
+  "  * coloredNumerical - A dense colored numerical Jacobian, which is default for dassl. Usable with dassl and ida.",
+  "  * internalNumerical - A dense internal numerical Jacobian. Usable with dassl and ida.",
+  "  * coloredSymbolical - A dense colored symbolical Jacobian. Needs omc compiler flag --postOptModules+=generateSymbolicJacobian. Usable with dassl and ida.",
+  "  * numerical - A dense numerical Jacobian. Usable with dassl and ida.",
+  "  * symbolical - A dense symbolical Jacobian. Needs omc compiler flag --postOptModules+=generateSymbolicJacobian. Usable with dassl and ida.",
+  "  * kluSparse - A sparse colored numerical Jacobian, which is default for ida. Usable with ida.",
+
+  "JAC_MAX"
+ };
+
+const char *IDA_LS_METHOD[IDA_LS_MAX+1] = {
+  "unknown",
+
+  "dense",
+  "klu",
+  "spgmr",
+  "spbcg",
+  "sptfqmr",
+
+  "IDA_LS_MAX"
+};
+
+const char *IDA_LS_METHOD_DESC[IDA_LS_MAX+1] = {
+  "unknown",
+
+  "ida internal dense method",
+  "ida use sparse direct solver KLU",
+  "ida generalized minimal residual method. Iterativ method",
+  "ida Bi-CGStab. Iterativ method",
+  "ida TFQMR. Iterativ method",
+
+  "IDA_LS_MAX"
+};
+
+const char *NLS_LS_METHOD[NLS_LS_MAX+1] = {
+  "unknown",
+
+  "totalpivot",
+  "lapack",
+
+  "NLS_LS_MAX"
+};
+
+const char *NLS_LS_METHOD_DESC[NLS_LS_MAX+1] = {
+  "unknown",
+
+  "internal total pivot implementation. Solve in some case even under-determined systems.",
+  "use external lapack implementation.",
+
+  "NLS_LS_MAX"
+};
+
