@@ -153,6 +153,15 @@ protected:
   bool _isRefArray;
 };
 
+/**
+ * Add a const qualifier to force use of const methods
+ */
+template <typename T>
+const T& ConstArray(T& arr)
+{
+  return arr;
+}
+
 /**
  * Wrapper to convert a string array to c_str array
  */
@@ -170,7 +179,6 @@ class CStrArray
       _c_str_array[i] = data[i].c_str();
   }
 
-
   /**
    * Convert to c_str array
    */
@@ -179,6 +187,18 @@ class CStrArray
     return &_c_str_array[0];
   }
 
+  /**
+   * Write back to string array and free c strings if allocated
+   */
+  void writeBack(BaseArray<string>& stringArray)
+  {
+    string *data = stringArray.getData();
+    for(size_t i = 0; i < _c_str_array.size(); i++) {
+      data[i] = _c_str_array[i];
+      _ModelicaFreeStringIfAllocated(_c_str_array[i]);
+    }
+  }
+
  private:
   vector<const char *> _c_str_array;
 };
@@ -432,9 +452,9 @@ public:
    */
   virtual const T& operator()(const vector<size_t>& idx) const
   {
-    assert((size1*size2) > ((idx[0]-1)*size2 + (idx[1]-1)));
+    assert((size1*size2) > ((idx[0]-1) + size1*(idx[1]-1)));
     return *(RefArray<T, size1*size2>::
-             _ref_array[(idx[0]-1)*size2 + (idx[1]-1)]);
+             _ref_array[(idx[0]-1) + size1*(idx[1]-1)]);
   }
 
   /**
@@ -443,9 +463,9 @@ public:
    */
   virtual T& operator()(const vector<size_t>& idx)
   {
-    assert((size1*size2) > ((idx[0]-1)*size2 + (idx[1]-1)));
+    assert((size1*size2) > ((idx[0]-1) + size1*(idx[1]-1)));
     return *(RefArray<T, size1*size2>::
-             _ref_array[(idx[0]-1)*size2 + (idx[1]-1)]);
+             _ref_array[(idx[0]-1) + size1*(idx[1]-1)]);
   }
 
   /**
@@ -455,9 +475,9 @@ public:
    */
   inline virtual T& operator()(size_t i, size_t j)
   {
-    assert((size1*size2) > ((i-1)*size2 + (j-1)));
+    assert((size1*size2) > ((i-1) + size1*(j-1)));
     return *(RefArray<T, size1*size2>::
-             _ref_array[(i-1)*size2 + (j-1)]);
+             _ref_array[(i-1) + size1*(j-1)]);
   }
 
   /**
@@ -560,9 +580,9 @@ public:
    */
   virtual const T& operator()(const vector<size_t>& idx) const
   {
-    assert((size1*size2*size3) > (size3*(idx[0]-1 + size2*(idx[1]-1)) + idx[2]-1));
+    assert(size1*size2*size3 > idx[0]-1 + size1*(idx[1]-1 + size2*(idx[2]-1)));
     return *(RefArray<T, size1*size2*size3>::
-             _ref_array[size3*(idx[0]-1 + size2*(idx[1]-1)) + idx[2]-1]);
+             _ref_array[idx[0]-1 + size1*(idx[1]-1 + size2*(idx[2]-1))]);
   }
 
   /**
@@ -571,9 +591,9 @@ public:
    */
   virtual T& operator()(const vector<size_t>& idx)
   {
-    assert((size1*size2*size3) > (size3*(idx[0]-1 + size2*(idx[1]-1)) + idx[2]-1));
+    assert(size1*size2*size3 > idx[0]-1 + size1*(idx[1]-1 + size2*(idx[2]-1)));
     return *(RefArray<T, size1*size2*size3>::
-             _ref_array[size3*(idx[0]-1 + size2*(idx[1]-1)) + idx[2]-1]);
+             _ref_array[idx[0]-1 + size1*(idx[1]-1 + size2*(idx[2]-1))]);
   }
 
   /**
@@ -584,9 +604,9 @@ public:
    */
   inline virtual T& operator()(size_t i, size_t j, size_t k)
   {
-    assert((size1*size2*size3) > (size3*(i-1 + size2*(j-1)) + (k-1)));
+    assert(size1*size2*size3 > i-1 + size1*(j-1 + size2*(k-1)));
     return *(RefArray<T, size1*size2*size3>::
-             _ref_array[size3*(i-1 + size2*(j-1)) + (k-1)]);
+             _ref_array[i-1 + size1*(j-1 + size2*(k-1))]);
   }
 
   /**
@@ -1527,6 +1547,16 @@ class DynArrayDim1 : public DynArray<T, 1>
     this->resize(dims);
   }
 
+  DynArrayDim1(size_t size1, const T *data)
+    :DynArray<T, 1>()
+  {
+    std::vector<size_t> dims;
+    dims.push_back(size1);
+    this->resize(dims);
+    if (size1 > 0)
+      std::copy(data, data + size1, this->_array_data);
+  }
+
   virtual ~DynArrayDim1()
   {
   }
