@@ -29,14 +29,12 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- *
  */
 
+#include "MainWindow.h"
 #include "LineAnnotation.h"
-#include "Commands.h"
+#include "Modeling/Commands.h"
 
 LineAnnotation::LineAnnotation(QString annotation, GraphicsView *pGraphicsView)
   : ShapeAnnotation(false, pGraphicsView, 0)
@@ -44,6 +42,10 @@ LineAnnotation::LineAnnotation(QString annotation, GraphicsView *pGraphicsView)
   setLineType(LineAnnotation::ShapeType);
   setStartComponent(0);
   setEndComponent(0);
+  setDelay("");
+  setZf("");
+  setZfr("");
+  setAlpha("");
   // set the default values
   GraphicItem::setDefaults();
   ShapeAnnotation::setDefaults();
@@ -57,9 +59,14 @@ LineAnnotation::LineAnnotation(ShapeAnnotation *pShapeAnnotation, Component *pPa
   : ShapeAnnotation(pParent)
 {
   updateShape(pShapeAnnotation);
+  initUpdateVisible(); // DynamicSelect for visible attribute
   setLineType(LineAnnotation::ComponentType);
   setStartComponent(0);
   setEndComponent(0);
+  setDelay("");
+  setZf("");
+  setZfr("");
+  setAlpha("");
   setPos(mOrigin);
   setRotation(mRotation);
   connect(pShapeAnnotation, SIGNAL(updateReferenceShapes()), pShapeAnnotation, SIGNAL(changed()));
@@ -99,6 +106,10 @@ LineAnnotation::LineAnnotation(Component *pStartComponent, GraphicsView *pGraphi
   // set the start component
   setStartComponent(pStartComponent);
   setEndComponent(0);
+  setDelay("1e-4");
+  setZf("10000");
+  setZfr("100");
+  setAlpha("0.2");
 }
 
 LineAnnotation::LineAnnotation(QString annotation, Component *pStartComponent, Component *pEndComponent, GraphicsView *pGraphicsView)
@@ -114,6 +125,10 @@ LineAnnotation::LineAnnotation(QString annotation, Component *pStartComponent, C
   setStartComponent(pStartComponent);
   // set the end component
   setEndComponent(pEndComponent);
+  setDelay("");
+  setZf("");
+  setZfr("");
+  setAlpha("");
   parseShapeAnnotation(annotation);
   /* make the points relative to origin */
   QList<QPointF> points;
@@ -133,6 +148,10 @@ LineAnnotation::LineAnnotation(Component *pParent)
   setLineType(LineAnnotation::ComponentType);
   setStartComponent(0);
   setEndComponent(0);
+  setDelay("");
+  setZf("");
+  setZfr("");
+  setAlpha("");
   // set the default values
   GraphicItem::setDefaults();
   ShapeAnnotation::setDefaults();
@@ -157,6 +176,10 @@ LineAnnotation::LineAnnotation(GraphicsView *pGraphicsView)
   setLineType(LineAnnotation::ShapeType);
   setStartComponent(0);
   setEndComponent(0);
+  setDelay("");
+  setZf("");
+  setZfr("");
+  setAlpha("");
   // set the default values
   GraphicItem::setDefaults();
   ShapeAnnotation::setDefaults();
@@ -269,7 +292,7 @@ void LineAnnotation::paint(QPainter *painter, const QStyleOptionGraphicsItem *op
 {
   Q_UNUSED(option);
   Q_UNUSED(widget);
-  if (mVisible) {
+  if (mVisible || !mDynamicVisible.isEmpty()) {
     drawLineAnnotaion(painter);
   }
 }
@@ -466,7 +489,7 @@ QString LineAnnotation::getShapeAnnotation()
   return QString("Line(").append(annotationString.join(",")).append(")");
 }
 
-QString LineAnnotation::getTLMShapeAnnotation()
+QString LineAnnotation::getMetaModelShapeAnnotation()
 {
   QStringList annotationString;
   annotationString.append(GraphicItem::getShapeAnnotation());
@@ -617,6 +640,7 @@ void LineAnnotation::updateEndPoint(QPointF point)
       mPoints.back() = point;
       updateCornerItem(lastIndex);
       /* update the 2nd point */
+      assert(secondLastIndex < mGeometries.size());
       if (mGeometries[secondLastIndex] == ShapeAnnotation::HorizontalLine) {
         mPoints[secondLastIndex] = QPointF(mPoints[secondLastIndex].x(), mPoints[secondLastIndex].y() + dy);
       } else if (mGeometries[secondLastIndex] == ShapeAnnotation::VerticalLine) {
@@ -632,11 +656,17 @@ void LineAnnotation::updateEndPoint(QPointF point)
   }
 }
 
+/*!
+ * \brief LineAnnotation::moveAllPoints
+ * Moves all the whole connection.
+ * \param offsetX
+ * \param offsetY
+ */
 void LineAnnotation::moveAllPoints(qreal offsetX, qreal offsetY)
 {
   prepareGeometryChange();
   for(int i = 0 ; i < mPoints.size() ; i++) {
-    mPoints[i] = QPointF(mPoints[i].x()+offsetX, mPoints[i].y()+offsetY);
+    mPoints[i] = QPointF(mPoints[i].x() + offsetX, mPoints[i].y() + offsetY);
     /* updated the corresponding CornerItem */
     updateCornerItem(i);
   }
@@ -670,6 +700,10 @@ void LineAnnotation::updateShape(ShapeAnnotation *pShapeAnnotation)
   setStartComponentName(pLineAnnotation->getStartComponentName());
   setEndComponent(pLineAnnotation->getEndComponent());
   setEndComponentName(pLineAnnotation->getEndComponentName());
+  setDelay(pLineAnnotation->getDelay());
+  setZf(pLineAnnotation->getZf());
+  setZfr(pLineAnnotation->getZfr());
+  setAlpha(pLineAnnotation->getAlpha());
   // set the default values
   GraphicItem::setDefaults(pShapeAnnotation);
   mPoints.clear();
@@ -680,9 +714,25 @@ void LineAnnotation::updateShape(ShapeAnnotation *pShapeAnnotation)
   ShapeAnnotation::setDefaults(pShapeAnnotation);
 }
 
+/*!
+ * \brief LineAnnotation::setAligned
+ * Marks the connection line as aligned or not aligned.
+ * \param aligned
+ */
+void LineAnnotation::setAligned(bool aligned)
+{
+  if (aligned) {
+    setLineColor(QColor(Qt::black));
+  } else {
+    setLineColor(QColor(Qt::red));
+  }
+  update();
+}
+
 /*!
  * \brief LineAnnotation::handleComponentMoved
- * If the component associated with the connection is moved then update the connection accordingly.
+ * If the component associated with the connection is moved then update the connection accordingly.\n
+ * If the both start and end components associated with the connection are moved then move whole connection.
  */
 void LineAnnotation::handleComponentMoved()
 {
@@ -690,16 +740,22 @@ void LineAnnotation::handleComponentMoved()
     return;
   }
   prepareGeometryChange();
-  if (mpStartComponent) {
-    Component *pComponent = qobject_cast<Component*>(sender());
-    if (pComponent == mpStartComponent->getRootParentComponent()) {
-      updateStartPoint(mpGraphicsView->roundPoint(mpStartComponent->mapToScene(mpStartComponent->boundingRect().center())));
+  if (mpStartComponent && mpStartComponent->getRootParentComponent()->isSelected() &&
+      mpEndComponent && mpEndComponent->getRootParentComponent()->isSelected()) {
+    moveAllPoints(mpStartComponent->mapToScene(mpStartComponent->boundingRect().center()).x() - mPoints[0].x(),
+        mpStartComponent->mapToScene(mpStartComponent->boundingRect().center()).y() - mPoints[0].y());
+  } else {
+    if (mpStartComponent) {
+      Component *pComponent = qobject_cast<Component*>(sender());
+      if (pComponent == mpStartComponent->getRootParentComponent()) {
+        updateStartPoint(mpGraphicsView->roundPoint(mpStartComponent->mapToScene(mpStartComponent->boundingRect().center())));
+      }
     }
-  }
-  if (mpEndComponent) {
-    Component *pComponent = qobject_cast<Component*>(sender());
-    if (pComponent == mpEndComponent->getRootParentComponent()) {
-      updateEndPoint(mpGraphicsView->roundPoint(mpEndComponent->mapToScene(mpEndComponent->boundingRect().center())));
+    if (mpEndComponent) {
+      Component *pComponent = qobject_cast<Component*>(sender());
+      if (pComponent == mpEndComponent->getRootParentComponent()) {
+        updateEndPoint(mpGraphicsView->roundPoint(mpEndComponent->mapToScene(mpEndComponent->boundingRect().center())));
+      }
     }
   }
 }
@@ -710,15 +766,14 @@ void LineAnnotation::handleComponentMoved()
  */
 void LineAnnotation::updateConnectionAnnotation()
 {
-  if (mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getLibraryType()== LibraryTreeItem::TLM) {
-    TLMEditor *pTLMEditor = dynamic_cast<TLMEditor*>(mpGraphicsView->getModelWidget()->getEditor());
-    pTLMEditor->updateTLMConnectiontAnnotation(getStartComponentName(), getEndComponentName(), getTLMShapeAnnotation());
-
-   } else {
+  if (mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getLibraryType()== LibraryTreeItem::MetaModel) {
+    MetaModelEditor *pMetaModelEditor = dynamic_cast<MetaModelEditor*>(mpGraphicsView->getModelWidget()->getEditor());
+    pMetaModelEditor->updateConnection(this);
+  } else {
     // get the connection line annotation.
     QString annotationString = QString("annotate=").append(getShapeAnnotation());
     // update the connection
-    OMCProxy *pOMCProxy = mpGraphicsView->getModelWidget()->getModelWidgetContainer()->getMainWindow()->getOMCProxy();
+    OMCProxy *pOMCProxy = MainWindow::instance()->getOMCProxy();
     pOMCProxy->updateConnection(getStartComponentName(), getEndComponentName(),
                                 mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getNameStructure(), annotationString);
   }
@@ -1117,6 +1172,7 @@ ExpandableConnectorTreeView::ExpandableConnectorTreeView(CreateConnectionDialog
   setItemDelegate(new ItemDelegate(this));
   setTextElideMode(Qt::ElideMiddle);
   setIndentation(Helper::treeIndentation);
+  setUniformRowHeights(true);
 }
 
 /*!
@@ -1130,18 +1186,14 @@ ExpandableConnectorTreeView::ExpandableConnectorTreeView(CreateConnectionDialog
  * \param pParent
  */
 CreateConnectionDialog::CreateConnectionDialog(GraphicsView *pGraphicsView, LineAnnotation *pConnectionLineAnnotation, QWidget *pParent)
-  : QDialog(pParent, Qt::WindowTitleHint), mpGraphicsView(pGraphicsView), mpConnectionLineAnnotation(pConnectionLineAnnotation)
+  : QDialog(pParent), mpGraphicsView(pGraphicsView), mpConnectionLineAnnotation(pConnectionLineAnnotation)
 {
   setWindowTitle(QString(Helper::applicationName).append(" - ").append(Helper::createConnection));
   setAttribute(Qt::WA_DeleteOnClose);
   // heading
-  mpHeading = new Label(Helper::createConnection);
-  mpHeading->setFont(QFont(Helper::systemFontInfo.family(), Helper::headingFontSize));
-  mpHeading->setAlignment(Qt::AlignTop);
+  mpHeading = Utilities::getHeadingLabel(Helper::createConnection);
   // horizontal line
-  mpHorizontalLine = new QFrame();
-  mpHorizontalLine->setFrameShape(QFrame::HLine);
-  mpHorizontalLine->setFrameShadow(QFrame::Sunken);
+  mpHorizontalLine = Utilities::getHeadingLine();
   // Start expandable connector treeview
   mpStartExpandableConnectorTreeView = 0;
   if ((!mpConnectionLineAnnotation->getStartComponent()->getParentComponent() && mpConnectionLineAnnotation->getStartComponent()->getRootParentComponent()->getLibraryTreeItem()->getRestriction() == StringHandler::ExpandableConnector) ||
@@ -1223,7 +1275,7 @@ CreateConnectionDialog::CreateConnectionDialog(GraphicsView *pGraphicsView, Line
   // Create the buttons
   mpOkButton = new QPushButton(Helper::ok);
   mpOkButton->setAutoDefault(true);
-  connect(mpOkButton, SIGNAL(clicked()), SLOT(createArrayConnection()));
+  connect(mpOkButton, SIGNAL(clicked()), SLOT(createConnection()));
   mpCancelButton = new QPushButton(Helper::cancel);
   mpCancelButton->setAutoDefault(false);
   connect(mpCancelButton, SIGNAL(clicked()), SLOT(reject()));
@@ -1458,7 +1510,7 @@ void CreateConnectionDialog::endConnectorChanged(const QModelIndex &current, con
  * \brief ConnectionArray::createArrayConnection
  * Slot activated when mpOkButton clicked SIGNAL is raised. Creates an array connection.
  */
-void CreateConnectionDialog::createArrayConnection()
+void CreateConnectionDialog::createConnection()
 {
   QString startComponentName, endComponentName;
   // set start component name
