@@ -29,44 +29,24 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- * RCS: $Id$
- *
  */
 
 #include "TextAnnotation.h"
+#include "Modeling/Commands.h"
 
 /*!
  * \class TextAnnotation
  * \brief Draws the text shapes.
  */
-/*!
- * \brief TextAnnotation::TextAnnotation
- * \param annotation - text annotation string.
- * \param pComponent - pointer to Component
- */
-TextAnnotation::TextAnnotation(QString annotation, Component *pComponent)
-  : ShapeAnnotation(pComponent), mpComponent(pComponent)
-{
-  // set the default values
-  GraphicItem::setDefaults();
-  FilledShape::setDefaults();
-  ShapeAnnotation::setDefaults();
-  parseShapeAnnotation(annotation);
-  setPos(mOrigin);
-  setRotation(mRotation);
-}
-
 /*!
  * \brief TextAnnotation::TextAnnotation
  * \param annotation - text annotation string.
  * \param inheritedShape
  * \param pGraphicsView - pointer to GraphicsView
  */
-TextAnnotation::TextAnnotation(QString annotation, bool inheritedShape, GraphicsView *pGraphicsView)
-  : ShapeAnnotation(inheritedShape, pGraphicsView, 0)
+TextAnnotation::TextAnnotation(QString annotation, GraphicsView *pGraphicsView)
+  : ShapeAnnotation(false, pGraphicsView, 0)
 {
   mpComponent = 0;
   // set the default values
@@ -77,9 +57,49 @@ TextAnnotation::TextAnnotation(QString annotation, bool inheritedShape, Graphics
   ShapeAnnotation::setUserDefaults();
   parseShapeAnnotation(annotation);
   setShapeFlags(true);
-  mpGraphicsView->addShapeObject(this);
-  mpGraphicsView->scene()->addItem(this);
-  connect(this, SIGNAL(updateClassAnnotation()), mpGraphicsView, SLOT(addClassAnnotation()));
+}
+
+TextAnnotation::TextAnnotation(ShapeAnnotation *pShapeAnnotation, Component *pParent)
+  : ShapeAnnotation(pParent), mpComponent(pParent)
+{
+  updateShape(pShapeAnnotation);
+  initUpdateVisible(); // DynamicSelect for visible attribute
+  initUpdateTextString();
+  setPos(mOrigin);
+  setRotation(mRotation);
+  connect(pShapeAnnotation, SIGNAL(updateReferenceShapes()), pShapeAnnotation, SIGNAL(changed()));
+  connect(pShapeAnnotation, SIGNAL(added()), this, SLOT(referenceShapeAdded()));
+  connect(pShapeAnnotation, SIGNAL(changed()), this, SLOT(referenceShapeChanged()));
+  connect(pShapeAnnotation, SIGNAL(deleted()), this, SLOT(referenceShapeDeleted()));
+}
+
+TextAnnotation::TextAnnotation(ShapeAnnotation *pShapeAnnotation, GraphicsView *pGraphicsView)
+  : ShapeAnnotation(true, pGraphicsView, 0)
+{
+  mpComponent = 0;
+  updateShape(pShapeAnnotation);
+  setShapeFlags(true);
+  mpGraphicsView->addItem(this);
+  connect(pShapeAnnotation, SIGNAL(updateReferenceShapes()), pShapeAnnotation, SIGNAL(changed()));
+  connect(pShapeAnnotation, SIGNAL(added()), this, SLOT(referenceShapeAdded()));
+  connect(pShapeAnnotation, SIGNAL(changed()), this, SLOT(referenceShapeChanged()));
+  connect(pShapeAnnotation, SIGNAL(deleted()), this, SLOT(referenceShapeDeleted()));
+}
+
+TextAnnotation::TextAnnotation(Component *pParent)
+  : ShapeAnnotation(pParent), mpComponent(pParent)
+{
+  // set the default values
+  GraphicItem::setDefaults();
+  FilledShape::setDefaults();
+  ShapeAnnotation::setDefaults();
+  // give a reasonable size to default component text
+  mExtents.replace(0, QPointF(-50, -50));
+  mExtents.replace(1, QPointF(50, 50));
+  setTextString("%name");
+  initUpdateTextString();
+  setPos(mOrigin);
+  setRotation(mRotation);
 }
 
 /*!
@@ -93,27 +113,32 @@ void TextAnnotation::parseShapeAnnotation(QString annotation)
   FilledShape::parseShapeAnnotation(annotation);
   // parse the shape to get the list of attributes of Text.
   QStringList list = StringHandler::getStrings(annotation);
-  if (list.size() < 11)
+  if (list.size() < 11) {
     return;
+  }
   // 9th item of the list contains the extent points
   QStringList extentsList = StringHandler::getStrings(StringHandler::removeFirstLastCurlBrackets(list.at(8)));
-  for (int i = 0 ; i < qMin(extentsList.size(), 2) ; i++)
-  {
+  for (int i = 0 ; i < qMin(extentsList.size(), 2) ; i++) {
     QStringList extentPoints = StringHandler::getStrings(StringHandler::removeFirstLastCurlBrackets(extentsList[i]));
     if (extentPoints.size() >= 2)
       mExtents.replace(i, QPointF(extentPoints.at(0).toFloat(), extentPoints.at(1).toFloat()));
   }
   // 10th item of the list contains the textString.
-  mOriginalTextString = StringHandler::removeFirstLastQuotes(list.at(9));
-  mTextString = mOriginalTextString;
-  if (mpComponent)
-  {
-    if (mOriginalTextString.contains("%"))
-    {
-      updateTextString();
-      connect(mpComponent->getRootParentComponent(), SIGNAL(componentDisplayTextChanged()), SLOT(updateTextString()));
-    }
+  if (list.at(9).startsWith("{")) {
+    // DynamicSelect
+    QStringList args = StringHandler::getStrings(StringHandler::removeFirstLastCurlBrackets(list.at(9)));
+    if (args.count() > 0)
+      mOriginalTextString = StringHandler::removeFirstLastQuotes(args.at(0));
+    if (args.count() > 1)
+      mDynamicTextString << args.at(1);  // variable name
+    if (args.count() > 2)
+      mDynamicTextString << args.at(2);  // significantDigits
   }
+  else {
+    mOriginalTextString = StringHandler::removeFirstLastQuotes(list.at(9));
+  }
+  mTextString = mOriginalTextString;
+  initUpdateTextString();
   // 11th item of the list contains the fontSize.
   mFontSize = list.at(10).toFloat();
   //Now comes the optional parameters; fontName and textStyle.
@@ -122,43 +147,30 @@ void TextAnnotation::parseShapeAnnotation(QString annotation)
   // parse the shape to get the list of attributes of Text Annotation.
   list = StringHandler::getStrings(annotation);
   int index = 19;
-  while(index < list.size())
-  {
+  mTextStyles.clear();
+  while(index < list.size()) {
     QString annotationValue = StringHandler::removeFirstLastQuotes(list.at(index));
     // check textStyles enumeration.
-    if(annotationValue == "TextStyle.Bold")
-    {
+    if(annotationValue == "TextStyle.Bold") {
       mTextStyles.append(StringHandler::TextStyleBold);
       index++;
-    }
-    else if(annotationValue == "TextStyle.Italic")
-    {
+    } else if(annotationValue == "TextStyle.Italic") {
       mTextStyles.append(StringHandler::TextStyleItalic);
       index++;
-    }
-    else if(annotationValue == "TextStyle.UnderLine")
-    {
+    } else if(annotationValue == "TextStyle.UnderLine") {
       mTextStyles.append(StringHandler::TextStyleUnderLine);
       index++;
-    }
-    // check textAlignment enumeration.
-    else if(annotationValue == "TextAlignment.Left")
-    {
+    } else if(annotationValue == "TextAlignment.Left") {
+      // check textAlignment enumeration.
       mHorizontalAlignment = StringHandler::TextAlignmentLeft;
       index++;
-    }
-    else if(annotationValue == "TextAlignment.Center")
-    {
+    } else if(annotationValue == "TextAlignment.Center") {
       mHorizontalAlignment = StringHandler::TextAlignmentCenter;
       index++;
-    }
-    else if(annotationValue == "TextAlignment.Right")
-    {
+    } else if(annotationValue == "TextAlignment.Right") {
       mHorizontalAlignment = StringHandler::TextAlignmentRight;
       index++;
-    }
-    else
-    {
+    } else {
       mFontName = annotationValue;
       index++;
     }
@@ -198,7 +210,18 @@ void TextAnnotation::paint(QPainter *painter, const QStyleOptionGraphicsItem *op
 {
   Q_UNUSED(option);
   Q_UNUSED(widget);
-  if (mVisible) {
+  //! @note We don't show text annotation that contains % for Library Icons. Only static text for functions are shown.
+  if (mpGraphicsView && mpGraphicsView->isRenderingLibraryPixmap()) {
+    if (mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getRestriction() != StringHandler::Function) {
+      return;
+    }
+    if (mOriginalTextString.contains("%")) {
+      return;
+    }
+  } else if (mpComponent && mpComponent->getGraphicsView()->isRenderingLibraryPixmap()) {
+    return;
+  }
+  if (mVisible || !mDynamicVisible.isEmpty()) {
     drawTextAnnotaion(painter);
   }
 }
@@ -213,9 +236,11 @@ void TextAnnotation::drawTextAnnotaion(QPainter *painter)
   applyLinePattern(painter);
   /* Don't apply the fill patterns on Text shapes. */
   /*applyFillPattern(painter);*/
+  qreal dx = ((-boundingRect().left()) - boundingRect().right());
+  qreal dy = ((-boundingRect().top()) - boundingRect().bottom());
   // first we invert the painter since we have our coordinate system inverted.
   painter->scale(1.0, -1.0);
-  painter->translate(0, ((-boundingRect().top()) - boundingRect().bottom()));
+  painter->translate(0, dy);
   mTextString = StringHandler::removeFirstLastQuotes(mTextString);
   mTextString = StringHandler::unparse(QString("\"").append(mTextString).append("\""));
   QFont font;
@@ -251,27 +276,43 @@ void TextAnnotation::drawTextAnnotaion(QPainter *painter)
   }
   if (mpComponent) {
     Component *pComponent = mpComponent->getRootParentComponent();
-    if (pComponent && pComponent->getTransformation()) {
-      QPointF extent1 = pComponent->getTransformation()->getExtent1();
-      QPointF extent2 = pComponent->getTransformation()->getExtent2();
-      qreal dy = ((-boundingRect().top()) - boundingRect().bottom());
-      // if horizontal flip
-      if (extent2.x() < extent1.x()) {
-        painter->scale(-1.0, 1.0);
-      }
-      // if vertical flip
-      if (extent2.y() < extent1.y()) {
-        painter->scale(1.0, -1.0);
-        painter->translate(0, dy);
-      }
-      qreal angle = StringHandler::getNormalizedAngle(pComponent->getTransformation()->getRotateAngle());
-      if (angle == 180) {
-        painter->scale(-1.0, -1.0);
-        painter->translate(0, dy);
+    if (pComponent && pComponent->mTransformation.isValid()) {
+      QPointF extent1 = pComponent->mTransformation.getExtent1();
+      QPointF extent2 = pComponent->mTransformation.getExtent2();
+      qreal componentAngle = StringHandler::getNormalizedAngle(pComponent->mTransformation.getRotateAngle());
+      qreal shapeAngle = StringHandler::getNormalizedAngle(getRotation());
+      // if shape has its own angle
+      if (shapeAngle > 0) {
+        shapeAngle = StringHandler::getNormalizedAngle(pComponent->mTransformation.getRotateAngle() + getRotation());
+        if (shapeAngle == 180) {
+          painter->scale(-1.0, -1.0);
+          painter->translate(dx, dy);
+        }
+        if (extent2.x() < extent1.x()) {  // if vertical flip
+          painter->scale(1.0, -1.0);
+          painter->translate(0, dy);
+        }
+        if (extent2.y() < extent1.y()) {  // if horizontal flip
+          painter->scale(-1.0, 1.0);
+          painter->translate(dx, 0);
+        }
+      } else {
+        if (componentAngle == 180) {
+          painter->scale(-1.0, -1.0);
+          painter->translate(dx, dy);
+        }
+        if (extent2.x() < extent1.x()) {  // if horizontal flip
+          painter->scale(-1.0, 1.0);
+          painter->translate(dx, 0);
+        }
+        if (extent2.y() < extent1.y()) {  // if vertical flip
+          painter->scale(1.0, -1.0);
+          painter->translate(0, dy);
+        }
       }
     }
   } else {
-    qreal angle = StringHandler::getNormalizedAngle(mpTransformation->getRotateAngle());
+    qreal angle = StringHandler::getNormalizedAngle(mTransformation.getRotateAngle());
     if (angle == 180) {
       painter->scale(-1.0, -1.0);
       painter->translate(((-boundingRect().left()) - boundingRect().right()), ((-boundingRect().top()) - boundingRect().bottom()));
@@ -285,6 +326,51 @@ void TextAnnotation::drawTextAnnotaion(QPainter *painter)
   }
 }
 
+/*!
+ * \brief TextAnnotation::getOMCShapeAnnotation
+ * \return the shape annotation in format as returned by OMC.
+ */
+QString TextAnnotation::getOMCShapeAnnotation()
+{
+  QStringList annotationString;
+  annotationString.append(GraphicItem::getOMCShapeAnnotation());
+  annotationString.append(FilledShape::getOMCShapeAnnotation());
+  // get the extents
+  QString extentString;
+  extentString.append("{");
+  extentString.append("{").append(QString::number(mExtents.at(0).x())).append(",");
+  extentString.append(QString::number(mExtents.at(0).y())).append("},");
+  extentString.append("{").append(QString::number(mExtents.at(1).x())).append(",");
+  extentString.append(QString::number(mExtents.at(1).y())).append("}");
+  extentString.append("}");
+  annotationString.append(extentString);
+  // get the text string
+  annotationString.append(QString("\"").append(mOriginalTextString).append("\""));
+  // get the font size
+  annotationString.append(QString::number(mFontSize));
+  // get the font name
+  if (!mFontName.isEmpty()) {
+    annotationString.append(QString("\"").append(mFontName).append("\""));
+  }
+  // get the font styles
+  QString textStylesString;
+  QStringList stylesList;
+  if (mTextStyles.size() > 0) {
+    textStylesString.append("{");
+  }
+  for (int i = 0 ; i < mTextStyles.size() ; i++) {
+    stylesList.append(StringHandler::getTextStyleString(mTextStyles[i]));
+  }
+  if (mTextStyles.size() > 0) {
+    textStylesString.append(stylesList.join(","));
+    textStylesString.append("}");
+    annotationString.append(textStylesString);
+  }
+  // get the font horizontal alignment
+  annotationString.append(StringHandler::getTextAlignmentString(mHorizontalAlignment));
+  return annotationString.join(",");
+}
+
 /*!
  * \brief TextAnnotation::getShapeAnnotation
  * \return the shape annotation in Modelica syntax.
@@ -295,8 +381,7 @@ QString TextAnnotation::getShapeAnnotation()
   annotationString.append(GraphicItem::getShapeAnnotation());
   annotationString.append(FilledShape::getShapeAnnotation());
   // get the extents
-  if (mExtents.size() > 1)
-  {
+  if (mExtents.size() > 1) {
     QString extentString;
     extentString.append("extent={");
     extentString.append("{").append(QString::number(mExtents.at(0).x())).append(",");
@@ -309,32 +394,55 @@ QString TextAnnotation::getShapeAnnotation()
   // get the text string
   annotationString.append(QString("textString=\"").append(mOriginalTextString).append("\""));
   // get the font size
-  if (mFontSize != 0)
+  if (mFontSize != 0) {
     annotationString.append(QString("fontSize=").append(QString::number(mFontSize)));
+  }
   // get the font name
-  if (!mFontName.isEmpty())
+  /* Ticket:4204
+   * Don't insert the default font name as it might be operating system specific.
+   */
+  if (!mFontName.isEmpty() && mFontName.compare(Helper::systemFontInfo.family()) != 0) {
     annotationString.append(QString("fontName=\"").append(mFontName).append("\""));
+  }
   // get the font styles
   QString textStylesString;
   QStringList stylesList;
-  if (mTextStyles.size() > 0)
+  if (mTextStyles.size() > 0) {
     textStylesString.append("textStyle={");
-  for (int i = 0 ; i < mTextStyles.size() ; i++)
-  {
+  }
+  for (int i = 0 ; i < mTextStyles.size() ; i++) {
     stylesList.append(StringHandler::getTextStyleString(mTextStyles[i]));
   }
-  if (mTextStyles.size() > 0)
-  {
+  if (mTextStyles.size() > 0) {
     textStylesString.append(stylesList.join(","));
     textStylesString.append("}");
     annotationString.append(textStylesString);
   }
   // get the font horizontal alignment
-  if (mHorizontalAlignment != StringHandler::TextAlignmentCenter)
+  if (mHorizontalAlignment != StringHandler::TextAlignmentCenter) {
     annotationString.append(QString("horizontalAlignment=").append(StringHandler::getTextAlignmentString(mHorizontalAlignment)));
+  }
   return QString("Text(").append(annotationString.join(",")).append(")");
 }
 
+void TextAnnotation::updateShape(ShapeAnnotation *pShapeAnnotation)
+{
+  // set the default values
+  GraphicItem::setDefaults(pShapeAnnotation);
+  FilledShape::setDefaults(pShapeAnnotation);
+  ShapeAnnotation::setDefaults(pShapeAnnotation);
+}
+
+void TextAnnotation::initUpdateTextString()
+{
+  if (mpComponent) {
+    if (mOriginalTextString.contains("%") || mDynamicTextString.count() > 0) {
+      updateTextString();
+      connect(mpComponent, SIGNAL(displayTextChanged()), SLOT(updateTextString()), Qt::UniqueConnection);
+    }
+  }
+}
+
 /*!
  * \brief TextAnnotation::updateTextStringHelper
  * Helper function for TextAnnotation::updateTextString()
@@ -343,28 +451,26 @@ QString TextAnnotation::getShapeAnnotation()
 void TextAnnotation::updateTextStringHelper(QRegExp regExp)
 {
   int pos = 0;
-  while ((pos = regExp.indexIn(mTextString, pos)) != -1)
-  {
+  while ((pos = regExp.indexIn(mTextString, pos)) != -1) {
     QString variable = regExp.cap(0);
-    if ((!variable.isEmpty()) && (variable.compare("%%") != 0) && (variable.compare("%name") != 0) && (variable.compare("%class") != 0))
-    {
+    if ((!variable.isEmpty()) && (variable.compare("%%") != 0) && (variable.compare("%name") != 0) && (variable.compare("%class") != 0)) {
       variable.remove("%");
-      if (!variable.isEmpty())
-      {
-        QString textValue = mpComponent->getParameterDisplayString(variable);
-        if (!textValue.isEmpty())
-        {
-          mTextString.replace(pos, regExp.matchedLength(), textValue);
+      if (!variable.isEmpty()) {
+        QString textValue;
+        /* Ticket:4204
+         * If we have extend component then call Component::getParameterDisplayString from root component.
+         */
+        if (mpComponent->getComponentType() == Component::Extend) {
+          textValue = mpComponent->getRootParentComponent()->getParameterDisplayString(variable);
+        } else {
+          textValue = mpComponent->getRootParentComponent()->getParameterDisplayString(variable);
         }
-        /* if the value of %\\W* is empty then remove the % sign. */
-        else
-        {
+        if (!textValue.isEmpty()) {
+          mTextString.replace(pos, regExp.matchedLength(), textValue);
+        } else { /* if the value of %\\W* is empty then remove the % sign. */
           mTextString.replace(pos, 1, "");
         }
-      }
-      /* if there is just alone % then remove it. Because if you want to print % then use %%. */
-      else
-      {
+      } else { /* if there is just alone % then remove it. Because if you want to print % then use %%. */
         mTextString.replace(pos, 1, "");
       }
     }
@@ -378,68 +484,74 @@ void TextAnnotation::updateTextStringHelper(QRegExp regExp)
  */
 void TextAnnotation::updateTextString()
 {
-  /*
-    From Modelica Spec 32revision2,
-    There are a number of common macros that can be used in the text, and they should be replaced when displaying
-    the text as follows:
-    - %par replaced by the value of the parameter par. The intent is that the text is easily readable, thus if par is
-    of an enumeration type, replace %par by the item name, not by the full name.
-    [Example: if par="Modelica.Blocks.Types.Enumeration.Periodic", then %par should be displayed as
-    "Periodic"]
-    - %% replaced by %
-    - %name replaced by the name of the component (i.e. the identifier for it in in the enclosing class).
-    - %class replaced by the name of the class.
-  */
+  /* optional DynamicSelect of textString attribute */
+  QVariant dynamicValue; // isNull() per default
+  if (mDynamicTextString.count() > 0) {
+    dynamicValue = getDynamicValue(mDynamicTextString.at(0).toString());
+  }
+  if (!dynamicValue.isNull()) {
+    mTextString = dynamicValue.toString();
+    if (mTextString.isEmpty()) {
+      /* use variable name as default value if result not found */
+      mTextString = mDynamicTextString.at(0).toString();
+    }
+    else if (mDynamicTextString.count() > 1) {
+      int digits = mDynamicTextString.at(1).toInt();
+      mTextString = QString::number(mTextString.toDouble(), 'g', digits);
+    }
+    return;
+  }
+  /* alternatively use model provided value */
+  /* From Modelica Spec 32revision2,
+   * There are a number of common macros that can be used in the text, and they should be replaced when displaying
+   * the text as follows:
+   * - %par replaced by the value of the parameter par. The intent is that the text is easily readable, thus if par is
+   * of an enumeration type, replace %par by the item name, not by the full name.
+   * [Example: if par="Modelica.Blocks.Types.Enumeration.Periodic", then %par should be displayed as "Periodic"]
+   * - %% replaced by %
+   * - %name replaced by the name of the component (i.e. the identifier for it in in the enclosing class).
+   * - %class replaced by the name of the class.
+   */
   mTextString = mOriginalTextString;
-  if (!mTextString.contains("%"))
+  if (!mTextString.contains("%")) {
     return;
-  if (mOriginalTextString.toLower().contains("%name"))
-  {
-    mTextString.replace(QRegExp("%name"), mpComponent->getRootParentComponent()->getName());
   }
-  if (mOriginalTextString.toLower().contains("%class"))
-  {
-    mTextString.replace(QRegExp("%class"), mpComponent->getRootParentComponent()->getClassName());
+  if (mOriginalTextString.toLower().contains("%name")) {
+    mTextString.replace(QRegExp("%name"), mpComponent->getName());
   }
-  if (!mTextString.contains("%"))
+  if (mOriginalTextString.toLower().contains("%class")) {
+    mTextString.replace(QRegExp("%class"), mpComponent->getLibraryTreeItem()->getNameStructure());
+  }
+  if (!mTextString.contains("%")) {
     return;
+  }
   /* handle variables now */
   updateTextStringHelper(QRegExp("(%%|%\\w*)"));
   /* call again with non-word characters so invalid % can be removed. */
   updateTextStringHelper(QRegExp("(%%|%\\W*)"));
   /* handle %% */
-  if (mOriginalTextString.toLower().contains("%%"))
-  {
+  if (mOriginalTextString.toLower().contains("%%")) {
     mTextString.replace(QRegExp("%%"), "%");
   }
 }
 
 /*!
  * \brief TextAnnotation::duplicate
- * Creates a duplicate of this object.
+ * Duplicates the shape.
  */
 void TextAnnotation::duplicate()
 {
-  TextAnnotation *pTextAnnotation = new TextAnnotation("", false, mpGraphicsView);
-  QPointF gridStep(mpGraphicsView->getCoOrdinateSystem()->getHorizontalGridStep(),
-                   mpGraphicsView->getCoOrdinateSystem()->getVerticalGridStep());
+  TextAnnotation *pTextAnnotation = new TextAnnotation("", mpGraphicsView);
+  pTextAnnotation->updateShape(this);
+  QPointF gridStep(mpGraphicsView->mCoOrdinateSystem.getHorizontalGridStep() * 5,
+                   mpGraphicsView->mCoOrdinateSystem.getVerticalGridStep() * 5);
   pTextAnnotation->setOrigin(mOrigin + gridStep);
-  pTextAnnotation->setRotationAngle(mRotation);
   pTextAnnotation->initializeTransformation();
-  pTextAnnotation->setLineColor(getLineColor());
-  pTextAnnotation->setFillColor(getFillColor());
-  pTextAnnotation->setLinePattern(getLinePattern());
-  pTextAnnotation->setFillPattern(getFillPattern());
-  pTextAnnotation->setLineThickness(getLineThickness());
-  pTextAnnotation->setExtents(getExtents());
-  pTextAnnotation->setTextString(getTextString());
-  pTextAnnotation->setFontSize(getFontSize());
-  pTextAnnotation->setFontName(getFontName());
-  pTextAnnotation->setTextStyles(getTextStyles());
-  pTextAnnotation->setTextHorizontalAlignment(getTextHorizontalAlignment());
   pTextAnnotation->drawCornerItems();
-  pTextAnnotation->setCornerItemsPassive();
+  pTextAnnotation->setCornerItemsActiveOrPassive();
   pTextAnnotation->update();
-  mpGraphicsView->addClassAnnotation();
-  mpGraphicsView->setCanAddClassAnnotation(true);
+  mpGraphicsView->getModelWidget()->getUndoStack()->push(new AddShapeCommand(pTextAnnotation));
+  mpGraphicsView->getModelWidget()->getLibraryTreeItem()->emitShapeAdded(pTextAnnotation, mpGraphicsView);
+  setSelected(false);
+  pTextAnnotation->setSelected(true);
 }
