@@ -29,23 +29,23 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- *
  */
 
 #include "Transformation.h"
+#include "Component.h"
 
 Transformation::Transformation()
 {
   mValid = false;
+  mpComponent = 0;
   initialize(StringHandler::Diagram);
 }
 
-Transformation::Transformation(StringHandler::ViewType viewType)
+Transformation::Transformation(StringHandler::ViewType viewType, Component *pComponent)
 {
   mValid = true;
+  mpComponent = pComponent;
   initialize(viewType);
 }
 
@@ -119,39 +119,30 @@ void Transformation::parseTransformationString(QString value, qreal width, qreal
       // origin x position
       bool hasExtent1X, hasExtent1Y, hasExtent2X, hasExtent2Y, hasRotation = false;
       mOriginIcon.setX(list.at(8).toFloat(&mHasOriginIconX));
-      if (!mHasOriginIconX) {
-        mOriginIcon.setX(mOriginDiagram.x());
-      }
       // origin y position
       mOriginIcon.setY(list.at(9).toFloat(&mHasOriginIconY));
-      if (!mHasOriginIconY) {
-        mOriginIcon.setY(mOriginDiagram.y());
-      }
       // extent1 x
       mExtent1Icon.setX(list.at(10).toFloat(&hasExtent1X));
-      if (!hasExtent1X) {
-        mExtent1Icon.setX(mExtent1Diagram.x());
-      }
       // extent1 y
       mExtent1Icon.setY(list.at(11).toFloat(&hasExtent1Y));
-      if (!hasExtent1Y) {
-        mExtent1Icon.setY(mExtent1Diagram.y());
-      }
       // extent1 x
       mExtent2Icon.setX(list.at(12).toFloat(&hasExtent2X));
-      if (!hasExtent2X) {
-        mExtent2Icon.setX(mExtent2Diagram.x());
-      }
       // extent1 y
       mExtent2Icon.setY(list.at(13).toFloat(&hasExtent2Y));
-      if (!hasExtent2Y) {
-        mExtent2Icon.setY(mExtent2Diagram.y());
-      }
       // rotate angle
       if (list.size() > 14) {
         mRotateAngleIcon = list.at(14).toFloat(&hasRotation);
       }
-      if (!hasRotation) {
+      /* Ticket:4215
+       * Only use transformation values when no iconTransformation value is available. Don't mix.
+       */
+      if (!mHasOriginIconX && !mHasOriginIconY && !hasExtent1X && !hasExtent1Y && !hasExtent2X && !hasExtent2Y && !hasRotation) {
+        mOriginIcon.setX(mOriginDiagram.x());
+        mOriginIcon.setY(mOriginDiagram.y());
+        mExtent1Icon.setX(mExtent1Diagram.x());
+        mExtent1Icon.setY(mExtent1Diagram.y());
+        mExtent2Icon.setX(mExtent2Diagram.x());
+        mExtent2Icon.setY(mExtent2Diagram.y());
         mRotateAngleIcon = mRotateAngleDiagram;
       }
     }
@@ -161,6 +152,7 @@ void Transformation::parseTransformationString(QString value, qreal width, qreal
 void Transformation::updateTransformation(const Transformation &transformation)
 {
   mValid = transformation.isValid();
+  mpComponent = transformation.getComponent();
   mViewType = transformation.getViewType();
   mWidth = transformation.getWidth();
   mHeight = transformation.getHeight();
@@ -341,6 +333,10 @@ QTransform Transformation::getTransformationMatrixDiagram()
   mPositionDiagram.setX(mOriginDiagram.x() + ((mExtent1Diagram.x() + mExtent2Diagram.x()) / 2));
   // calculate Y position
   mPositionDiagram.setY(mOriginDiagram.y() + ((mExtent1Diagram.y() + mExtent2Diagram.y()) / 2));
+  /* Ticket #3032. Adjust position based on the coordinate system of the component. */
+  if (mpComponent) {
+    mPositionDiagram = mPositionDiagram - (mpComponent->boundingRect().center() * mpComponent->getCoOrdinateSystem().getInitialScale());
+  }
   // get scale
   qreal tempwidth = fabs(mExtent1Diagram.x() - mExtent2Diagram.x());
   qreal sx = tempwidth / mWidth;
@@ -400,6 +396,10 @@ QTransform Transformation::getTransformationMatrixIcon()
   mPositionIcon.setX(mOriginIcon.x() + ((mExtent1Icon.x() + mExtent2Icon.x()) / 2));
   // calculate Y position
   mPositionIcon.setY(mOriginIcon.y() + ((mExtent1Icon.y() + mExtent2Icon.y()) / 2));
+  /* Ticket #3032. Adjust position based on the coordinate system of the component. */
+  if (mpComponent) {
+    mPositionIcon = mPositionIcon - (mpComponent->boundingRect().center() * mpComponent->getCoOrdinateSystem().getInitialScale());
+  }
   // get scale
   qreal tempwidth = fabs(mExtent1Icon.x() - mExtent2Icon.x());
   qreal sx = tempwidth / mWidth;
@@ -431,14 +431,14 @@ QTransform Transformation::getTransformationMatrixIcon()
 void Transformation::adjustPositionIcon(qreal x, qreal y)
 {
   // determine X position
-  if (mExtent1Icon.x() + mExtent2Icon.x() == 0) {
+  if (mHasOriginIconX) {
     mOriginIcon.setX(mOriginIcon.x() +  x);
   } else {
     mExtent1Icon.setX(mExtent1Icon.x() +  x);
     mExtent2Icon.setX(mExtent2Icon.x() +  x);
   }
   // determine Y position
-  if (mExtent1Icon.y() + mExtent2Icon.y() == 0) {
+  if (mHasOriginIconY) {
     mOriginIcon.setY(mOriginIcon.y() +  y);
   } else {
     mExtent1Icon.setY(mExtent1Icon.y() +  y);
