@@ -28,27 +28,33 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- *
  */
 
-#include "LocalsWidget.h"
-#include "ModelicaValue.h"
-#include "CommandFactory.h"
+#include "Debugger/Locals/LocalsWidget.h"
+#include "MainWindow.h"
+#include "Modeling/LibraryTreeWidget.h"
+#include "Debugger/GDB/GDBAdapter.h"
+#include "Debugger/StackFrames/StackFramesWidget.h"
+#include "Debugger/Locals/ModelicaValue.h"
+#include "Debugger/GDB/CommandFactory.h"
+#include "Util/Helper.h"
 
 /*!
-  \class LocalsTreeItem
-  \brief Contains the information about the local variable.
-  */
+ * \class LocalsTreeItem
+ * \brief Contains the information about the local variable.
+ */
 /*!
-  \param localItemData - a list of items.\n
-  0 -> name\n
-  1 -> displayName\n
-  2 -> type\n
-  3 -> value
-  */
+ * \brief LocalsTreeItem::LocalsTreeItem
+ * \param localItemData - a list of items.\n
+ * 0 -> name\n
+ * 1 -> displayName\n
+ * 2 -> type\n
+ * 3 -> value
+ * \param localItemData
+ * \param pLocalsTreeModel
+ * \param pLocalsTreeItem
+ */
 LocalsTreeItem::LocalsTreeItem(const QVector<QVariant> &localItemData, LocalsTreeModel *pLocalsTreeModel, LocalsTreeItem *pLocalsTreeItem)
   : QObject(pLocalsTreeModel)
 {
@@ -60,22 +66,18 @@ LocalsTreeItem::LocalsTreeItem(const QVector<QVariant> &localItemData, LocalsTre
   setNameStructure("");
   setType(localItemData[2].toString());
   /* if the item is a root item then its just a header */
-  if (!mpParentLocalsTreeItem)
-  {
+  if (!mpParentLocalsTreeItem) {
     setDisplayName(getName());
     setDisplayType(getType());
     setDisplayValue(localItemData[3].toString());
-  }
-  /* if the item is a top level item then we need to fetch the type and value. */
-  else if (mpParentLocalsTreeItem == mpLocalsTreeModel->getRootLocalsTreeItem())
-  {
+  } else if (mpParentLocalsTreeItem == mpLocalsTreeModel->getRootLocalsTreeItem()) {
+    /* if the item is a top level item then we need to fetch the type and value. */
     setDisplayType("");
     retrieveType();
     setDisplayValue("");
     retrieveValue();
-  }
-  else  /* child node */
-  {
+  } else {
+    /* child node */
     setDisplayName(getDisplayName());
     setDisplayType(getType());
     setDisplayValue("");
@@ -91,25 +93,37 @@ LocalsTreeItem::~LocalsTreeItem()
   mChildren.clear();
 }
 
+/*!
+ * \brief LocalsTreeItem::isCoreType
+ * Returns true if is core type.
+ * \return
+ */
 bool LocalsTreeItem::isCoreType()
 {
   if ((getDisplayType().compare(Helper::STRING) == 0) ||
       (getDisplayType().compare(Helper::BOOLEAN) == 0) ||
       (getDisplayType().compare(Helper::INTEGER) == 0) ||
-      (getDisplayType().compare(Helper::REAL) == 0))
+      (getDisplayType().compare(Helper::REAL) == 0)) {
     return true;
-  else
+  } else {
     return false;
+  }
 }
 
+/*!
+ * \brief LocalsTreeItem::isCoreTypeExceptString
+ * Returns true if is core type except STRING.
+ * \return
+ */
 bool LocalsTreeItem::isCoreTypeExceptString()
 {
   if ((getDisplayType().compare(Helper::BOOLEAN) == 0) ||
       (getDisplayType().compare(Helper::INTEGER) == 0) ||
-      (getDisplayType().compare(Helper::REAL) == 0))
+      (getDisplayType().compare(Helper::REAL) == 0)) {
     return true;
-  else
+  } else {
     return false;
+  }
 }
 
 void LocalsTreeItem::insertChild(int position, LocalsTreeItem *pLocalsTreeItem)
@@ -138,33 +152,27 @@ int LocalsTreeItem::columnCount() const
   return 3;
 }
 
-bool LocalsTreeItem::setData(int column, const QVariant &value, int role)
-{
-  Q_UNUSED(column);
-  Q_UNUSED(value);
-  Q_UNUSED(role);
-  return false;
-}
-
 QVariant LocalsTreeItem::data(int column, int role) const
 {
-  switch (role)
-  {
+  switch (role) {
     case Qt::DisplayRole:
-      switch (column)
-      {
+      switch (column) {
         case 0:
           return getDisplayName();
         case 1:
           return getDisplayType();
         case 2:
-          return getDisplayValue();
+          // display the string type with quotes.
+          if (getDisplayType().compare(Helper::STRING) == 0) {
+            return QString("\"%1\"").arg(getDisplayValue());
+          } else {
+            return getDisplayValue();
+          }
         default:
           break;
       }
     case Qt::ToolTipRole:
-      switch (column)
-      {
+      switch (column) {
         case 0:
           return getDisplayName();
         case 1:
@@ -184,9 +192,9 @@ QVariant LocalsTreeItem::data(int column, int role) const
 
 int LocalsTreeItem::row() const
 {
-  if (mpParentLocalsTreeItem)
+  if (mpParentLocalsTreeItem) {
     return mpParentLocalsTreeItem->mChildren.indexOf(const_cast<LocalsTreeItem*>(this));
-
+  }
   return 0;
 }
 
@@ -212,59 +220,64 @@ void LocalsTreeItem::retrieveType()
 
 void LocalsTreeItem::retrieveModelicaMetaType()
 {
-  if (getDisplayType().isEmpty()
-      || (getDisplayType().compare(Helper::VALUE_OPTIMIZED_OUT) == 0)
-      || (getDisplayType().compare(Helper::REPLACEABLE_TYPE_ANY) == 0))
-  {
-    GDBAdapter *pGDBAdapter = mpLocalsTreeModel->getLocalsWidget()->getDebuggerMainWindow()->getGDBAdapter();
-    pGDBAdapter->postCommand(CommandFactory::getTypeOfAny(getName()), this, &GDBAdapter::getTypeOfAnyCB);
-  }
-  else
-  {
+  if (getDisplayType().isEmpty() || (getDisplayType().compare(Helper::VALUE_OPTIMIZED_OUT) == 0)
+      || (getDisplayType().compare(Helper::REPLACEABLE_TYPE_ANY) == 0)) {
+    GDBAdapter *pGDBAdapter = GDBAdapter::instance();
+    StackFramesWidget *pStackFramesWidget = MainWindow::instance()->getStackFramesWidget();
+    if (parent() && parent()->getModelicaValue() && qobject_cast<ModelicaRecordValue*>(parent()->getModelicaValue())) {
+      pGDBAdapter->postCommand(CommandFactory::getTypeOfAny(pStackFramesWidget->getSelectedThread(), pStackFramesWidget->getSelectedFrame(),
+                                                            getName(), true),
+                               GDBAdapter::BlockUntilResponse, this, &GDBAdapter::getTypeOfAnyCB);
+    } else {
+      pGDBAdapter->postCommand(CommandFactory::getTypeOfAny(pStackFramesWidget->getSelectedThread(), pStackFramesWidget->getSelectedFrame(),
+                                                            getName(), false),
+                               GDBAdapter::BlockUntilResponse, this, &GDBAdapter::getTypeOfAnyCB);
+    }
+  } else {
     retrieveValue();
   }
 }
 
+/*!
+ * \brief LocalsTreeItem::retrieveValue
+ * Gets the value of the LocalsTreeItem using CommandFactory::anyString
+ */
 void LocalsTreeItem::retrieveValue()
 {
-  GDBAdapter *pGDBAdapter = mpLocalsTreeModel->getLocalsWidget()->getDebuggerMainWindow()->getGDBAdapter();
-  if ((getDisplayType().compare(Helper::STRING) == 0) || (parent() != mpLocalsTreeModel->getRootLocalsTreeItem()))
-  {
-    pGDBAdapter->postCommand(CommandFactory::anyString(getName()), this, &GDBAdapter::anyStringCB);
-  }
-  else if (isCoreTypeExceptString())
-  {
-    pGDBAdapter->postCommand(CommandFactory::dataEvaluateExpression(getName()), this, &GDBAdapter::dataEvaluateExpressionCB);
-  }
-  else
-  {
+  GDBAdapter *pGDBAdapter = GDBAdapter::instance();
+  StackFramesWidget *pStackFramesWidget = MainWindow::instance()->getStackFramesWidget();
+  if (isCoreTypeExceptString()) {
+    pGDBAdapter->postCommand(CommandFactory::dataEvaluateExpression(pStackFramesWidget->getSelectedThread(),
+                                                                    pStackFramesWidget->getSelectedFrame(), getName()),
+                             GDBAdapter::BlockUntilResponse, this, &GDBAdapter::dataEvaluateExpressionCB);
+  } else if (isCoreType()) {
+    pGDBAdapter->postCommand(CommandFactory::anyString(pStackFramesWidget->getSelectedThread(), pStackFramesWidget->getSelectedFrame(),
+                                                       getName()), GDBAdapter::BlockUntilResponse, this, &GDBAdapter::anyStringCB);
+  } else {
     setValue(getDisplayType());
   }
 }
 
+/*!
+ * \brief LocalsTreeItem::setModelicaMetaType
+ * Sets the type of the LocalsTreeItem and sends the command to retireve value.
+ * \param type
+ */
 void LocalsTreeItem::setModelicaMetaType(QString type)
 {
   setDisplayType(type);
-  if (getDisplayType().compare(Helper::REPLACEABLE_TYPE_ANY) == 0)
-  {
+  if (getDisplayType().compare(Helper::REPLACEABLE_TYPE_ANY) == 0) {
     setDisplayValue(tr("<uninitialized variable>"));
-  }
-  else if (isCoreType())
-  {
+  } else {
     retrieveValue();
   }
-  else
-  {
-    setValue(getDisplayType());
-  }
-  /* invalidate the view so that the items show the updated values. */
-  mpLocalsTreeModel->getLocalsWidget()->getLocalsTreeProxyModel()->invalidate();
+  /* update the view with new values of LocalsTreeItem */
+  mpLocalsTreeModel->updateLocalsTreeItem(this);
 }
 
 void LocalsTreeItem::setValue(QString value)
 {
-  if (mpModelicaValue)
-  {
+  if (mpModelicaValue) {
     QString previousValue = mpModelicaValue->getValueString();
     mpModelicaValue->setValue(value);
     setDisplayValue(mpModelicaValue->getValueString());
@@ -274,57 +287,46 @@ void LocalsTreeItem::setValue(QString value)
     } else {
       setValueChanged(true);
     }
-  }
-  else if (isCoreType())
-  {
+  } else if (isCoreType()) {
     mpModelicaValue = new ModelicaCoreValue(this);
     mpModelicaValue->setValue(value);
     setDisplayValue(mpModelicaValue->getValueString());
-  }
-  else if (getDisplayType().startsWith(Helper::RECORD))
-  {
+  } else if (getDisplayType().startsWith(Helper::RECORD)) {
     mpModelicaValue = new ModelicaRecordValue(this);
     mpModelicaValue->setValue(value);
     setDisplayValue(mpModelicaValue->getValueString());
     /* get the record elements size */
     mpModelicaValue->retrieveChildrenSize();
-  }
-  else if (getDisplayType().startsWith(Helper::LIST))
-  {
+  } else if (getDisplayType().startsWith(Helper::LIST)) {
     mpModelicaValue = new ModelicaListValue(this);
     setDisplayValue(mpModelicaValue->getValueString());
     /* get the list items size */
     mpModelicaValue->retrieveChildrenSize();
-  }
-  else if (getDisplayType().startsWith(Helper::OPTION))
-  {
+  } else if (getDisplayType().startsWith(Helper::OPTION)) {
     mpModelicaValue = new ModelicaOptionValue(this);
     setDisplayValue(mpModelicaValue->getValueString());
     /* get the option item elements size */
     mpModelicaValue->retrieveChildrenSize();
-  }
-  else if (getDisplayType().startsWith(Helper::TUPLE))
-  {
+  } else if (getDisplayType().startsWith(Helper::TUPLE)) {
     mpModelicaValue = new ModelicaTupleValue(this);
     setDisplayValue(mpModelicaValue->getValueString());
     /* get the tuple elements size */
     mpModelicaValue->retrieveChildrenSize();
-  }
-  else if (getDisplayType().startsWith(Helper::ARRAY))
-  {
+  } else if (getDisplayType().startsWith(Helper::ARRAY)) {
     mpModelicaValue = new MetaModelicaArrayValue(this);
     setDisplayValue(mpModelicaValue->getValueString());
     /* get the tuple elements size */
     mpModelicaValue->retrieveChildrenSize();
   }
-  /* invalidate the view so that the items show the updated values. */
-  mpLocalsTreeModel->getLocalsWidget()->getLocalsTreeProxyModel()->invalidate();
+  /* update the view with new values of LocalsTreeItem */
+  mpLocalsTreeModel->updateLocalsTreeItem(this);
 }
 
 void LocalsTreeItem::retrieveLocalChildren()
 {
-  if (!mpModelicaValue)
+  if (!mpModelicaValue) {
     return;
+  }
   mpModelicaValue->retrieveChildren();
 }
 
@@ -339,30 +341,34 @@ LocalsTreeModel::LocalsTreeModel(LocalsWidget *pLocalsWidget)
 
 int LocalsTreeModel::columnCount(const QModelIndex &parent) const
 {
-  if (parent.isValid())
+  if (parent.isValid()) {
     return static_cast<LocalsTreeItem*>(parent.internalPointer())->columnCount();
-  else
+  } else {
     return mpRootLocalsTreeItem->columnCount();
+  }
 }
 
 int LocalsTreeModel::rowCount(const QModelIndex &parent) const
 {
   LocalsTreeItem *pParentLocalsTreeViewItem;
-  if (parent.column() > 0)
+  if (parent.column() > 0) {
     return 0;
+  }
 
-  if (!parent.isValid())
+  if (!parent.isValid()) {
     pParentLocalsTreeViewItem = mpRootLocalsTreeItem;
-  else
+  } else {
     pParentLocalsTreeViewItem = static_cast<LocalsTreeItem*>(parent.internalPointer());
+  }
   return pParentLocalsTreeViewItem->getChildren().size();
 }
 
 bool LocalsTreeModel::hasChildren(const QModelIndex &parent) const
 {
   LocalsTreeItem *pParentLocalsTreeViewItem = static_cast<LocalsTreeItem*>(parent.internalPointer());
-  if (!pParentLocalsTreeViewItem)
+  if (!pParentLocalsTreeViewItem) {
     return true;
+  }
   return pParentLocalsTreeViewItem->getModelicaValue() && pParentLocalsTreeViewItem->getModelicaValue()->hasChildren();
 }
 
@@ -376,58 +382,53 @@ bool LocalsTreeModel::canFetchMore(const QModelIndex &parent) const
 
 QVariant LocalsTreeModel::headerData(int section, Qt::Orientation orientation, int role) const
 {
-  if (orientation == Qt::Horizontal && role == Qt::DisplayRole)
+  if (orientation == Qt::Horizontal && role == Qt::DisplayRole) {
     return mpRootLocalsTreeItem->data(section);
+  }
   return QVariant();
 }
 
 QModelIndex LocalsTreeModel::index(int row, int column, const QModelIndex &parent) const
 {
-  if (!hasIndex(row, column, parent))
+  if (!hasIndex(row, column, parent)) {
     return QModelIndex();
+  }
 
   LocalsTreeItem *pParentLocalsTreeViewItem;
-
-  if (!parent.isValid())
+  if (!parent.isValid()) {
     pParentLocalsTreeViewItem = mpRootLocalsTreeItem;
-  else
+  } else {
     pParentLocalsTreeViewItem = static_cast<LocalsTreeItem*>(parent.internalPointer());
+  }
 
   LocalsTreeItem *pChildLocalsTreeViewItem = pParentLocalsTreeViewItem->child(row);
-  if (pChildLocalsTreeViewItem)
+  if (pChildLocalsTreeViewItem) {
     return createIndex(row, column, pChildLocalsTreeViewItem);
-  else
+  } else {
     return QModelIndex();
+  }
 }
 
 QModelIndex LocalsTreeModel::parent(const QModelIndex &index) const
 {
-  if (!index.isValid())
+  if (!index.isValid()) {
     return QModelIndex();
+  }
 
   LocalsTreeItem *pChildLocalsTreeViewItem = static_cast<LocalsTreeItem*>(index.internalPointer());
   LocalsTreeItem *pParentLocalsTreeViewItem = pChildLocalsTreeViewItem->parent();
-  if (pParentLocalsTreeViewItem == mpRootLocalsTreeItem)
+  if (pParentLocalsTreeViewItem == mpRootLocalsTreeItem) {
     return QModelIndex();
+  }
 
   return createIndex(pParentLocalsTreeViewItem->row(), 0, pParentLocalsTreeViewItem);
 }
 
-bool LocalsTreeModel::setData(const QModelIndex &index, const QVariant &value, int role)
-{
-  Q_UNUSED(role);
-  LocalsTreeItem *pLocalsTreeViewItem = static_cast<LocalsTreeItem*>(index.internalPointer());
-  if (!pLocalsTreeViewItem)
-    return false;
-  bool result = pLocalsTreeViewItem->setData(index.column(), value);
-  emit dataChanged(index, index);
-  return result;
-}
-
 QVariant LocalsTreeModel::data(const QModelIndex &index, int role) const
 {
-  if (!index.isValid())
+  if (!index.isValid()) {
     return QVariant();
+  }
 
   LocalsTreeItem *pLocalsTreeViewItem = static_cast<LocalsTreeItem*>(index.internalPointer());
   return pLocalsTreeViewItem->data(index.column(), role);
@@ -435,11 +436,14 @@ QVariant LocalsTreeModel::data(const QModelIndex &index, int role) const
 
 LocalsTreeItem* LocalsTreeModel::findLocalsTreeItem(const QString &name, LocalsTreeItem *root) const
 {
-  if (root->getNameStructure() == name)
+  if (root->getNameStructure() == name) {
     return root;
-  for (int i = root->getChildren().size(); --i >= 0; )
-    if (LocalsTreeItem *item = findLocalsTreeItem(name, root->getChildren().at(i)))
+  }
+  for (int i = root->getChildren().size(); --i >= 0;) {
+    if (LocalsTreeItem *item = findLocalsTreeItem(name, root->getChildren().at(i))) {
       return item;
+    }
+  }
   return 0;
 }
 
@@ -451,14 +455,16 @@ QModelIndex LocalsTreeModel::localsTreeItemIndex(const LocalsTreeItem *pLocalsTr
 QModelIndex LocalsTreeModel::localsTreeItemIndexHelper(const LocalsTreeItem *pLocalsTreeItem, const LocalsTreeItem *pParentLocalsTreeItem,
                                                        const QModelIndex &parentIndex) const
 {
-  if (pLocalsTreeItem == pParentLocalsTreeItem)
+  if (pLocalsTreeItem == pParentLocalsTreeItem) {
     return parentIndex;
+  }
   for (int i = pParentLocalsTreeItem->getChildren().size(); --i >= 0; ) {
     const LocalsTreeItem *childItem = pParentLocalsTreeItem->getChildren().at(i);
     QModelIndex childIndex = index(i, 0, parentIndex);
     QModelIndex index = localsTreeItemIndexHelper(pLocalsTreeItem, childItem, childIndex);
-    if (index.isValid())
+    if (index.isValid()) {
       return index;
+    }
   }
   return QModelIndex();
 }
@@ -467,26 +473,20 @@ void LocalsTreeModel::insertLocalItemData(const QVector<QVariant> &localItemData
 {
   QString nameStructure;
   /* construct the item name structure. */
-  if (mpRootLocalsTreeItem == pParentLocalsTreeItem)
-  {
+  if (mpRootLocalsTreeItem == pParentLocalsTreeItem) {
     nameStructure = QString("%1.%2").arg(pParentLocalsTreeItem->getNameStructure()).arg(localItemData[0].toString());
-  }
-  else
-  {
+  } else {
     nameStructure = QString("%1.%2%3").arg(pParentLocalsTreeItem->getNameStructure()).arg(localItemData[0].toString()).arg(localItemData[1].toString());
   }
   /* find the item */
   LocalsTreeItem *pLocalsTreeItem = findLocalsTreeItem(nameStructure, pParentLocalsTreeItem);
-  if (pLocalsTreeItem)
-  {
+  if (pLocalsTreeItem) {
     pLocalsTreeItem->retrieveModelicaMetaType();
-  }
-  else
-  {
+  } else {
     QModelIndex index = localsTreeItemIndex(pParentLocalsTreeItem);
     pLocalsTreeItem = new LocalsTreeItem(localItemData, this, pParentLocalsTreeItem);
     pLocalsTreeItem->setNameStructure(nameStructure);
-    int row = pLocalsTreeItem->getChildren().size();
+    int row = pParentLocalsTreeItem->getChildren().size();
     beginInsertRows(index, row, row);
     pParentLocalsTreeItem->insertChild(row, pLocalsTreeItem);
     endInsertRows();
@@ -496,38 +496,31 @@ void LocalsTreeModel::insertLocalItemData(const QVector<QVariant> &localItemData
 void LocalsTreeModel::insertLocalsList(const QList<QVector<QVariant> > &locals)
 {
   QList<LocalsTreeItem*> localsTreeItems = mpRootLocalsTreeItem->getChildren();
-  foreach (LocalsTreeItem *pLocalsTreeItem, localsTreeItems)
-  {
+  foreach (LocalsTreeItem *pLocalsTreeItem, localsTreeItems) {
     bool isFound = false;
-    foreach (QVector<QVariant> local, locals)
-    {
+    foreach (QVector<QVariant> local, locals) {
       QString nameStructure;
       /* construct the item name structure. */
-      if (pLocalsTreeItem->parent() == mpRootLocalsTreeItem)
-      {
+      if (pLocalsTreeItem->parent() == mpRootLocalsTreeItem) {
         nameStructure = QString("%1.%2").arg(pLocalsTreeItem->parent()->getNameStructure()).arg(local[0].toString());
-      }
-      else
-      {
+      } else {
         nameStructure = QString("%1.%2%3").arg(pLocalsTreeItem->parent()->getNameStructure()).arg(local[0].toString()).arg(local[1].toString());
       }
       /* compare the name structure to find the item. */
-      if (nameStructure.compare(pLocalsTreeItem->getNameStructure()) == 0)
-      {
+      if (nameStructure.compare(pLocalsTreeItem->getNameStructure()) == 0) {
         isFound = true;
         break;
       }
     }
     /* if not found then the item has been removed from the stack so we must also remove it from the locals tree. */
-    if (!isFound)
-    {
+    if (!isFound) {
       removeLocalItem(pLocalsTreeItem);
     }
   }
-  foreach (QVector<QVariant> local, locals)
-  {
+  foreach (QVector<QVariant> local, locals) {
     insertLocalItemData(local, mpRootLocalsTreeItem);
   }
+  mpLocalsWidget->getLocalsTreeProxyModel()->invalidate();
 }
 
 void LocalsTreeModel::removeLocalItem(LocalsTreeItem *pLocalsTreeItem)
@@ -545,47 +538,85 @@ void LocalsTreeModel::removeLocalItem(LocalsTreeItem *pLocalsTreeItem)
 void LocalsTreeModel::removeLocalItems()
 {
   int n = mpRootLocalsTreeItem->getChildren().size();
-  if (n == 0)
-      return;
+  if (n == 0) {
+    return;
+  }
   QModelIndex index = localsTreeItemIndex(mpRootLocalsTreeItem);
   beginRemoveRows(index, 0, n - 1);
   mpRootLocalsTreeItem->removeChildren();
   endRemoveRows();
 }
 
+/*!
+ * \brief LocalsTreeModel::updateLocalsTreeItem
+ * Triggers a view update for the LocalsTreeItem in the Locals Browser.
+ * \param pLocalsTreeItem
+ */
+void LocalsTreeModel::updateLocalsTreeItem(LocalsTreeItem *pLocalsTreeItem)
+{
+  QModelIndex index = localsTreeItemIndex(pLocalsTreeItem);
+  emit dataChanged(index, index);
+}
+
+/*!
+ * \class LocalsTreeProxyModel
+ * \brief A sort filter proxy model for Locals Browser.
+ */
+/*!
+ * \brief LocalsTreeProxyModel::LocalsTreeProxyModel
+ * \param parent
+ */
 LocalsTreeProxyModel::LocalsTreeProxyModel(QObject *parent)
   : QSortFilterProxyModel(parent)
 {
 }
 
-void LocalsTreeProxyModel::clearfilter()
+/*!
+ * \brief LocalsTreeProxyModel::lessThan
+ * Sorts the LocalsTreeItems except for members of a record type.
+ * \param left
+ * \param right
+ * \return
+ */
+bool LocalsTreeProxyModel::lessThan(const QModelIndex &left, const QModelIndex &right) const
 {
-  invalidateFilter();
-}
-
-bool LocalsTreeProxyModel::filterAcceptsRow(int sourceRow, const QModelIndex &sourceParent) const
-{
-  return QSortFilterProxyModel::filterAcceptsRow(sourceRow, sourceParent);
+  /* Ticket:4078
+   * Do not sort the record members.
+   */
+  LocalsTreeItem *pLocalsTreeItem = static_cast<LocalsTreeItem*>(left.internalPointer());
+  if (pLocalsTreeItem && pLocalsTreeItem->parent() && pLocalsTreeItem->parent()->getModelicaValue() &&
+      qobject_cast<ModelicaRecordValue*>(pLocalsTreeItem->parent()->getModelicaValue())) {
+    return false;
+  } else {
+    return QSortFilterProxyModel::lessThan(left, right);
+  }
 }
 
 LocalsTreeView::LocalsTreeView(LocalsWidget *pLocalsWidget)
   : QTreeView(pLocalsWidget)
 {
   mpLocalsWidget = pLocalsWidget;
-  setObjectName("TreeWithBranches");
-  setItemDelegate(new ItemDelegate(this, false, true));
+  setItemDelegate(new ItemDelegate(this));
   setTextElideMode(Qt::ElideMiddle);
   setIndentation(Helper::treeIndentation);
   setContextMenuPolicy(Qt::CustomContextMenu);
   setExpandsOnDoubleClick(false);
   setSortingEnabled(true);
   sortByColumn(0, Qt::AscendingOrder);
+  setUniformRowHeights(true);
 }
 
-LocalsWidget::LocalsWidget(DebuggerMainWindow *pDebuggerMainWindow)
-  : QWidget(pDebuggerMainWindow)
+/*!
+ * \class LocalsWidget
+ * \brief A widget containing local variables with type and values while debugging.
+ */
+/*!
+ * \brief LocalsWidget::LocalsWidget
+ * \param pParent
+ */
+LocalsWidget::LocalsWidget(QWidget *pParent)
+  : QWidget(pParent)
 {
-  mpDebuggerMainWindow = pDebuggerMainWindow;
   /* Locals Tree View */
   mpLocalsTreeView = new LocalsTreeView(this);
   mpLocalsTreeModel = new LocalsTreeModel(this);
@@ -611,15 +642,16 @@ LocalsWidget::LocalsWidget(DebuggerMainWindow *pDebuggerMainWindow)
   pMainLayout->setContentsMargins(0, 0, 1, 0);
   pMainLayout->addWidget(pLocalsSplitter, 0, 0);
   setLayout(pMainLayout);
-  connect(mpDebuggerMainWindow->getGDBAdapter(), SIGNAL(GDBProcessFinished()), SLOT(handleGDBProcessFinished()));
+  connect(GDBAdapter::instance(), SIGNAL(GDBProcessFinished()), SLOT(handleGDBProcessFinished()));
 }
 
 void LocalsWidget::localsTreeItemExpanded(QModelIndex index)
 {
   index = mpLocalsTreeProxyModel->mapToSource(index);
   LocalsTreeItem *pLocalsTreeItem = static_cast<LocalsTreeItem*>(index.internalPointer());
-  if (!pLocalsTreeItem || pLocalsTreeItem->isExpanded())
+  if (!pLocalsTreeItem || pLocalsTreeItem->isExpanded()) {
     return;
+  }
 
   pLocalsTreeItem->setExpanded(true);
   pLocalsTreeItem->retrieveLocalChildren();
@@ -631,16 +663,21 @@ void LocalsWidget::showLocalValue(QModelIndex currentIndex, QModelIndex previous
 
   currentIndex = mpLocalsTreeProxyModel->mapToSource(currentIndex);
   LocalsTreeItem *pLocalsTreeItem = static_cast<LocalsTreeItem*>(currentIndex.internalPointer());
-  if (!pLocalsTreeItem)
+  if (!pLocalsTreeItem) {
     return;
-
-  mpLocalValueViewer->setPlainText(pLocalsTreeItem->getDisplayValue());
+  }
+  if (pLocalsTreeItem->getDisplayType().compare(Helper::STRING) == 0) {
+    mpLocalValueViewer->setPlainText(QString("\"%1\"").arg(pLocalsTreeItem->getDisplayValue()));
+  } else {
+    mpLocalValueViewer->setPlainText(pLocalsTreeItem->getDisplayValue());
+  }
 }
 
 /*!
-  Slot activated when GDBProcessFinished signal of GDBAdapter is raised.
-  Clears the LocalsTreeView by removing all the items.
-  */
+ * \brief LocalsWidget::handleGDBProcessFinished
+ * Slot activated when GDBProcessFinished signal of GDBAdapter is raised.
+ * Clears the LocalsTreeView by removing all the items.
+ */
 void LocalsWidget::handleGDBProcessFinished()
 {
   mpLocalsTreeModel->removeLocalItems();
