@@ -28,13 +28,13 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- *
  */
 
 #include "TransformationsEditor.h"
+#include "Util/Helper.h"
+
+#include <QMenu>
 
 /*!
   \class TransformationsEditor
@@ -45,7 +45,7 @@
   \param pTransformationsWidget - pointer to TransformationsWidget
   */
 TransformationsEditor::TransformationsEditor(TransformationsWidget *pTransformationsWidget)
-  : BaseEditor(pTransformationsWidget->getMainWindow())
+  : BaseEditor(pTransformationsWidget)
 {
   mpTransformationsWidget = pTransformationsWidget;
 }
