@@ -29,28 +29,22 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- *
  */
 
 #ifndef MESSAGESWIDGET_H
 #define MESSAGESWIDGET_H
 
-#include "MainWindow.h"
+#include "Util/StringHandler.h"
 
-class MainWindow;
-class StringHandler;
-
-class MessagesWidget;
+#include <QTextBrowser>
 
 class MessageItem
 {
 public:
   enum MessageItemType {
     Modelica,   /* Used to represent error messages of Modelica models. */
-    TLM         /* Used to represent error messages of TLM files. */
+    MetaModel   /* Used to represent error messages of MetaModel files. */
   };
   QString mTime;
   QString mFileName;
@@ -82,14 +76,21 @@ class MessagesWidget : public QWidget
 {
   Q_OBJECT
 private:
-  MainWindow *mpMainWindow;
+  // the only class that is allowed to create and destroy
+  friend class MainWindow;
+
+  static void create();
+  static void destroy();
+  MessagesWidget(QWidget *pParent = 0);
+
+  static MessagesWidget *mpInstance;
   int mMessageNumber;
   QTextBrowser *mpMessagesTextBrowser;
   QAction *mpSelectAllAction;
   QAction *mpCopyAction;
   QAction *mpClearAllAction;
 public:
-  MessagesWidget(MainWindow *pMainWindow);
+  static MessagesWidget* instance() {return mpInstance;}
   void resetMessagesNumber() {mMessageNumber = 1;}
   QTextBrowser* getMessagesTextBrowser() {return mpMessagesTextBrowser;}
   void applyMessagesSettings();
