@@ -29,21 +29,30 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- *
  */
 
 #ifndef MODELICACLASSDIALOG_H
 #define MODELICACLASSDIALOG_H
 
-#include "MainWindow.h"
+#include <QDialog>
+#include <QLineEdit>
+#include <QTreeView>
+#include <QPushButton>
+#include <QDialogButtonBox>
+#include <QComboBox>
+#include <QCheckBox>
+#include <QGroupBox>
+#include <QTableWidget>
+#include <QPlainTextEdit>
+#include <QListWidget>
+#include <QToolButton>
 
-class MainWindow;
 class Label;
 class LibraryWidget;
 class LibraryTreeProxyModel;
+class TreeSearchFilters;
+class DoubleSpinBox;
 
 class LibraryBrowseDialog : public QDialog
 {
@@ -68,11 +77,9 @@ class ModelicaClassDialog : public QDialog
 {
   Q_OBJECT
 public:
-  ModelicaClassDialog(MainWindow *pParent);
-  MainWindow* getMainWindow();
-  QLineEdit* getParentClassTextBox();
+  ModelicaClassDialog(QWidget *pParent = 0);
+  QLineEdit* getParentClassTextBox() {return mpParentClassTextBox;}
 private:
-  MainWindow *mpMainWindow;
   Label *mpNameLabel;
   QLineEdit *mpNameTextBox;
   Label *mpSpecializationLabel;
@@ -100,9 +107,8 @@ class OpenModelicaFile : public QDialog
 {
   Q_OBJECT
 public:
-  OpenModelicaFile(MainWindow *pParent);
+  OpenModelicaFile(QWidget *pParent = 0);
 private:
-  MainWindow *mpMainWindow;
   QStringList mFileNames;
   Label *mpFileLabel;
   QLineEdit *mpFileTextBox;
@@ -127,11 +133,10 @@ class SaveAsClassDialog : public QDialog
 {
   Q_OBJECT
 public:
-  SaveAsClassDialog(ModelWidget *pModelWidget, MainWindow *pParent);
-  QComboBox* getParentClassComboBox();
+  SaveAsClassDialog(ModelWidget *pModelWidget, QWidget *pParent = 0);
+  QComboBox* getParentClassComboBox() {return mpParentClassComboBox;}
 private:
   ModelWidget *mpModelWidget;
-  MainWindow *mpMainWindow;
   Label *mpNameLabel;
   QLineEdit *mpNameTextBox;
   Label *mpParentPackageLabel;
@@ -150,11 +155,10 @@ class DuplicateClassDialog : public QDialog
 {
   Q_OBJECT
 public:
-  DuplicateClassDialog(bool saveAs, LibraryTreeItem *pLibraryTreeItem, MainWindow *pMainWindow);
+  DuplicateClassDialog(bool saveAs, LibraryTreeItem *pLibraryTreeItem, QWidget *pParent = 0);
 private:
   bool mSaveAs;
   LibraryTreeItem *mpLibraryTreeItem;
-  MainWindow *mpMainWindow;
   Label *mpNameLabel;
   QLineEdit *mpNameTextBox;
   Label *mpPathLabel;
@@ -172,9 +176,7 @@ class RenameClassDialog : public QDialog
 {
   Q_OBJECT
 public:
-  RenameClassDialog(QString name, QString nameStructure, MainWindow *parent);
-
-  MainWindow *mpMainWindow;
+  RenameClassDialog(QString name, QString nameStructure, QWidget *pParent = 0);
 private:
   QString mName;
   QString mNameStructure;
@@ -189,10 +191,8 @@ public slots:
 
 class InformationDialog : public QWidget
 {
-private:
-  MainWindow *mpMainWindow;
 public:
-  InformationDialog(QString windowTitle, QString informationText, bool modelicaTextHighlighter = false, MainWindow *pMainWindow = 0);
+  InformationDialog(QString windowTitle, QString informationText, bool modelicaTextHighlighter = false, QWidget *pParent = 0);
   void closeEvent(QCloseEvent *event);
 };
 
@@ -227,12 +227,16 @@ private:
   Label *mpVersionLabel;
   QLineEdit *mpVersionTextBox;
   QGroupBox *mpUsesGroupBox;
+  QList<QList<QString> > mUsesAnnotation;
   QTableWidget *mpUsesTableWidget;
   QToolButton *mpMoveUpButton;
   QToolButton *mpMoveDownButton;
   QToolButton *mpAddUsesAnnotationButton;
   QToolButton *mpRemoveUsesAnnotationButton;
   QDialogButtonBox *mpUsesButtonBox;
+  Label *mpOMCFlagsLabel;
+  QString mOMCFlags;
+  QPlainTextEdit *mpOMCFlagsTextBox;
   QPushButton *mpOkButton;
   QPushButton *mpCancelButton;
   QDialogButtonBox *mpButtonBox;
@@ -248,10 +252,9 @@ class SaveChangesDialog : public QDialog
 {
   Q_OBJECT
 public:
-  SaveChangesDialog(MainWindow *pMainWindow);
+  SaveChangesDialog(QWidget *pParent = 0);
   void listUnSavedClasses();
 private:
-  MainWindow *mpMainWindow;
   Label *mpSaveChangesLabel;
   QListWidget *mpUnsavedClassesListWidget;
   QPushButton *mpYesButton;
@@ -270,9 +273,8 @@ class ExportFigaroDialog : public QDialog
 {
   Q_OBJECT
 public:
-  ExportFigaroDialog(MainWindow *pMainWindow, LibraryTreeItem *pLibraryTreeItem);
+  ExportFigaroDialog(LibraryTreeItem *pLibraryTreeItem, QWidget *pParent = 0);
 private:
-  MainWindow *mpMainWindow;
   LibraryTreeItem *mpLibraryTreeItem;
   Label *mpFigaroModeLabel;
   QComboBox *mpFigaroModeComboBox;
@@ -287,4 +289,61 @@ public slots:
   void exportModelFigaro();
 };
 
+class CreateNewItemDialog : public QDialog
+{
+  Q_OBJECT
+public:
+  CreateNewItemDialog(QString path, bool isCreateFile, QWidget *pParent = 0);
+private:
+  QString mPath;
+  bool mIsCreateFile;
+  Label *mpNameLabel;
+  QLineEdit *mpNameTextBox;
+  Label *mpPathLabel;
+  QLineEdit *mpPathTextBox;
+  QPushButton *mpPathBrowseButton;
+  QPushButton *mpOkButton;
+  QPushButton *mpCancelButton;
+  QDialogButtonBox *mpButtonBox;
+private slots:
+  void browsePath();
+  void createNewFileOrFolder();
+};
+
+class RenameItemDialog : public QDialog
+{
+  Q_OBJECT
+public:
+  RenameItemDialog(LibraryTreeItem *pLibraryTreeItem, QWidget *pParent = 0);
+private:
+  LibraryTreeItem *mpLibraryTreeItem;
+  Label *mpNameLabel;
+  QLineEdit *mpNameTextBox;
+  QPushButton *mpOkButton;
+  QPushButton *mpCancelButton;
+  QDialogButtonBox *mpButtonBox;
+
+  void updateChildrenPath(LibraryTreeItem *pLibraryTreeItem);
+private slots:
+  void renameItem();
+};
+
+class ComponentNameDialog : public QDialog
+{
+  Q_OBJECT
+public:
+  ComponentNameDialog(QString name, GraphicsView *pGraphicsView, QWidget *pParent = 0);
+  QString getComponentName() {return mpNameTextBox->text();}
+private:
+  GraphicsView *mpGraphicsView;
+  Label *mpNameLabel;
+  QLineEdit *mpNameTextBox;
+  QCheckBox *mpDontShowThisMessageAgainCheckBox;
+  QPushButton *mpOkButton;
+  QPushButton *mpCancelButton;
+  QDialogButtonBox *mpButtonBox;
+private slots:
+  void updateComponentName();
+};
+
 #endif // MODELICACLASSDIALOG_H
