@@ -29,11 +29,16 @@
  */
 
 #include "FetchInterfaceDataThread.h"
+#include "Modeling/LibraryTreeWidget.h"
+#include "Options/OptionsDialog.h"
+
+#include <QDir>
 
 FetchInterfaceDataThread::FetchInterfaceDataThread(FetchInterfaceDataDialog *pFetchInterfaceDataDialog)
   : QThread(pFetchInterfaceDataDialog), mpFetchInterfaceDataDialog(pFetchInterfaceDataDialog)
 {
   mpManagerProcess = 0;
+  mManagerProcessId = 0;
   setIsManagerProcessRunning(false);
 }
 
@@ -54,17 +59,19 @@ void FetchInterfaceDataThread::run()
   connect(mpManagerProcess, SIGNAL(finished(int,QProcess::ExitStatus)), SLOT(managerProcessFinished(int,QProcess::ExitStatus)));
   QStringList args;
   args << "-r" << fileInfo.absoluteFilePath();
+  TLMPage *pTLMPage = OptionsDialog::instance()->getTLMPage();
   QProcessEnvironment environment;
 #ifdef WIN32
   environment = StringHandler::simulationProcessEnvironment();
+  environment.insert("PATH", pTLMPage->getTLMPluginPathTextBox()->text() + ";" + environment.value("PATH"));
 #else
   environment = QProcessEnvironment::systemEnvironment();
+  environment.insert("PATH", pTLMPage->getTLMPluginPathTextBox()->text() + ":" + environment.value("PATH"));
 #endif
-  TLMPage *pTLMPage = mpFetchInterfaceDataDialog->getMainWindow()->getOptionsDialog()->getTLMPage();
-  environment.insert("PATH", pTLMPage->getTLMPluginPathTextBox()->text() + ";" + environment.value("PATH"));
   environment.insert("TLMPluginPath", pTLMPage->getTLMPluginPathTextBox()->text());
   mpManagerProcess->setProcessEnvironment(environment);
   mpManagerProcess->start(pTLMPage->getTLMManagerProcessTextBox()->text(), args);
+  mManagerProcessId = Utilities::getProcessId(mpManagerProcess);
   emit sendManagerOutput(QString("%1 %2").arg(pTLMPage->getTLMManagerProcessTextBox()->text()).arg(args.join(" ")), StringHandler::OMEditInfo);
   exec();
 }
@@ -119,5 +126,11 @@ void FetchInterfaceDataThread::managerProcessFinished(int exitCode, QProcess::Ex
     emit sendManagerOutput(mpManagerProcess->errorString() + "\n" + exitCodeStr, StringHandler::Error);
   }
   emit sendManagerFinished(exitCode, exitStatus);
+#ifdef WIN32
+  Utilities::killProcessTreeWindows(mManagerProcessId);
+#else
+  /*! @todo do similar stuff for Linux! */
+#endif /*  WIN32 */
+  quit();
 }
 
