@@ -29,18 +29,16 @@
  *
  */
 /*
- *
  * @author Martin Sjölund <martin.sjolund@liu.se>
- *
- *
  */
 
-#include <QtGui>
-#include <QDebug>
-#include <QXmlStreamReader>
 #include "OMDumpXML.h"
 #include "diff_match_patch.h"
 
+#include <QDebug>
+#include <QXmlStreamReader>
+#include <QTextDocument>
+
 QString OMOperation::toString()
 {
   return "unknown operation";
@@ -317,6 +315,8 @@ bool MyHandler::characters( const QString & ch )
 
 bool MyHandler::startElement( const QString & namespaceURI, const QString & localName, const QString & qName, const QXmlAttributes & atts)
 {
+  Q_UNUSED(namespaceURI);
+  Q_UNUSED(localName);
   if (qName == "variable") {
     currentVariable.name = atts.value("name");
     currentVariable.comment = atts.value("comment");
@@ -366,6 +366,8 @@ bool MyHandler::startElement( const QString & namespaceURI, const QString & loca
 
 bool MyHandler::endElement( const QString & namespaceURI, const QString & localName, const QString & qName)
 {
+  Q_UNUSED(namespaceURI);
+  Q_UNUSED(localName);
   if (qName == "type") {
     currentVariable.types.append(currentText);
   } else if (operationExpTags.contains(qName) || equationPartTags.contains(qName)) {
@@ -460,7 +462,7 @@ double rt_tock() {
 }
 
 int test_dump_xml_reader() {
-  QFile file("Modelica.Mechanics.MultiBody.Examples.Elementary.DoublePendulum_info.xml");
+  QFile file("Modelica.Mechanics.MultiBody.Examples.Elementary.DoublePendulum_info.json");
   rt_tick();
   MyHandler handler(file);
   qDebug() << QString("streaming done in: %1 ms\n").arg(rt_tock() *1e3);
