@@ -29,16 +29,22 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- * RCS: $Id$
- *
  */
 
 #include "TransformationsWidget.h"
+#include "Options/OptionsDialog.h"
+#include "Util/StringHandler.h"
+#include "Modeling/LibraryTreeWidget.h"
+#include "Editors/TransformationsEditor.h"
+#include "Editors/ModelicaEditor.h"
 #include <qjson/parser.h>
 
+#include <QStatusBar>
+#include <QGridLayout>
+#include <QVBoxLayout>
+#include <QMessageBox>
+
 /*!
   \class TVariablesTreeItem
   \brief Contains the information about the result variable.
@@ -387,7 +393,7 @@ bool TVariableTreeProxyModel::filterAcceptsRow(int sourceRow, const QModelIndex
       if (pTVariablesTreeItem)
       {
         QString variableName = pTVariablesTreeItem->getVariableName();
-        variableName.remove(QRegExp("(_res.mat|_res.plt|_res.csv)"));
+        variableName.remove(QRegExp("(\\.mat|\\.plt|\\.csv|_res.mat|_res.plt|_res.csv)"));
         return variableName.contains(filterRegExp());
       }
       else
@@ -405,13 +411,13 @@ TVariablesTreeView::TVariablesTreeView(TransformationsWidget *pTransformationsWi
   : QTreeView(pTransformationsWidget)
 {
   mpTransformationsWidget = pTransformationsWidget;
-  setObjectName("TreeWithBranches");
   setItemDelegate(new ItemDelegate(this));
   setTextElideMode(Qt::ElideMiddle);
   setIndentation(Helper::treeIndentation);
   setSortingEnabled(true);
   sortByColumn(0, Qt::AscendingOrder);
   setExpandsOnDoubleClick(false);
+  setUniformRowHeights(true);
 }
 
 EquationTreeWidget::EquationTreeWidget(TransformationsWidget *pTransformationWidget)
@@ -437,17 +443,17 @@ EquationTreeWidget::EquationTreeWidget(TransformationsWidget *pTransformationWid
   connect(this, SIGNAL(itemDoubleClicked(QTreeWidgetItem*,int)), mpTransformationWidget, SLOT(fetchEquationData(QTreeWidgetItem*,int)));
 }
 
-TransformationsWidget::TransformationsWidget(QString infoXMLFullFileName, MainWindow *pMainWindow)
-  : mpMainWindow(pMainWindow), mInfoXMLFullFileName(infoXMLFullFileName)
+TransformationsWidget::TransformationsWidget(QString infoJSONFullFileName, QWidget *pParent)
+  : QWidget(pParent), mInfoJSONFullFileName(infoJSONFullFileName)
 {
-  if (!mInfoXMLFullFileName.endsWith("_info.json")) {
+  if (!mInfoJSONFullFileName.endsWith("_info.json")) {
     mProfJSONFullFileName = "";
     mProfilingDataRealFileName = "";
   } else {
-    mProfJSONFullFileName = infoXMLFullFileName.left(infoXMLFullFileName.size() - 9) + "prof.json";
-    mProfilingDataRealFileName = infoXMLFullFileName.left(infoXMLFullFileName.size() - 9) + "prof.realdata";
+    mProfJSONFullFileName = infoJSONFullFileName.left(infoJSONFullFileName.size() - 9) + "prof.json";
+    mProfilingDataRealFileName = infoJSONFullFileName.left(infoJSONFullFileName.size() - 9) + "prof.realdata";
   }
-  setWindowIcon(QIcon(":/Resources/icons/debugger.svg"));
+  setWindowIcon(QIcon(":/Resources/icons/equational-debugger.svg"));
   setWindowTitle(QString(Helper::applicationName).append(" - ").append(Helper::transformationalDebugger));
   QToolButton *pReloadToolButton = new QToolButton;
   pReloadToolButton->setToolTip(Helper::reload);
@@ -455,7 +461,7 @@ TransformationsWidget::TransformationsWidget(QString infoXMLFullFileName, MainWi
   pReloadToolButton->setIcon(QIcon(":/Resources/icons/refresh.svg"));
   connect(pReloadToolButton, SIGNAL(clicked()), SLOT(reloadTransformations()));
   /* info xml file path label */
-  Label *pInfoXMLFilePathLabel = new Label(mInfoXMLFullFileName, this);
+  Label *pInfoXMLFilePathLabel = new Label(mInfoJSONFullFileName, this);
   pInfoXMLFilePathLabel->setElideMode(Qt::ElideMiddle);
   /* create status bar */
   QStatusBar *pStatusBar = new QStatusBar;
@@ -466,27 +472,13 @@ TransformationsWidget::TransformationsWidget(QString infoXMLFullFileName, MainWi
   /* Variables Heading */
   Label *pVariablesBrowserLabel = new Label(Helper::variablesBrowser);
   pVariablesBrowserLabel->setObjectName("LabelWithBorder");
-  // create the find text box
-  mpFindVariablesTextBox = new QLineEdit;
-  mpFindVariablesTextBox->setPlaceholderText(Helper::findVariables);
-  connect(mpFindVariablesTextBox, SIGNAL(returnPressed()), SLOT(findVariables()));
-  connect(mpFindVariablesTextBox, SIGNAL(textEdited(QString)), SLOT(findVariables()));
-  // create the case sensitivity checkbox
-  mpFindCaseSensitiveCheckBox = new QCheckBox(tr("Case Sensitive"));
-  connect(mpFindCaseSensitiveCheckBox, SIGNAL(toggled(bool)), SLOT(findVariables()));
-  // create the find syntax combobox
-  mpFindSyntaxComboBox = new QComboBox;
-  mpFindSyntaxComboBox->addItem(tr("Regular Expression"), QRegExp::RegExp);
-  mpFindSyntaxComboBox->setItemData(0, tr("A rich Perl-like pattern matching syntax."), Qt::ToolTipRole);
-  mpFindSyntaxComboBox->addItem(tr("Wildcard"), QRegExp::Wildcard);
-  mpFindSyntaxComboBox->setItemData(1, tr("A simple pattern matching syntax similar to that used by shells (command interpreters) for \"file globbing\"."), Qt::ToolTipRole);
-  mpFindSyntaxComboBox->addItem(tr("Fixed String"), QRegExp::FixedString);
-  mpFindSyntaxComboBox->setItemData(2, tr("Fixed string matching."), Qt::ToolTipRole);
-  connect(mpFindSyntaxComboBox, SIGNAL(currentIndexChanged(int)), SLOT(findVariables()));
-  // expand all button
-  mpExpandAllButton = new QPushButton(Helper::expandAll);
-  // collapse all button
-  mpCollapseAllButton = new QPushButton(Helper::collapseAll);
+  // tree search filters
+  mpTreeSearchFilters = new TreeSearchFilters(this);
+  mpTreeSearchFilters->getFilterTextBox()->setPlaceholderText(Helper::filterVariables);
+  connect(mpTreeSearchFilters->getFilterTextBox(), SIGNAL(returnPressed()), SLOT(findVariables()));
+  connect(mpTreeSearchFilters->getFilterTextBox(), SIGNAL(textEdited(QString)), SLOT(findVariables()));
+  connect(mpTreeSearchFilters->getCaseSensitiveCheckBox(), SIGNAL(toggled(bool)), SLOT(findVariables()));
+  connect(mpTreeSearchFilters->getSyntaxComboBox(), SIGNAL(currentIndexChanged(int)), SLOT(findVariables()));
   /* variables tree view */
   mpTVariablesTreeView = new TVariablesTreeView(this);
   mpTVariablesTreeModel = new TVariablesTreeModel(mpTVariablesTreeView);
@@ -496,18 +488,14 @@ TransformationsWidget::TransformationsWidget(QString infoXMLFullFileName, MainWi
   mpTVariablesTreeView->setModel(mpTVariableTreeProxyModel);
   mpTVariablesTreeView->setColumnWidth(2, 40);  /* line number column */
   connect(mpTVariablesTreeView, SIGNAL(doubleClicked(QModelIndex)), SLOT(fetchVariableData(QModelIndex)));
-  connect(mpExpandAllButton, SIGNAL(clicked()), mpTVariablesTreeView, SLOT(expandAll()));
-  connect(mpCollapseAllButton, SIGNAL(clicked()), mpTVariablesTreeView, SLOT(collapseAll()));
+  connect(mpTreeSearchFilters->getExpandAllButton(), SIGNAL(clicked()), mpTVariablesTreeView, SLOT(expandAll()));
+  connect(mpTreeSearchFilters->getCollapseAllButton(), SIGNAL(clicked()), mpTVariablesTreeView, SLOT(collapseAll()));
   QGridLayout *pVariablesGridLayout = new QGridLayout;
   pVariablesGridLayout->setSpacing(1);
   pVariablesGridLayout->setContentsMargins(0, 0, 0, 0);
-  pVariablesGridLayout->addWidget(pVariablesBrowserLabel, 0, 0, 1, 2);
-  pVariablesGridLayout->addWidget(mpFindVariablesTextBox, 1, 0, 1, 2);
-  pVariablesGridLayout->addWidget(mpFindCaseSensitiveCheckBox, 2, 0);
-  pVariablesGridLayout->addWidget(mpFindSyntaxComboBox, 2, 1);
-  pVariablesGridLayout->addWidget(mpExpandAllButton, 3, 0);
-  pVariablesGridLayout->addWidget(mpCollapseAllButton, 3, 1);
-  pVariablesGridLayout->addWidget(mpTVariablesTreeView, 4, 0, 1, 2);
+  pVariablesGridLayout->addWidget(pVariablesBrowserLabel, 0, 0);
+  pVariablesGridLayout->addWidget(mpTreeSearchFilters, 1, 0);
+  pVariablesGridLayout->addWidget(mpTVariablesTreeView, 2, 0);
   QFrame *pVariablesFrame = new QFrame;
   pVariablesFrame->setLayout(pVariablesGridLayout);
   /* Defined in tree widget */
@@ -553,7 +541,6 @@ TransformationsWidget::TransformationsWidget(QString infoXMLFullFileName, MainWi
   pEquationsBrowserLabel->setObjectName("LabelWithBorder");
   /* Equations tree widget */
   mpEquationsTreeWidget = new EquationTreeWidget(this);
-  mpEquationsTreeWidget->setObjectName("TreeWithBranches");
   mpEquationsTreeWidget->setIndentation(Helper::treeIndentation);
   QGridLayout *pEquationsGridLayout = new QGridLayout;
   pEquationsGridLayout->setSpacing(1);
@@ -625,10 +612,10 @@ TransformationsWidget::TransformationsWidget(QString infoXMLFullFileName, MainWi
   mpTSourceEditorInfoBar = new InfoBar(this);
   mpTSourceEditorInfoBar->hide();
   mpTransformationsEditor = new TransformationsEditor(this);
-  ModelicaTextHighlighter *pModelicaTextHighlighter;
-  pModelicaTextHighlighter = new ModelicaTextHighlighter(mpMainWindow->getOptionsDialog()->getModelicaTextEditorPage(),
-                                                         mpTransformationsEditor->getPlainTextEdit());
-  connect(mpMainWindow->getOptionsDialog(), SIGNAL(modelicaTextSettingsChanged()), pModelicaTextHighlighter, SLOT(settingsChanged()));
+  ModelicaHighlighter *pModelicaTextHighlighter;
+  pModelicaTextHighlighter = new ModelicaHighlighter(OptionsDialog::instance()->getModelicaEditorPage(),
+                                                     mpTransformationsEditor->getPlainTextEdit());
+  connect(OptionsDialog::instance(), SIGNAL(modelicaEditorSettingsChanged()), pModelicaTextHighlighter, SLOT(settingsChanged()));
   QVBoxLayout *pTSourceEditorVerticalLayout = new QVBoxLayout;
   pTSourceEditorVerticalLayout->setSpacing(1);
   pTSourceEditorVerticalLayout->setContentsMargins(0, 0, 0, 0);
@@ -734,8 +721,8 @@ TransformationsWidget::TransformationsWidget(QString infoXMLFullFileName, MainWi
   pMainLayout->addWidget(mpTransformationsHorizontalSplitter, 1, 0);
   setLayout(pMainLayout);
   /* restore the TransformationsWidget geometry and splitters state. */
-  QSettings *pSettings = OpenModelica::getApplicationSettings();
-  if (mpMainWindow->getOptionsDialog()->getGeneralSettingsPage()->getPreserveUserCustomizations())
+  QSettings *pSettings = Utilities::getApplicationSettings();
+  if (OptionsDialog::instance()->getGeneralSettingsPage()->getPreserveUserCustomizations())
   {
     pSettings->beginGroup("transformationalDebugger");
     restoreGeometry(pSettings->value("geometry").toByteArray());
@@ -785,6 +772,7 @@ static OMOperation* variantToOperationPtr(QVariantMap var)
 
 static void variantToSource(QVariantMap var, OMInfo &info, QStringList &types, QList<OMOperation*> &ops)
 {
+  Q_UNUSED(types);
   QVariantMap vinfo = var["info"].toMap();
   info.file = vinfo["file"].toString();
   info.lineStart = vinfo["lineStart"].toInt();
@@ -813,17 +801,17 @@ static OMEquation* getOMEquation(QList<OMEquation*> equations, int index)
 
 void TransformationsWidget::loadTransformations()
 {
-  QFile file(mInfoXMLFullFileName);
+  QFile file(mInfoJSONFullFileName);
   mEquations.clear();
   mVariables.clear();
   hasOperationsEnabled = false;
-  if (mInfoXMLFullFileName.endsWith(".json")) {
+  if (mInfoJSONFullFileName.endsWith(".json")) {
     QJson::Parser parser;
     bool ok;
     QVariantMap result;
     result = parser.parse(&file, &ok).toMap();
     if (!ok) {
-      QMessageBox::critical(this, QString(Helper::applicationName).append(" - ").append(Helper::parsingFailedJson), Helper::parsingFailedJson + ": " + mInfoXMLFullFileName, Helper::ok);
+      QMessageBox::critical(this, QString(Helper::applicationName).append(" - ").append(Helper::parsingFailedJson), Helper::parsingFailedJson + ": " + mInfoJSONFullFileName, Helper::ok);
       return;
     }
     QVariantMap vars = result["variables"].toMap();
@@ -846,9 +834,6 @@ void TransformationsWidget::loadTransformations()
     for (int i=0; i<eqs.size(); i++) {
       QVariantMap veq = eqs[i].toMap();
       OMEquation *eq = mEquations[i];
-      if (!hasOperationsEnabled && eq->ops.size() > 0) {
-        hasOperationsEnabled = true;
-      }
       eq->section = veq["section"].toString();
       if (veq["eqIndex"].toInt() != i) {
         QMessageBox::critical(this, QString(Helper::applicationName).append(" - ").append(Helper::parsingFailedJson), Helper::parsingFailedJson + QString(": got index ") + veq["eqIndex"].toString() + QString(" expected ") + QString::number(i), Helper::ok);
@@ -882,6 +867,9 @@ void TransformationsWidget::loadTransformations()
         eq->display = eq->tag;
       }
       variantToSource(veq["source"].toMap(), eq->info, eq->types, eq->ops);
+      if (!hasOperationsEnabled && eq->ops.size() > 0) {
+        hasOperationsEnabled = true;
+      }
     }
     parseProfiling(mProfJSONFullFileName);
     fetchEquations();
@@ -893,6 +881,7 @@ void TransformationsWidget::loadTransformations()
     fetchEquations();
     hasOperationsEnabled = mpInfoXMLFileHandler->hasOperationsEnabled;
   }
+  fetchVariableData(mpTVariableTreeProxyModel->index(0, 0));
 }
 
 void TransformationsWidget::fetchDefinedInEquations(const OMVariable &variable)
@@ -900,16 +889,17 @@ void TransformationsWidget::fetchDefinedInEquations(const OMVariable &variable)
   /* Clear the defined in tree. */
   clearTreeWidgetItems(mpDefinedInEquationsTreeWidget);
   /* add defined in equations */
-  for (int i=0; i<variable.definedIn.size(); i++)
-  {
+  for (int i=0; i<variable.definedIn.size(); i++) {
     OMEquation *equation = getOMEquation(mEquations, variable.definedIn[i]);
-    QStringList values;
-    values << QString::number(variable.definedIn[i]) << equation->section << equation->toString();
-    QTreeWidgetItem *pDefinedInTreeItem = new IntegerTreeWidgetItem(values, mpDefinedInEquationsTreeWidget);
-    pDefinedInTreeItem->setToolTip(0, values[0]);
-    pDefinedInTreeItem->setToolTip(1, values[1]);
-    pDefinedInTreeItem->setToolTip(2, values[2]);
-    mpDefinedInEquationsTreeWidget->addTopLevelItem(pDefinedInTreeItem);
+    if (equation) {
+      QStringList values;
+      values << QString::number(variable.definedIn[i]) << equation->section << equation->toString();
+      QTreeWidgetItem *pDefinedInTreeItem = new IntegerTreeWidgetItem(values, mpDefinedInEquationsTreeWidget);
+      pDefinedInTreeItem->setToolTip(0, values[0]);
+      pDefinedInTreeItem->setToolTip(1, values[1]);
+      pDefinedInTreeItem->setToolTip(2, values[2]);
+      mpDefinedInEquationsTreeWidget->addTopLevelItem(pDefinedInTreeItem);
+    }
   }
 }
 
@@ -918,16 +908,17 @@ void TransformationsWidget::fetchUsedInEquations(const OMVariable &variable)
   /* Clear the used in tree. */
   clearTreeWidgetItems(mpUsedInEquationsTreeWidget);
   /* add used in equations */
-  foreach (int index, variable.usedIn)
-  {
+  foreach (int index, variable.usedIn) {
     OMEquation *equation = getOMEquation(mEquations, index);
-    QStringList values;
-    values << QString::number(index) << equation->section << equation->toString();
-    QTreeWidgetItem *pUsedInTreeItem = new IntegerTreeWidgetItem(values, mpUsedInEquationsTreeWidget);
-    pUsedInTreeItem->setToolTip(0, values[0]);
-    pUsedInTreeItem->setToolTip(1, values[1]);
-    pUsedInTreeItem->setToolTip(2, values[2]);
-    mpUsedInEquationsTreeWidget->addTopLevelItem(pUsedInTreeItem);
+    if (equation) {
+      QStringList values;
+      values << QString::number(index) << equation->section << equation->toString();
+      QTreeWidgetItem *pUsedInTreeItem = new IntegerTreeWidgetItem(values, mpUsedInEquationsTreeWidget);
+      pUsedInTreeItem->setToolTip(0, values[0]);
+      pUsedInTreeItem->setToolTip(1, values[1]);
+      pUsedInTreeItem->setToolTip(2, values[2]);
+      mpUsedInEquationsTreeWidget->addTopLevelItem(pUsedInTreeItem);
+    }
   }
 }
 
@@ -945,16 +936,7 @@ void TransformationsWidget::fetchOperations(const OMVariable &variable)
       mpVariableOperationsTreeWidget->setItemWidget(pOperationTreeItem, 0, opText);
     }
   } else {
-    QString message;
-#ifdef Q_OS_MAC
-    message = GUIMessages::getMessage(GUIMessages::SET_INFO_XML_FLAG)
-        .arg(GUIMessages::getMessage(GUIMessages::GENERATE_OPERATIONS_MSG_MAC))
-        .arg(GUIMessages::getMessage(GUIMessages::SET_INFO_XML_FLAG_MSG_MAC));
-#else
-    message = GUIMessages::getMessage(GUIMessages::SET_INFO_XML_FLAG)
-        .arg(GUIMessages::getMessage(GUIMessages::GENERATE_OPERATIONS_MSG))
-        .arg(GUIMessages::getMessage(GUIMessages::SET_INFO_XML_FLAG_MSG));
-#endif
+    QString message = GUIMessages::getMessage(GUIMessages::SET_INFO_XML_FLAG).arg(Helper::toolsOptionsPath).arg(Helper::toolsOptionsPath);
     QStringList values;
     values << message;
     QString toolTip = message;
@@ -1041,6 +1023,9 @@ QTreeWidgetItem* TransformationsWidget::findEquationTreeItem(int equationIndex)
 void TransformationsWidget::fetchEquationData(int equationIndex)
 {
   OMEquation *equation = getOMEquation(mEquations, equationIndex);
+  if (!equation) {
+    return;
+  }
   /* fetch defines */
   fetchDefines(equation);
   /* fetch depends */
@@ -1081,12 +1066,12 @@ void TransformationsWidget::fetchEquationData(int equationIndex)
   }
 #endif
 
-  if (!equation->info.isValid)
+  if (!equation->info.isValid) {
     return;
+  }
   /* open the model with and go to the equation line */
   QFile file(equation->info.file);
-  if (file.exists())
-  {
+  if (file.exists()) {
     mpTSourceEditorFileLabel->setText(file.fileName());
     mpTSourceEditorFileLabel->show();
     file.open(QIODevice::ReadOnly);
@@ -1095,7 +1080,6 @@ void TransformationsWidget::fetchEquationData(int equationIndex)
     file.close();
     mpTransformationsEditor->goToLineNumber(equation->info.lineStart);
   }
-
 }
 
 void TransformationsWidget::fetchDefines(OMEquation *equation)
@@ -1103,16 +1087,17 @@ void TransformationsWidget::fetchDefines(OMEquation *equation)
   /* Clear the defines tree. */
   clearTreeWidgetItems(mpDefinesVariableTreeWidget);
   /* add defines */
-  foreach (QString define, equation->defines)
-  {
-    QStringList values;
-    values << define;
-    QString toolTip = define;
-    QTreeWidgetItem *pDefineTreeItem = new QTreeWidgetItem(values);
-    pDefineTreeItem->setToolTip(0, toolTip);
-    mpDefinesVariableTreeWidget->addTopLevelItem(pDefineTreeItem);
+  if (equation) {
+    foreach (QString define, equation->defines) {
+      QStringList values;
+      values << define;
+      QString toolTip = define;
+      QTreeWidgetItem *pDefineTreeItem = new QTreeWidgetItem(values);
+      pDefineTreeItem->setToolTip(0, toolTip);
+      mpDefinesVariableTreeWidget->addTopLevelItem(pDefineTreeItem);
+    }
+    mpDefinesVariableTreeWidget->resizeColumnToContents(0);
   }
-  mpDefinesVariableTreeWidget->resizeColumnToContents(0);
 }
 
 void TransformationsWidget::fetchDepends(OMEquation *equation)
@@ -1120,16 +1105,17 @@ void TransformationsWidget::fetchDepends(OMEquation *equation)
   /* Clear the depends tree. */
   clearTreeWidgetItems(mpDependsVariableTreeWidget);
   /* add depends */
-  foreach (QString depend, equation->depends)
-  {
-    QStringList values;
-    values << depend;
-    QString toolTip = depend;
-    QTreeWidgetItem *pDependTreeItem = new QTreeWidgetItem(values);
-    pDependTreeItem->setToolTip(0, toolTip);
-    mpDependsVariableTreeWidget->addTopLevelItem(pDependTreeItem);
+  if (equation) {
+    foreach (QString depend, equation->depends) {
+      QStringList values;
+      values << depend;
+      QString toolTip = depend;
+      QTreeWidgetItem *pDependTreeItem = new QTreeWidgetItem(values);
+      pDependTreeItem->setToolTip(0, toolTip);
+      mpDependsVariableTreeWidget->addTopLevelItem(pDependTreeItem);
+    }
+    mpDependsVariableTreeWidget->resizeColumnToContents(0);
   }
-  mpDependsVariableTreeWidget->resizeColumnToContents(0);
 }
 
 void TransformationsWidget::fetchOperations(OMEquation *equation)
@@ -1138,24 +1124,17 @@ void TransformationsWidget::fetchOperations(OMEquation *equation)
   clearTreeWidgetItems(mpEquationOperationsTreeWidget);
   /* add operations */
   if (hasOperationsEnabled) {
-    foreach (OMOperation *op, equation->ops) {
-      QTreeWidgetItem *pOperationTreeItem = new QTreeWidgetItem();
-      mpEquationOperationsTreeWidget->addTopLevelItem(pOperationTreeItem);
-      // set label item
-      Label *opText = new Label("<html><div style=\"margin:3px;\">" + op->toHtml() + "</div></html>");
-      mpEquationOperationsTreeWidget->setItemWidget(pOperationTreeItem, 0, opText);
+    if (equation) {
+      foreach (OMOperation *op, equation->ops) {
+        QTreeWidgetItem *pOperationTreeItem = new QTreeWidgetItem();
+        mpEquationOperationsTreeWidget->addTopLevelItem(pOperationTreeItem);
+        // set label item
+        Label *opText = new Label("<html><div style=\"margin:3px;\">" + op->toHtml() + "</div></html>");
+        mpEquationOperationsTreeWidget->setItemWidget(pOperationTreeItem, 0, opText);
+      }
     }
   } else {
-    QString message;
-#ifdef Q_OS_MAC
-    message = GUIMessages::getMessage(GUIMessages::SET_INFO_XML_FLAG)
-        .arg(GUIMessages::getMessage(GUIMessages::GENERATE_OPERATIONS_MSG_MAC))
-        .arg(GUIMessages::getMessage(GUIMessages::SET_INFO_XML_FLAG_MSG_MAC));
-#else
-    message = GUIMessages::getMessage(GUIMessages::SET_INFO_XML_FLAG)
-        .arg(GUIMessages::getMessage(GUIMessages::GENERATE_OPERATIONS_MSG))
-        .arg(GUIMessages::getMessage(GUIMessages::SET_INFO_XML_FLAG_MSG));
-#endif
+    QString message = GUIMessages::getMessage(GUIMessages::SET_INFO_XML_FLAG).arg(Helper::toolsOptionsPath).arg(Helper::toolsOptionsPath);
     QStringList values;
     values << message;
     QString toolTip = message;
@@ -1188,9 +1167,15 @@ void TransformationsWidget::reloadTransformations()
   /* Clear the variable operations tree. */
   clearTreeWidgetItems(mpVariableOperationsTreeWidget);
   /* clear the variable tree filters. */
-  mpFindVariablesTextBox->setText(Helper::findVariables);
-  mpFindSyntaxComboBox->setCurrentIndex(0);
-  mpFindCaseSensitiveCheckBox->setChecked(false);
+  bool signalsState = mpTreeSearchFilters->getFilterTextBox()->blockSignals(true);
+  mpTreeSearchFilters->getFilterTextBox()->clear();
+  mpTreeSearchFilters->getFilterTextBox()->blockSignals(signalsState);
+  signalsState = mpTreeSearchFilters->getSyntaxComboBox()->blockSignals(true);
+  mpTreeSearchFilters->getSyntaxComboBox()->setCurrentIndex(0);
+  mpTreeSearchFilters->getSyntaxComboBox()->blockSignals(signalsState);
+  signalsState = mpTreeSearchFilters->getCaseSensitiveCheckBox()->blockSignals(true);
+  mpTreeSearchFilters->getCaseSensitiveCheckBox()->setChecked(false);
+  mpTreeSearchFilters->getCaseSensitiveCheckBox()->blockSignals(signalsState);
   mpTVariableTreeProxyModel->setFilterRegExp(QRegExp());
   /* clear equations tree */
   clearTreeWidgetItems(mpEquationsTreeWidget);
@@ -1211,20 +1196,21 @@ void TransformationsWidget::reloadTransformations()
   loadTransformations();
 }
 
+/*!
+ * \brief TransformationsWidget::findVariables
+ * Finds the variables in the TransformationsWidget Variables Browser.
+ */
 void TransformationsWidget::findVariables()
 {
-  QString findText = mpFindVariablesTextBox->text();
-  if (mpFindVariablesTextBox->text().isEmpty() || (mpFindVariablesTextBox->text().compare(Helper::findVariables) == 0))
-  {
-    findText = "";
-  }
-  QRegExp::PatternSyntax syntax = QRegExp::PatternSyntax(mpFindSyntaxComboBox->itemData(mpFindSyntaxComboBox->currentIndex()).toInt());
-  Qt::CaseSensitivity caseSensitivity = mpFindCaseSensitiveCheckBox->isChecked() ? Qt::CaseSensitive: Qt::CaseInsensitive;
+  QString findText = mpTreeSearchFilters->getFilterTextBox()->text();
+  QRegExp::PatternSyntax syntax = QRegExp::PatternSyntax(mpTreeSearchFilters->getSyntaxComboBox()->itemData(mpTreeSearchFilters->getSyntaxComboBox()->currentIndex()).toInt());
+  Qt::CaseSensitivity caseSensitivity = mpTreeSearchFilters->getCaseSensitiveCheckBox()->isChecked() ? Qt::CaseSensitive: Qt::CaseInsensitive;
   QRegExp regExp(findText, caseSensitivity, syntax);
   mpTVariableTreeProxyModel->setFilterRegExp(regExp);
   /* expand all so that the filtered items can be seen. */
-  if (!findText.isEmpty())
+  if (!findText.isEmpty()) {
     mpTVariablesTreeView->expandAll();
+  }
 }
 
 void TransformationsWidget::fetchVariableData(const QModelIndex &index)
