@@ -29,10 +29,7 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- *
  */
 
 #ifndef HELPER_H
@@ -43,7 +40,6 @@
 #include <QSize>
 #include <QObject>
 #include <QFontInfo>
-#include <QSettings>
 
 class Helper : public QObject
 {
@@ -72,9 +68,9 @@ public:
   static QString exeFileTypes;
   static QString txtFileTypes;
   static QString figaroFileTypes;
+  static QString visualizationFileTypes;
   static int treeIndentation;
   static QSize iconSize;
-  static QSize buttonIconSize;
   static int tabWidth;
   static QString modelicaComponentFormat;
   static QString modelicaFileFormat;
@@ -99,7 +95,6 @@ public:
   static QFontInfo systemFontInfo;
   static QFontInfo monospacedFontInfo;
   static QString toolsOptionsPath;
-  static QString toolsOptionsPathMAC;
   /* Meta Modelica Types */
   static QString MODELICA_METATYPE;
   static QString MODELICA_STRING;
@@ -126,10 +121,13 @@ public:
   static QString libraries;
   static QString clearRecentFiles;
   static QString encoding;
+  static QString fileLabel;
   static QString file;
+  static QString folder;
   static QString browse;
   static QString ok;
   static QString cancel;
+  static QString reset;
   static QString close;
   static QString error;
   static QString chooseFile;
@@ -155,6 +153,7 @@ public:
   static QString type;
   static QString information;
   static QString rename;
+  static QString renameTip;
   static QString checkModel;
   static QString checkModelTip;
   static QString checkAllModels;
@@ -189,7 +188,7 @@ public:
   static QString duplicateTip;
   static QString unloadClass;
   static QString unloadClassTip;
-  static QString unloadTLMOrTextTip;
+  static QString unloadMetaModelOrTextTip;
   static QString refresh;
   static QString simulate;
   static QString simulateTip;
@@ -197,10 +196,13 @@ public:
   static QString reSimulateTip;
   static QString reSimulateSetup;
   static QString reSimulateSetupTip;
+  static QString exportVariables;
   static QString simulateWithTransformationalDebugger;
   static QString simulateWithTransformationalDebuggerTip;
   static QString simulateWithAlgorithmicDebugger;
   static QString simulateWithAlgorithmicDebuggerTip;
+  static QString simulateWithAnimation;
+  static QString simulateWithAnimationTip;
   static QString simulationSetup;
   static QString simulationSetupTip;
   static QString simulation;
@@ -228,7 +230,7 @@ public:
   static QString size;
   static QString lineStyle;
   static QString color;
-  static QString fontAndColors;
+  static QString Colors;
   static QString fontFamily;
   static QString fontSize;
   static QString pickColor;
@@ -253,13 +255,14 @@ public:
   static QString diagramView;
   static QString textView;
   static QString documentationView;
-  static QString searchClasses;
+  static QString filterClasses;
   static QString findReplaceModelicaText;
   static QString left;
   static QString center;
   static QString right;
   static QString createConnection;
-  static QString findVariables;
+  static QString connectionAttributes;
+  static QString filterVariables;
   static QString openClass;
   static QString openClassTip;
   static QString viewDocumentation;
@@ -282,6 +285,7 @@ public:
   static QString debuggingFileNotSaveInfo;
   static QString algorithmicDebugger;
   static QString debugConfigurations;
+  static QString debugConfigurationsTip;
   static QString resume;
   static QString interrupt;
   static QString exit;
@@ -289,6 +293,7 @@ public:
   static QString stepInto;
   static QString stepReturn;
   static QString attachToRunningProcess;
+  static QString attachToRunningProcessTip;
   static QString crashReport;
   static QString parsingFailedJson;
   static QString expandAll;
@@ -299,23 +304,41 @@ public:
   static QString cancelSimulation;
   static QString fetchInterfaceData;
   static QString fetchInterfaceDataTip;
+  static QString alignInterfaces;
+  static QString alignInterfacesTip;
   static QString tlmCoSimulationSetup;
   static QString tlmCoSimulationSetupTip;
   static QString tlmCoSimulation;
+  static QString animationChooseFile;
+  static QString animationChooseFileTip;
+  static QString animationInitialize;
+  static QString animationInitializeTip;
+  static QString animationPlay;
+  static QString animationPlayTip;
+  static QString animationPause;
+  static QString animationPauseTip;
+  static QString simulationParams;
+  static QString simulationParamsTip;
   static QString running;
   static QString finished;
   static QString newVariable;
   static QString library;
   static QString moveUp;
   static QString moveDown;
+  static QString fixErrorsManually;
+  static QString revertToLastCorrectVersion;
+  static QString OMCFlags;
+  static QString OMCFlagsTip;
+  static QString saveExperimentAnnotation;
+  static QString saveOpenModelicaSimulationFlagsAnnotation;
+  static QString item;
 };
 
 class GUIMessages : public QObject
 {
   Q_OBJECT
 public:
-  enum MessagesTypes
-  {
+  enum MessagesTypes {
     CHECK_MESSAGES_BROWSER,
     SAME_COMPONENT_NAME,
     SAME_COMPONENT_CONNECT,
@@ -344,7 +367,6 @@ public:
     DIAGRAM_VIEW_DROP_MSG,
     ICON_VIEW_DROP_MSG,
     PLOT_PARAMETRIC_DIFF_FILES,
-    FILE_FORMAT_NOT_SUPPORTED,
     ENTER_VALID_NUMBER,
     ENTER_VALUE,
     ITEM_DROPPED_ON_ITSELF,
@@ -355,6 +377,7 @@ public:
     FIGARO_GENERATED,
     UNLOAD_CLASS_MSG,
     DELETE_CLASS_MSG,
+    UNLOAD_TEXT_FILE_MSG,
     DELETE_TEXT_FILE_MSG,
     WRONG_MODIFIER,
     SET_INFO_XML_FLAG,
@@ -368,16 +391,13 @@ public:
     TLMMANAGER_NOT_SET,
     METAMODEL_UNSAVED,
     TLMCOSIMULATION_ALREADY_RUNNING,
-    TERMINAL_COMMAND_NOT_SET
+    TERMINAL_COMMAND_NOT_SET,
+    UNABLE_FIND_COMPONENT,
+    SELECT_SIMULATION_OPTION,
+    MULTIPLE_DECLARATIONS_COMPONENT
   };
 
   static QString getMessage(int type);
 };
 
-
-namespace OpenModelica {
-  QString& tempDirectory();
-  QSettings* getApplicationSettings();
-}
-
 #endif // HELPER_H
