@@ -29,23 +29,15 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- *
  */
 
 #ifndef STRINGHANDLER_H
 #define STRINGHANDLER_H
 
-#include <QtGlobal>
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
-#include <QtWidgets>
-#else
-#include <QtGui>
-#endif
-
-#include <QtCore>
+#include <QObject>
+#include <QComboBox>
+#include <QProcessEnvironment>
 
 class StringHandler : public QObject
 {
@@ -113,7 +105,6 @@ public:
   static QString removeFirstLastCurlBrackets(QString value);
   static QString removeFirstLastBrackets(QString value);
   static QString removeFirstLastQuotes(QString value);
-  static QString getSubStringBeforeDots(QString value);
   static QStringList getStrings(QString value);
   static QStringList getStrings(QString value, char start, char end);
   /* Handles quoted identifiers A.B.'C.D' -> A.B, A.B.C.D -> A.B.C */
@@ -121,8 +112,8 @@ public:
   static QString removeLastWordAfterDot(QString value);
   static QString getFirstWordBeforeDot(QString value);
   static QString removeFirstWordAfterDot(QString value);
-  static QString getModifierValue(QString value);
   static QString escapeString(QString value);
+  static QString escapeStringQuotes(QString value);
   // Returns "" if the string is not a standard Modelica string. Else it unparses it into normal form.
   static QString unparse(QString value);
   // Returns empty list if the string is not a standard Modelica string-array. Else it unparses it into normal form.
@@ -148,8 +139,6 @@ public:
   static QStringList splitStringWithSpaces(QString value);
   static void fillEncodingComboBox(QComboBox *pEncodingComboBox);
   static QStringList makeVariableParts(QString variable);
-  static bool isCFile(QString extension);
-  static bool isModelicaFile(QString extension);
   static bool naturalSort(const QString &s1, const QString &s2);
 #ifdef WIN32
   static QProcessEnvironment simulationProcessEnvironment();
@@ -162,6 +151,7 @@ public:
   static QMap<int, int> getLeadingSpaces(QString contents);
   static int getLeadingSpacesSize(QString str);
   static bool isFileWritAble(QString filePath);
+  static bool containsSpace(QString str);
 protected:
   static QString mLastOpenDir;
 };
