@@ -29,13 +29,12 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- *
  */
 
+#include <QSplashScreen>
 #include <QMdiArea>
+#include <QLineEdit>
 #include <QThread>
 #include <QToolButton>
 #include <QComboBox>
@@ -46,19 +45,36 @@
 #include <QCheckBox>
 #include <QVariant>
 #include <QAbstractMessageHandler>
+#include <QDebug>
+#include <QPlainTextEdit>
+#include <QTextEdit>
+#include <QProcess>
+#include <QSettings>
+#include <QFileIconProvider>
+#include <QGroupBox>
+#include <QListWidget>
+#include <QListWidgetItem>
+#include <QScrollArea>
+#include <QScrollBar>
+#include <QGenericMatrix>
+
+#ifdef WIN32
+#include <windows.h>
+#include <tlhelp32.h>
+#endif
 
 #ifndef UTILITIES_H
 #define UTILITIES_H
 
-class MainWindow;
-class MdiArea : public QMdiArea
+class SplashScreen : public QSplashScreen
 {
   Q_OBJECT
+private:
+  SplashScreen() : QSplashScreen() {}
+
+  static SplashScreen *mpInstance;
 public:
-  MdiArea(QWidget *pParent = 0);
-  MainWindow* getMainWindow();
-protected:
-  MainWindow *mpMainWindow;
+  static SplashScreen *instance();
 };
 
 //! @brief Used to create platform independent sleep for the application.
@@ -69,6 +85,7 @@ public:
   Sleep() {}
   ~Sleep() {}
   static void sleep(unsigned long secs) {QThread::sleep(secs);}
+  static void msleep(unsigned long msecs) {QThread::msleep(msecs);}
 protected:
   void run() {}
 };
@@ -78,13 +95,15 @@ class TreeSearchFilters : public QWidget
   Q_OBJECT
 public:
   TreeSearchFilters(QWidget *pParent = 0);
-  QLineEdit* getSearchTextBox() {return mpSearchTextBox;}
+  QLineEdit* getFilterTextBox() {return mpFilterTextBox;}
   QComboBox* getSyntaxComboBox() {return mpSyntaxComboBox;}
   QCheckBox* getCaseSensitiveCheckBox() {return mpCaseSensitiveCheckBox;}
   QPushButton* getExpandAllButton() {return mpExpandAllButton;}
   QPushButton* getCollapseAllButton() {return mpCollapseAllButton;}
+
+  bool eventFilter(QObject *pObject, QEvent *pEvent);
 private:
-  QLineEdit *mpSearchTextBox;
+  QLineEdit *mpFilterTextBox;
   QToolButton *mpShowHideButton;
   QWidget *mpFiltersWidget;
   QComboBox *mpSyntaxComboBox;
@@ -101,6 +120,7 @@ class FileDataNotifier : public QThread
 public:
   FileDataNotifier(const QString fileName);
   void exit(int retcode = 0);
+  QByteArray read(qint64 maxlen);
 private:
   QFile mFile;
   bool mStop;
@@ -114,9 +134,9 @@ signals:
 };
 
 /*!
-  \class Label
-  \brief Creates a QLabel with elidable text. The default elide mode is Qt::ElideMiddle.Allows text selection via mouse.
-  */
+ * \class Label
+ * \brief Creates a QLabel with elidable text. The default elide mode is Qt::ElideNone. Allows text selection via mouse.
+ */
 class Label : public QLabel
 {
 public:
@@ -302,8 +322,145 @@ private:
   bool mFailed;
 };
 
+typedef struct {
+  QString mDelay;
+  QString mZf;
+  QString mZfr;
+  QString mAlpha;
+} MetaModelConnection;
+
+class PreviewPlainTextEdit : public QPlainTextEdit
+{
+  Q_OBJECT
+public:
+  PreviewPlainTextEdit(QWidget *parent = 0);
+private:
+  QTextCharFormat mParenthesesMatchFormat;
+  QTextCharFormat mParenthesesMisMatchFormat;
+
+  void highlightCurrentLine();
+  void highlightParentheses();
+public slots:
+  void updateHighlights();
+};
+
+class ListWidgetItem : public QListWidgetItem
+{
+public:
+  ListWidgetItem(QString text, QColor color, QListWidget *pParentListWidget);
+  QColor getColor() {return mColor;}
+  void setColor(QColor color) {mColor = color;}
+private:
+  QColor mColor;
+};
+
+class CodeColorsWidget : public QWidget
+{
+  Q_OBJECT
+public:
+  CodeColorsWidget(QWidget *pParent = 0);
+  QListWidget* getItemsListWidget() {return mpItemsListWidget;}
+  PreviewPlainTextEdit* getPreviewPlainTextEdit() {return mpPreviewPlainTextEdit;}
+private:
+  QGroupBox *mpColorsGroupBox;
+  Label *mpItemsLabel;
+  QListWidget *mpItemsListWidget;
+  Label *mpItemColorLabel;
+  QPushButton *mpItemColorPickButton;
+  Label *mpPreviewLabel;
+  PreviewPlainTextEdit *mpPreviewPlainTextEdit;
+  ListWidgetItem *mpTextItem;
+  ListWidgetItem *mpNumberItem;
+  ListWidgetItem *mpKeywordItem;
+  ListWidgetItem *mpTypeItem;
+  ListWidgetItem *mpFunctionItem;
+  ListWidgetItem *mpQuotesItem;
+  ListWidgetItem *mpCommentItem;
+signals:
+  void colorUpdated();
+private slots:
+  void pickColor();
+};
+
+/*!
+ * \brief The VerticalScrollArea class
+ * A scroll area with vertical bar and adjustment of width
+ * See: https://forum.qt.io/topic/13374/solved-qscrollarea-vertical-scroll-only
+ */
+class VerticalScrollArea : public QScrollArea
+{
+public:
+  VerticalScrollArea()
+  {
+    setWidgetResizable(true);
+    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);
+  }
+
+  virtual bool eventFilter(QObject *o, QEvent *e)
+  {
+    if (o && o == widget() && e->type() == QEvent::Resize) {
+      setMinimumWidth(widget()->minimumSizeHint().width() + verticalScrollBar()->width());
+    }
+    return QScrollArea::eventFilter(o, e);
+  }
+};
+
 namespace Utilities {
-  void parseTLMText(MessageHandler *pMessageHandler, QString contents);
-}
+
+  enum LineEndingMode {
+    CRLFLineEnding = 0,
+    LFLineEnding = 1,
+    NativeLineEnding =
+#ifdef WIN32
+    CRLFLineEnding,
+#else
+    LFLineEnding
+#endif
+  };
+
+  enum BomMode {
+    AlwaysAddBom = 0,
+    KeepBom = 1,
+    AlwaysDeleteBom = 2
+  };
+
+  QString& tempDirectory();
+  QSettings* getApplicationSettings();
+  void parseMetaModelText(MessageHandler *pMessageHandler, QString contents);
+  qreal convertUnit(qreal value, qreal offset, qreal scaleFactor);
+  Label* getHeadingLabel(QString heading);
+  QFrame* getHeadingLine();
+  bool detectBOM(QString fileName);
+  QTextCharFormat getParenthesesMatchFormat();
+  QTextCharFormat getParenthesesMisMatchFormat();
+  void highlightCurrentLine(QPlainTextEdit *pPlainTextEdit);
+  void highlightParentheses(QPlainTextEdit *pPlainTextEdit, QTextCharFormat parenthesesMatchFormat, QTextCharFormat parenthesesMisMatchFormat);
+  qint64 getProcessId(QProcess *pProcess);
+#ifdef WIN32
+  void killProcessTreeWindows(DWORD myprocID);
+#endif
+  bool isCFile(QString extension);
+  bool isModelicaFile(QString extension);
+  void insertText(QPlainTextEdit *pPlainTextEdit, QString text, QTextCharFormat format = QTextCharFormat());
+  QGenericMatrix<3,3, double> getRotationMatrix(QGenericMatrix<3,1,double> rotation);
+
+  namespace FileIconProvider {
+    class FileIconProviderImplementation : public QFileIconProvider
+    {
+    public:
+      FileIconProviderImplementation();
+      QIcon icon(const QFileInfo &info);
+      using QFileIconProvider::icon;
+      // Mapping of file suffix to icon.
+      QHash<QString, QIcon> mIconsHash;
+      QIcon mUnknownFileIcon;
+    };
+    // Access to the single instance
+    QFileIconProvider *iconProvider();
+    QIcon icon(const QFileInfo &info);
+  } // namespace FileIconProvider
+
+} // namespace Utilities
 
 #endif // UTILITIES_H
