@@ -29,59 +29,47 @@
  *
  */
 /*
- *
  * @author Adeel Asghar <adeel.asghar@liu.se>
- *
- *
  */
 
 /*!
-  \mainpage OMEdit - OpenModelica Connection Editor Documentation
-  Source code documentation. Provides brief information about the classes used.
+ * \mainpage OMEdit - OpenModelica Connection Editor Documentation
+ * Source code documentation. Provides brief information about the classes used.
+ * \section contributors_section Contributors
+ * - Adeel Asghar - <a href="mailto:adeel.asghar@liu.se">adeel.asghar@liu.se</a>
+ * - Sonia Tariq
+ * - Martin Sjölund - <a href="mailto:martin.sjolund@liu.se">martin.sjolund@liu.se</a>
+ * - Alachew Shitahun - <a href="mailto:alachew.mengist@liu.se">alachew.mengist@liu.se</a>
+ * - Jan Kokert - <a href="mailto:jan.kokert@imtek.uni-freiburg.de">jan.kokert@imtek.uni-freiburg.de</a>
+ * - Dr. Henning Kiel
+ * - Haris Kapidzic
+ * - Abhinn Kothari
+ * - Lennart Ochel - <a href="mailto:lennart.ochel@fh-bielefeld.de">lennart.ochel@fh-bielefeld.de</a>
+ * - Volker Waurich - <a href="mailto:volker.waurich@tu-dresden.de">volker.waurich@tu-dresden.de</a>
+ * - Rüdiger Franke
+ * - Martin Flehmig
+ * - Robert Braun - <a href=\"mailto:robert.braun@liu.se\">robert.braun@liu.se</a>
+ * - Per Östlund - <a href=\"mailto:per.ostlund@liu.se\">per.ostlund@liu.se</a>
+ * - Dietmar Winkler
+ * - Anatoly Severin
+ * - Adrian Pop - <a href="mailto:adrian.pop@liu.se">adrian.pop@liu.se</a>
+ */
 
-  \section contributors_section Contributors
-  \subsection year_2015_subsection 2015
-  - Adeel Asghar - <a href="mailto:adeel.asghar@liu.se">adeel.asghar@liu.se</a>
-  - Martin Sjölund - <a href="mailto:martin.sjolund@liu.se">martin.sjolund@liu.se</a>
-  - Alachew Shitahun - <a href="mailto:alachew.mengist@liu.se">alachew.mengist@liu.se</a>
-
-  \subsection year_2014_subsection 2014
-  - Adeel Asghar - <a href="mailto:adeel.asghar@liu.se">adeel.asghar@liu.se</a>
-  - Martin Sjölund - <a href="mailto:martin.sjolund@liu.se">martin.sjolund@liu.se</a>
-  - Alachew Shitahun - <a href="mailto:alachew.mengist@liu.se">alachew.mengist@liu.se</a>
-  - Jan Kokert - <a href="mailto:jan.kokert@imtek.uni-freiburg.de">jan.kokert@imtek.uni-freiburg.de</a>
-
-  \subsection year_2013_subsection 2013
-  - Adeel Asghar - <a href="mailto:adeel.asghar@liu.se">adeel.asghar@liu.se</a>
-  - Martin Sjölund - <a href="mailto:martin.sjolund@liu.se">martin.sjolund@liu.se</a>
-  - Dr. Henning Kiel
-  - Alachew Shitahun
-
-  \subsection year_2012_subsection 2012
-  - Adeel Asghar - <a href="mailto:adeel.asghar@liu.se">adeel.asghar@liu.se</a>
-  - Martin Sjölund - <a href="mailto:martin.sjolund@liu.se">martin.sjolund@liu.se</a>
-  - Dr. Henning Kiel
-
-  \subsection year_2011_subsection 2011
-  - Adeel Asghar - <a href="mailto:adeel.asghar@liu.se">adeel.asghar@liu.se</a>
-  - Martin Sjölund - <a href="mailto:martin.sjolund@liu.se">martin.sjolund@liu.se</a>
-  - Haris Kapidzic
-  - Abhinn Kothari
-
-  \subsection year_2010_subsection 2010
-  - Adeel Asghar - <a href="mailto:adeel.asghar@liu.se">adeel.asghar@liu.se</a>
-  - Sonia Tariq
-  */
+#include <locale.h>
 
 #include "MainWindow.h"
-#include "Helper.h"
-#include "CrashReportDialog.h"
+#include "Util/Helper.h"
+#include "CrashReport/CrashReportDialog.h"
+#include "Modeling/LibraryTreeWidget.h"
 #include "meta/meta_modelica.h"
 
 #ifndef WIN32
 #include "omc_config.h"
 #endif
 
+#include <QApplication>
+#include <QMessageBox>
+
 #ifdef QT_NO_DEBUG
 #ifndef WIN32
 #include <signal.h>
@@ -133,7 +121,7 @@ void signalHandler(int signum)
   }
   // Dump a stack trace to a file.
   QFile stackTraceFile;
-  QString& tmpPath = OpenModelica::tempDirectory();
+  QString& tmpPath = Utilities::tempDirectory();
   stackTraceFile.setFileName(QString("%1openmodelica.stacktrace.%2").arg(tmpPath).arg(Helper::OMCServerName));
   if (stackTraceFile.open(QIODevice::WriteOnly | QIODevice::Text))
   {
@@ -156,7 +144,7 @@ void signalHandler(int signum)
 
 #ifdef QT_NO_DEBUG
 #ifdef WIN32
-#include "backtrace.h"
+#include "CrashReport/backtrace.h"
 static char *g_output = NULL;
 LONG WINAPI exceptionFilter(LPEXCEPTION_POINTERS info)
 {
@@ -180,7 +168,7 @@ LONG WINAPI exceptionFilter(LPEXCEPTION_POINTERS info)
   }
   // Dump a stack trace to a file.
   QFile stackTraceFile;
-  stackTraceFile.setFileName(QString("%1/openmodelica.stacktrace.%2").arg(OpenModelica::tempDirectory()).arg(Helper::OMCServerName));
+  stackTraceFile.setFileName(QString("%1/openmodelica.stacktrace.%2").arg(Utilities::tempDirectory()).arg(Helper::OMCServerName));
   if (stackTraceFile.open(QIODevice::WriteOnly | QIODevice::Text))
   {
     QTextStream out(&stackTraceFile);
@@ -199,8 +187,7 @@ LONG WINAPI exceptionFilter(LPEXCEPTION_POINTERS info)
 
 void printOMEditUsage()
 {
-  printf("Usage: OMEdit [--OMCLogger=true|false] --Debug=true|false] [files]\n");
-  printf("    --OMCLogger=[true|false]    Allows sending OMC commands from OMCLogger. Default is false.\n");
+  printf("Usage: OMEdit --Debug=true|false] [files]\n");
   printf("    --Debug=[true|false]        Enables the debugging features like QUndoView, diffModelicaFileListings view. Default is false.\n");
   printf("    files                       List of Modelica files(*.mo) to open.\n");
 }
@@ -256,7 +243,7 @@ int main(int argc, char *argv[])
 #else /* unix */
   omhome = omhome ? omhome : CONFIG_DEFAULT_OPENMODELICAHOME;
 #endif
-  QSettings *pSettings = OpenModelica::getApplicationSettings();
+  QSettings *pSettings = Utilities::getApplicationSettings();
   QLocale settingsLocale = QLocale(pSettings->value("language").toString());
   settingsLocale = settingsLocale.name() == "C" ? pSettings->value("language").toLocale() : settingsLocale;
   QString locale = settingsLocale.name().isEmpty() ? QLocale::system().name() : settingsLocale.name();
@@ -278,28 +265,19 @@ int main(int argc, char *argv[])
   a.installTranslator(&translator);
   // Splash Screen
   QPixmap pixmap(":/Resources/icons/omedit_splashscreen.png");
-  QSplashScreen splashScreen(pixmap);
-  //splashScreen.setMessage();
-  splashScreen.show();
+  SplashScreen *pSplashScreen = SplashScreen::instance();
+  pSplashScreen->setPixmap(pixmap);
+  pSplashScreen->show();
   Helper::initHelperVariables();
   /* Force C-style doubles */
   setlocale(LC_NUMERIC, "C");
   // if user has requested to open the file by passing it in argument then,
-  bool OMCLogger = false;
   bool debug = false;
   QString fileName = "";
   QStringList fileNames;
   if (a.arguments().size() > 1) {
     for (int i = 1; i < a.arguments().size(); i++) {
-      if (strncmp(a.arguments().at(i).toStdString().c_str(), "--OMCLogger=",12) == 0) {
-        QString omcLoggerArg = a.arguments().at(i);
-        omcLoggerArg.remove("--OMCLogger=");
-        if (0 == strcmp("true", omcLoggerArg.toStdString().c_str())) {
-          OMCLogger = true;
-        } else {
-          OMCLogger = false;
-        }
-      } else if (strncmp(a.arguments().at(i).toStdString().c_str(), "--Debug=",8) == 0) {
+      if (strncmp(a.arguments().at(i).toStdString().c_str(), "--Debug=",8) == 0) {
         QString debugArg = a.arguments().at(i);
         debugArg.remove("--Debug=");
         if (0 == strcmp("true", debugArg.toStdString().c_str())) {
@@ -312,30 +290,34 @@ int main(int argc, char *argv[])
         if (!fileName.isEmpty()) {
           // if path is relative make it absolute
           QFileInfo file (fileName);
+          QString absoluteFileName = fileName;
           if (file.isRelative()) {
-            fileName.prepend(QString(QDir::currentPath()).append("/"));
+            absoluteFileName = QString("%1/%2").arg(QDir::currentPath()).arg(fileName);
+          }
+          absoluteFileName = absoluteFileName.replace("\\", "/");
+          if (QFile::exists(absoluteFileName)) {
+            fileNames << absoluteFileName;
+          } else {
+            printf("Invalid command line argument: %s %s\n", fileName.toStdString().c_str(), absoluteFileName.toStdString().c_str());
           }
-          fileName = fileName.replace("\\", "/");
-          fileNames << fileName;
         }
       }
     }
   }
   // MainWindow Initialization
-  MainWindow mainwindow(&splashScreen, debug);
-  if (mainwindow.getExitApplicationStatus()) {        // if there is some issue in running the application.
+  MainWindow *pMainwindow = MainWindow::instance(debug);
+  pMainwindow->setUpMainWindow();
+  if (pMainwindow->getExitApplicationStatus()) {        // if there is some issue in running the application.
     a.quit();
     exit(1);
   }
   // open the files passed as command line arguments
   foreach (QString fileName, fileNames) {
-    mainwindow.getLibraryWidget()->openFile(fileName);
+    pMainwindow->getLibraryWidget()->openFile(fileName);
   }
-  // hide OMCLogger send custom expression feature if OMCLogger is false
-  mainwindow.getOMCProxy()->enableCustomExpression(OMCLogger);
   // finally show the main window
-  mainwindow.show();
+  pMainwindow->show();
   // hide the splash screen
-  splashScreen.finish(&mainwindow);
+  pSplashScreen->finish(pMainwindow);
   return a.exec();
 }
