@@ -42,13 +42,15 @@
 #include "omcinteractiveenvironment.h"
 #include "updategroupcellvisitor.h"
 #include "commandcompletion.h"
-#include "highlighterthread.h"
 #include "stylesheet.h"
 #include "inputcell.h"
 #include "notebookcommands.h"
+#include <QSplashScreen>
 
 #include <cstdlib>
 
+#include "omc_config.h"
+
 namespace IAEX
 {
   /*!
@@ -84,10 +86,59 @@ namespace IAEX
 
     // ******************************************************
 
+    class MyApp : public QApplication {
+    private:
+      CellApplication * ca = NULL;
+      public:
+        MyApp(int& argc, char**argv, CellApplication * c): QApplication(argc, argv)
+        {ca = c;}
+        bool event(QEvent *event) {
+          switch(event->type())
+          {
+            case QEvent::FileOpen:
+            {
+              QFileOpenEvent * fileOpenEvent = static_cast<QFileOpenEvent *>(event);
+              if(fileOpenEvent) {
+                ca->FileOpenEventTriggered = true;
+                ca->open(fileOpenEvent->file());
+
+                return true;
+              }
+            }
+            default:
+              return QApplication::event(event);
+          }
+        }
+    };
+
   CellApplication::CellApplication( int &argc, char *argv[] )
     : QObject()
   {
-    app_ = new QApplication(argc, argv);
+    app_ = new MyApp(argc, argv, this);
+
+
+    const char *omhome = getenv("OPENMODELICAHOME");
+  #ifdef WIN32
+    if (!omhome) {
+      QMessageBox::critical(0, "OMNotebook", "OPENMODELICAHOME not set", "OK");
+      app_->quit();
+      exit(1);
+    }
+  #else /* unix */
+    omhome = omhome ? omhome : CONFIG_DEFAULT_OPENMODELICAHOME;
+  #endif
+    QString translationDirectory = omhome + QString("/share/omnotebook/nls");
+    // install Qt's default translations
+  #ifdef Q_OS_WIN
+    qtTranslator.load("qt_" + QLocale::system().name(), translationDirectory);
+  #else
+    qtTranslator.load("qt_" + QLocale::system().name(), QLibraryInfo::location(QLibraryInfo::TranslationsPath));
+  #endif
+    app_->installTranslator(&qtTranslator);
+    // install application translations
+    translator.load("OMNotebook_" + QLocale::system().name(), translationDirectory);
+    app_->installTranslator(&translator);
+
     mainWindow = new QMainWindow();
     QDir dir;
 
@@ -175,35 +226,6 @@ namespace IAEX
     }
     else
     {
-      // 2006-01-09 AF, create a new highlight thread with the
-      // 'openmodelicahighlighter' as the highlighter that should be
-      // used.
-      // 2006-04-10 AF, use environment variable to find modelicacolors.xml
-      try
-      {
-        Stylesheet *sheet = Stylesheet::instance( "stylesheet.xml" );
-        CellStyle style = sheet->getStyle( "Input" );
-        style.textCharFormat()->setBackground( QBrush( QColor( 200, 200, 255 ) ));
-
-        QString modelicacolorsfile;
-        if( openmodelica.endsWith("/") || openmodelica.endsWith( "\\") )
-          modelicacolorsfile = openmodelica + "share/omnotebook/modelicacolors.xml";
-        else
-          modelicacolorsfile = openmodelica + "/share/omnotebook/modelicacolors.xml";
-
-        OpenModelicaHighlighter *highlighter = new OpenModelicaHighlighter( modelicacolorsfile, *style.textCharFormat() );
-        HighlighterThread *thread = HighlighterThread::instance( highlighter );
-        //thread->start( QThread::LowPriority );
-      }
-      catch( exception &e )
-      {
-        QString msg = e.what();
-        msg += "\nCould not create highlighter thread, exiting OMNotebook";
-        QMessageBox::warning( 0, "Error", msg, "OK" );
-        std::exit(-1);
-      }
-
-
       // second arg is a file that should be opened.
       if( argc > 1 )
       {
@@ -220,32 +242,44 @@ namespace IAEX
       }
       else
       {
-        // 2006-02-27 AF, use environment variable to find DrModelica
-        // 2006-03-24 AF, First try to find DrModelica.onb, then .nb
-        QString drmodelica = OmcInteractiveEnvironment::OpenModelicaHome() + "/share/omnotebook/drmodelica/DrModelica.onb";
-        //QString drmodelica = OmcInteractiveEnvironment::OpenModelicaHome() + "/share/omnotebook/drmodelica/QuickTour/HelloWorld.onb";
+        QIcon icon(":/Resources/OMNotebook_icon.svg");
+        QSplashScreen splash(icon.pixmap(300,400));
+        splash.show();
+        app_->processEvents();
+        splash.finish(mainWindow);
+        if (FileOpenEventTriggered)
+        {
 
-        if( dir.exists( drmodelica ))
-          open(drmodelica);
-        else if( dir.exists( "DrModelica/DrModelica.onb" ))
-          open( "DrModelica/DrModelica.onb" );
+        }
         else
         {
-          cout << "Unable to find (1): " << drmodelica.toStdString() << endl;
-          cout << "Unable to find (2): DrModelica/DrModelica.onb" << endl;
-
-          // NB
-          drmodelica = OmcInteractiveEnvironment::OpenModelicaHome() + "/share/omnotebook/drmodelica/DrModelica.onb";
+          // 2006-02-27 AF, use environment variable to find DrModelica
+          // 2006-03-24 AF, First try to find DrModelica.onb, then .nb
+          QString drmodelica = OmcInteractiveEnvironment::OpenModelicaHome() + "/share/omnotebook/drmodelica/DrModelica.onb";
+          //QString drmodelica = OmcInteractiveEnvironment::OpenModelicaHome() + "/share/omnotebook/drmodelica/QuickTour/HelloWorld.onb";
 
           if( dir.exists( drmodelica ))
             open(drmodelica);
-          else if( dir.exists( "DrModelica/DrModelica.nb" ))
-            open( "DrModelica/DrModelica.nb" );
+          else if( dir.exists( "DrModelica/DrModelica.onb" ))
+            open( "DrModelica/DrModelica.onb" );
           else
           {
-            cout << "Unable to find (3): " << drmodelica.toStdString() << endl;
-            cout << "Unable to find (4): DrModelica/DrModelica.nb" << endl;
-            open(QString::null);
+            cout << "Unable to find (1): " << drmodelica.toStdString() << endl;
+            cout << "Unable to find (2): DrModelica/DrModelica.onb" << endl;
+
+            // NB
+            drmodelica = OmcInteractiveEnvironment::OpenModelicaHome() + "/share/omnotebook/drmodelica/DrModelica.onb";
+
+            if( dir.exists( drmodelica ))
+              open(drmodelica);
+            else if( dir.exists( "DrModelica/DrModelica.nb" ))
+              open( "DrModelica/DrModelica.nb" );
+            else
+            {
+              cout << "Unable to find (3): " << drmodelica.toStdString() << endl;
+              cout << "Unable to find (4): DrModelica/DrModelica.nb" << endl;
+              open(QString::null);
+            }
           }
         }
       }
@@ -270,10 +304,6 @@ namespace IAEX
    */
   CellApplication::~CellApplication()
   {
-    // 2005-12-19 AF, stop highlighter thread
-    HighlighterThread *thread = HighlighterThread::instance();
-    thread->exit();
-
     // 2006-02-09 AF, moved code for quiting omc to the notebook windos
 
     // 2006-01-16 AF, remove temporary files
@@ -389,10 +419,6 @@ namespace IAEX
     // 2005-12-01 AF, Added try-catch
     try
     {
-      //2006-05-03 AF, during open, stop highlighter
-      HighlighterThread *thread = HighlighterThread::instance();
-      thread->setStop( true );
-
       //1. Create a new document.
       Document *d = new CellDocument( this, filename, readmode );
       add(d);
@@ -417,16 +443,17 @@ namespace IAEX
 
       // 2006-01-31 AF, show window again
       v->show();
+      v->raise();  // for MacOS
+      v->activateWindow(); // for Windows
 
-      v->resize( 801, 600 ); //fjass
+      QDesktopWidget dw;
+      v->move(0, 0);
+      v->resize(dw.geometry().width(),dw.geometry().height());
 
       // 2005-11-30 AF, apply hide() and show() to closed groupcells
       // childs in the documentview
       UpdateGroupcellVisitor visitor;
       v->document()->runVisitor( visitor );
-
-      // 2006-05-03 AF, done, start highlighter again
-      thread->setStop( false );
     }
     catch( exception &e )
     {
