@@ -40,7 +40,6 @@
 // QT Headers
 #include <QtGui/QTextCursor>
 #include <QtGui/QTextDocumentFragment>
-#include <QDebug>
 
 //STD Headers
 #include <exception>
@@ -218,14 +217,14 @@ namespace IAEX
       }
       else
       {
-    document()->clearSelection(); //Notice
-    application()->clearPasteboard();
+        document()->clearSelection(); //Notice
+        application()->clearPasteboard();
 
-    vector<Cell *>::iterator i = cells.begin();
-    for(;i != cells.end();++i)
-    {
-      application()->addToPasteboard((*i));
-    }
+        vector<Cell *>::iterator i = cells.begin();
+        for(;i != cells.end();++i)
+        {
+          application()->addToPasteboard((*i));
+        }
       }
    }
 
@@ -249,7 +248,6 @@ namespace IAEX
    }
    else
    {
-
       document()->clearSelection(); //Notice
       application()->clearPasteboard();
 
@@ -288,7 +286,7 @@ namespace IAEX
   void PasteCellsCommand::execute()
   {
     try
-    {
+      {
       vector<Cell *> cells = application()->pasteboard();
 
       // Insert new cells before this position.
@@ -311,6 +309,10 @@ namespace IAEX
           }
         }
       }
+      else
+      {
+          QMessageBox::warning( 0, "Warning", "No Cells Copied to paste, To paste a cell, select the cells and perform copy", "OK" );
+      }
 
       //2006-01-18 AF, set docuement changed
       document()->setChanged( true );
@@ -425,27 +427,26 @@ namespace IAEX
 
       newGraphCell->setClosed( oldGraphCell->isClosed() );
     }
-
     else if( typeid(LatexCell) == typeid( *newCell ))
     {
       LatexCell *newLatexCell = dynamic_cast<LatexCell *>( newCell );
       LatexCell *oldLatexCell = dynamic_cast<LatexCell *>( cell );
 
       newLatexCell->setStyle( style );
-      newLatexCell->setText( oldLatexCell->text() );
-
+      newLatexCell->setTextHtml(oldLatexCell->textHtml());
       if( oldLatexCell->isEvaluated() )
       {
-        newLatexCell->setEvaluated( true );
+        newLatexCell->setEvaluated(true);
         //newLatexCell->setTextOutput( oldLatexCell->textOutput() );
         newLatexCell->setTextOutputHtml(oldLatexCell->textOutputHtml());
+
       }
       else
-        newLatexCell->setEvaluated( false );
-
+      {
+        newLatexCell->setEvaluated(false);
+      }
       newLatexCell->setClosed( oldLatexCell->isClosed() );
     }
-
     else if( typeid(TextCell) == typeid( *newCell ))
     {
       newCell->setStyle( style );
