@@ -153,7 +153,7 @@ namespace IAEX
     if(ba.indexOf("<InputCell") != -1)
     {
       /*
-      QSettings s("PELAB", "OMNotebook");
+      QSettings s(QSettings::IniFormat, QSettings::UserScope, "openmodelica", "omnotebook");
       bool alwaysConvert = s.value("AlwaysConvert", true).toBool();
       QMessageBox m;
       int i;
@@ -577,6 +577,8 @@ namespace IAEX
           text = e.text();
           GraphCell *gCell = dynamic_cast<GraphCell*>(graphcell);
           gCell->setText(text);
+          // we need to call rehighlight when we set the text manually.
+          gCell->mpModelicaTextHighlighter->rehighlight();
         }
         else if( e.tagName() == XML_OUTPUTPART )
         {
@@ -625,7 +627,7 @@ namespace IAEX
             QDomElement curveElement = n.toElement();
             if (curveElement.tagName() == XML_GRAPHCELL_CURVE)
             {
-              PlotCurve *pPlotCurve = new PlotCurve("", curveElement.attribute(XML_GRAPHCELL_TITLE), "", gCell->mpPlotWindow->getPlot());
+              PlotCurve *pPlotCurve = new PlotCurve("", curveElement.attribute(XML_GRAPHCELL_TITLE), "", "", gCell->mpPlotWindow->getPlot());
               // read the curve data
               if (curveElement.hasAttribute(XML_GRAPHCELL_XDATA) && curveElement.hasAttribute(XML_GRAPHCELL_YDATA))
               {
@@ -731,12 +733,12 @@ namespace IAEX
           {
             text = e.text();
             LatexCell *gCell = dynamic_cast<LatexCell*>(latexcell);
-            gCell->setText(text);
+            gCell->setTextHtml(text);
           }
           else if( e.tagName() == XML_OUTPUTPART )
           {
             LatexCell *iCell = dynamic_cast<LatexCell*>(latexcell);
-            iCell->setTextOutput( e.text() );
+            iCell->setTextOutput(e.text());
           }
           else if( e.tagName() == XML_IMAGE )
           {
@@ -763,7 +765,7 @@ namespace IAEX
 
       //    graphcell->setText( text ); //fjass
 
-      LatexCell *gCell = dynamic_cast<LatexCell*>(latexcell);
+      /* LatexCell *gCell = dynamic_cast<LatexCell*>(latexcell);
 
       QString closed = element.attribute( XML_CLOSED, XML_FALSE );
       if( closed == XML_TRUE )
@@ -771,11 +773,10 @@ namespace IAEX
       else if( closed == XML_FALSE )
         gCell->setClosed( false,true );
       else
-        throw runtime_error( "Unknown closed value in latexcell" );
+        throw runtime_error( "Unknown closed value in latexcell" ); */
 
       parent->addChild(latexcell);
 
-
   }
 
 
@@ -845,12 +846,17 @@ namespace IAEX
       else if( typeid(LatexCell) == typeid(*parent) )
       {
         LatexCell *latexcell = dynamic_cast<LatexCell*>(parent);
-
+        QString html = latexcell->textHtml();
+        html.replace(imagename,newname);
+        latexcell->setTextHtml(html);
+        /*
         QString html = latexcell->textOutputHtml();
         html.replace( imagename, newname );
 
         latexcell->setTextOutputHtml( html );
         latexcell->output_->textCursor().insertImage(newname);
+        latexcell->output_->show();
+        latexcell->latexButton->show(); */
       }
 
       else
