@@ -78,6 +78,8 @@ private:
   PlotType mPlotType;
   QString mGridType;
   QString mUnit;
+  QString mDisplayUnit;
+  QString mTimeUnit;
   QString mXRangeMin;
   QString mXRangeMax;
   QString mYRangeMin;
@@ -107,6 +109,10 @@ public:
   void setYLabel(QString label);
   void setUnit(QString unit) {mUnit = unit;}
   QString getUnit() {return mUnit;}
+  void setDisplayUnit(QString displayUnit) {mDisplayUnit = displayUnit;}
+  QString getDisplayUnit() {return mDisplayUnit;}
+  void setTimeUnit(QString timeUnit) {mTimeUnit = timeUnit;}
+  QString getTimeUnit() {return mTimeUnit;}
   void setXRange(double min, double max);
   QString getXRangeMin();
   QString getXRangeMax();
